@@ -6,7 +6,7 @@
 #						 All Rights Reserved.
 #
 
-RCS_ID =  '$Id: http_server.py,v 1.30 2001/09/20 13:29:28 andreasjung Exp $'
+RCS_ID =  '$Id: http_server.py,v 1.33.10.1 2003/02/24 09:37:24 andreasjung Exp $'
 
 # python modules
 import os
@@ -28,7 +28,10 @@ import producers
 import status_handler
 import logger
 
-VERSION_STRING = string.split(RCS_ID)[2]
+if RCS_ID.startswith('$Id: '):
+    VERSION_STRING = string.split(RCS_ID)[2]
+else:
+    VERSION_STRING = '0.0'
 
 from counter import counter
 from urllib import unquote
@@ -286,7 +289,7 @@ class http_request:
 
         self.channel.server.logger.log (
             self.channel.addr[0],
-            ' - %s [%s] "%s" %d %d "%s" "%s"\n' % (
+            '- %s [%s] "%s" %d %d "%s" "%s"\n' % (
                 name,
                 self.log_date_string (time.time()),
                 self.request,
@@ -575,7 +578,10 @@ class http_server (asyncore.dispatcher):
         host, port = self.socket.getsockname()
         if not ip:
             self.log_info('Computing default hostname', 'warning')
-            ip = socket.gethostbyname (socket.gethostname())
+            try:
+                ip = socket.gethostbyname(socket.gethostname())
+            except socket.error:
+                ip = socket.gethostbyname('localhost')
         try:
             self.server_name = socket.gethostbyaddr (ip)[0]
         except socket.error:
@@ -624,22 +630,28 @@ class http_server (asyncore.dispatcher):
     def handle_accept (self):
         self.total_clients.increment()
         try:
-            conn, addr = self.accept()
+            tup = self.accept()
         except socket.error:
                 # linux: on rare occasions we get a bogus socket back from
                 # accept.  socketmodule.c:makesockaddr complains that the
                 # address family is unknown.  We don't want the whole server
                 # to shut down because of this.
-            self.log_info ('warning: server accept() threw an exception', 'warning')
+            self.log_info ('warning: server accept() threw an exception',
+                           'warning')
+            self.total_clients.decrement()
             return
+        try:
+            conn, addr = tup
         except TypeError:
-                # unpack non-sequence.  this can happen when a read event
-                # fires on a listening socket, but when we call accept()
-                # we get EWOULDBLOCK, so dispatcher.accept() returns None.
-                # Seen on FreeBSD3.
-            self.log_info ('warning: server accept() threw EWOULDBLOCK', 'warning')
+            # unpack non-sequence.  this can happen when a read event
+            # fires on a listening socket, but when we call accept()
+            # we get EWOULDBLOCK, so dispatcher.accept() returns None.
+            # Seen on FreeBSD3 and Linux.
+            #self.log_info ('warning: server accept() returned %s '
+            #               '(EWOULDBLOCK?)' % tup, 'warning')
+            self.total_clients.decrement()
             return
-            
+
         self.channel_class (self, conn, addr)
         
     def install_handler (self, handler, back=0):
