@@ -1,14 +1,14 @@
 ##############################################################################
 #
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
-# 
+#
 ##############################################################################
 
 import compileall, os, sys
@@ -35,7 +35,7 @@ def compile_non_test(dir):
         fullname = os.path.join(dir, name)
         if (name != os.curdir and name != os.pardir and
             os.path.isdir(fullname) and not os.path.islink(fullname) and
-            name != 'test' and name != 'tests'):
+            name != 'test' and name != 'tests' and name != 'skins'):
             success = success and compile_non_test(fullname)
     return success
 
@@ -58,11 +58,11 @@ except:
     success = 0
     import traceback
     traceback.print_exc()
-    
+
 if not success:
     print
     print '!' * 78
     print 'There were errors during Python module compilation.'
     print '!' * 78
     print
-    sys.exit(1)
+    #sys.exit(1)
