@@ -13,7 +13,7 @@
 
 static char intSet_module_documentation[] = 
 ""
-"\n$Id: intSet.c,v 1.21 2001/11/28 15:50:50 matt Exp $"
+"\n$Id: intSet.c,v 1.23.32.1 2003/01/06 16:58:34 chrisw Exp $"
 ;
 
 #include <limits.h>
@@ -368,9 +368,9 @@ intSet__p___reinit__(intSet *self, PyObject *args)
      && HasInstDict(self) && (dict=INSTANCE_DICT(self)))
     {
       PyDict_Clear(dict);
-      self->state=cPersistent_GHOST_STATE;
     }
 
+  PER_GHOSTIFY(self);
   Py_INCREF(Py_None);
   return Py_None;
 }
@@ -524,7 +524,6 @@ void
 initintSet(void)
 {
   PyObject *m, *d;
-  char *rev="$Revision: 1.21 $";
 
   UNLESS(ExtensionClassImported) return;
 
@@ -546,10 +545,6 @@ initintSet(void)
 
   PyExtensionClass_Export(d,"intSet",intSetType);
 
-  PyDict_SetItemString(d, "__version__",
-		       PyString_FromStringAndSize(rev+11,strlen(rev+11)-2));
-  
-
   /* Check for errors */
   if (PyErr_Occurred())
     Py_FatalError("can't initialize module intSet");
