@@ -1,101 +1,37 @@
 ##############################################################################
-# 
-# Zope Public License (ZPL) Version 1.0
-# -------------------------------------
-# 
-# Copyright (c) Digital Creations.  All rights reserved.
-# 
-# This license has been certified as Open Source(tm).
-# 
-# Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions are
-# met:
-# 
-# 1. Redistributions in source code must retain the above copyright
-#    notice, this list of conditions, and the following disclaimer.
-# 
-# 2. Redistributions in binary form must reproduce the above copyright
-#    notice, this list of conditions, and the following disclaimer in
-#    the documentation and/or other materials provided with the
-#    distribution.
-# 
-# 3. Digital Creations requests that attribution be given to Zope
-#    in any manner possible. Zope includes a "Powered by Zope"
-#    button that is installed by default. While it is not a license
-#    violation to remove this button, it is requested that the
-#    attribution remain. A significant investment has been put
-#    into Zope, and this effort will continue if the Zope community
-#    continues to grow. This is one way to assure that growth.
-# 
-# 4. All advertising materials and documentation mentioning
-#    features derived from or use of this software must display
-#    the following acknowledgement:
-# 
-#      "This product includes software developed by Digital Creations
-#      for use in the Z Object Publishing Environment
-#      (http://www.zope.org/)."
-# 
-#    In the event that the product being advertised includes an
-#    intact Zope distribution (with copyright and license included)
-#    then this clause is waived.
-# 
-# 5. Names associated with Zope or Digital Creations must not be used to
-#    endorse or promote products derived from this software without
-#    prior written permission from Digital Creations.
-# 
-# 6. Modified redistributions of any form whatsoever must retain
-#    the following acknowledgment:
-# 
-#      "This product includes software developed by Digital Creations
-#      for use in the Z Object Publishing Environment
-#      (http://www.zope.org/)."
-# 
-#    Intact (re-)distributions of any official Zope release do not
-#    require an external acknowledgement.
-# 
-# 7. Modifications are encouraged but must be packaged separately as
-#    patches to official Zope releases.  Distributions that do not
-#    clearly separate the patches from the original work must be clearly
-#    labeled as unofficial distributions.  Modifications which do not
-#    carry the name Zope may be packaged in any form, as long as they
-#    conform to all of the clauses above.
-# 
-# 
-# Disclaimer
-# 
-#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
-#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
-#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
-#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
-#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
-#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
-#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
-#   SUCH DAMAGE.
-# 
-# 
-# This software consists of contributions made by Digital Creations and
-# many individuals on behalf of Digital Creations.  Specific
-# attributions are listed in the accompanying credits file.
-# 
+#
+# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+#
+# This software is subject to the provisions of the Zope Public License,
+# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
+# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
+# FOR A PARTICULAR PURPOSE
+#
 ##############################################################################
 """Access control support"""
 
-__version__='$Revision: 1.36 $'[11:-2]
+__version__='$Revision: 1.55.6.2 $'[11:-2]
 
 
-from Globals import HTMLFile, MessageDialog, Dictionary
-from string import join, strip, split, find
-from Acquisition import Implicit, Acquired
+from Globals import DTMLFile, MessageDialog, Dictionary
+from Acquisition import Implicit, Acquired, aq_get
 import Globals, ExtensionClass, PermissionMapping, Products
 from Permission import Permission
 from App.Common import aq_base
+from cgi import escape
 
 ListType=type([])
 
+DEFAULTMAXLISTUSERS=250
+
+def _isBeingUsedAsAMethod(self):
+    return aq_get(self, '_isBeingUsedAsAMethod_', 0)
+
+def _isNotBeingUsedAsAMethod(self):
+    return not aq_get(self, '_isBeingUsedAsAMethod_', 0)
+
 class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
     """An obect that has configurable permissions"""
 
@@ -111,12 +47,22 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
           'manage_defined_roles', 'userdefined_roles',
           'manage_listLocalRoles', 'manage_editLocalRoles',
           'manage_setLocalRoles', 'manage_addLocalRoles',
-          'manage_delLocalRoles',
+          'manage_delLocalRoles'
           )),
-#        ('View management screens', ('manage_access',)),
         )
-   
-    __ac_roles__=('Manager', 'Owner', 'Anonymous')
+
+    manage_options=(
+        {'label':'Security', 'action':'manage_access',
+         'help':('OFSP','Security.stx'),
+         'filter': _isNotBeingUsedAsAMethod,
+         },
+        {'label':'Define Permissions', 'action':'manage_access',
+         'help':('OFSP','Security_Define-Permissions.stx'),
+         'filter': _isBeingUsedAsAMethod,
+         },
+        )
+
+    __ac_roles__=('Manager', 'Owner', 'Anonymous', 'Authenticated')
 
     permissionMappingPossibleValues=Acquired
 
@@ -129,19 +75,19 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
         d={}
         perms=self.__ac_permissions__
         for p in perms: d[p[0]]=None
-            
+
         r=gather_permissions(self.__class__, [], d)
         if all:
-           if hasattr(self, '_subobject_permissions'):
-               for p in self._subobject_permissions():
-                   pname=p[0]
-                   if not d.has_key(pname):
-                       d[pname]=1
-                       r.append(p)
-            
-           r=list(perms)+r
-           r.sort()
-            
+            if hasattr(self, '_subobject_permissions'):
+                for p in self._subobject_permissions():
+                    pname=p[0]
+                    if not d.has_key(pname):
+                        d[pname]=1
+                        r.append(p)
+
+            r=list(perms)+r
+            r.sort()
+
         return tuple(r)
 
     def permission_settings(self):
@@ -154,7 +100,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
         for p in self.ac_inherited_permissions(1):
             name, value = p[:2]
             p=Permission(name,value,self)
-            roles=p.getRoles()
+            roles=p.getRoles(default=[])
             d={'name': name,
                'acquire': type(roles) is ListType and 'CHECKED' or '',
                'roles': map(
@@ -167,10 +113,13 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
                }
             ip=ip+1
             result.append(d)
-
         return result
 
-    manage_roleForm=HTMLFile('roleEdit', globals())
+    manage_roleForm=DTMLFile('dtml/roleEdit', globals(),
+                             management_view='Security',
+                             help_topic='Security_Manage-Role.stx',
+                             help_product='OFSP')
+
     def manage_role(self, role_to_manage, permissions=[], REQUEST=None):
         "Change the permissions given to the given role"
         self._isBeingUsedAsAMethod(REQUEST, 0)
@@ -179,9 +128,13 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
             p=Permission(name,value,self)
             p.setRole(role_to_manage, name in permissions)
 
-        if REQUEST is not None: return self.manage_access(self,REQUEST)
+        if REQUEST is not None: return self.manage_access(REQUEST)
+
+    manage_acquiredForm=DTMLFile('dtml/acquiredEdit', globals(),
+                                 management_view='Security',
+                                 help_topic='Security_Manage-Acquisition.stx',
+                                 help_product='OFSP')
 
-    manage_acquiredForm=HTMLFile('acquiredEdit', globals())
     def manage_acquiredPermissions(self, permissions=[], REQUEST=None):
         "Change the permissions that acquire"
         self._isBeingUsedAsAMethod(REQUEST, 0)
@@ -193,10 +146,13 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
             if name in permissions: p.setRoles(list(roles))
             else:                   p.setRoles(tuple(roles))
 
-        if REQUEST is not None: return self.manage_access(self,REQUEST)
-        
+        if REQUEST is not None: return self.manage_access(REQUEST)
+
+    manage_permissionForm=DTMLFile('dtml/permissionEdit', globals(),
+                                   management_view='Security',
+                                   help_topic='Security_Manage-Permission.stx',
+                                   help_product='OFSP')
 
-    manage_permissionForm=HTMLFile('permissionEdit', globals())
     def manage_permission(self, permission_to_manage,
                           roles=[], acquire=0, REQUEST=None):
         """Change the settings for the given permission
@@ -212,24 +168,25 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
                 if acquire: roles=list(roles)
                 else: roles=tuple(roles)
                 p.setRoles(roles)
-                if REQUEST is not None: return self.manage_access(self,REQUEST)
+                if REQUEST is not None: return self.manage_access(REQUEST)
                 return
 
         raise 'Invalid Permission', (
-            "The permission <em>%s</em> is invalid." % permission_to_manage)
-        
-    
-    def manage_access(
-        trueself, self, REQUEST,
-        _normal_manage_access=HTMLFile('access', globals()),
-        _method_manage_access=HTMLFile('methodAccess', globals()),
-        **kw):
+            "The permission <em>%s</em> is invalid." %
+                escape(permission_to_manage))
+
+    _normal_manage_access=DTMLFile('dtml/access', globals())
+
+    _method_manage_access=DTMLFile('dtml/methodAccess', globals())
+
+    def manage_access(self, REQUEST, **kw):
         "Return an interface for making permissions settings"
-        if self._isBeingUsedAsAMethod():
-            return apply(_method_manage_access,(trueself, self, REQUEST), kw)
+        if hasattr(self, '_isBeingUsedAsAMethod') and \
+           self._isBeingUsedAsAMethod():
+            return apply(self._method_manage_access,(), kw)
         else:
-            return apply(_normal_manage_access,(trueself, self, REQUEST), kw)
-    
+            return apply(self._normal_manage_access,(), kw)
+
     def manage_changePermissions(self, REQUEST):
         "Change all permissions settings, called by management screen"
         self._isBeingUsedAsAMethod(REQUEST, 0)
@@ -237,15 +194,24 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
         indexes=range(len(valid_roles))
         have=REQUEST.has_key
         permissions=self.ac_inherited_permissions(1)
+        fails = []
         for ip in range(len(permissions)):
             roles=[]
             for ir in indexes:
                 if have("p%dr%d" % (ip,ir)): roles.append(valid_roles[ir])
             name, value = permissions[ip][:2]
-            p=Permission(name,value,self)
-            if not have('a%d' % ip): roles=tuple(roles)
-            p.setRoles(roles)
+            try:
+                p=Permission(name,value,self)
+                if not have('a%d' % ip): roles=tuple(roles)
+                p.setRoles(roles)
+            except:
+                fails.append(name)
 
+        if fails:
+            return MessageDialog(title="Warning!",
+                                 message="Some permissions had errors: "
+                                   + escape(', '.join(fails)),
+                                 action='manage_access')
         return MessageDialog(
             title  ='Success!',
             message='Your changes have been saved',
@@ -278,9 +244,9 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
                      'selected': role in roles and 'SELECTED' or '',
                      },
                     valid_roles)
-        
+
         raise 'Invalid Permission', (
-            "The permission <em>%s</em> is invalid." % permission)
+            "The permission <em>%s</em> is invalid." % escape(permission))
 
     def acquiredRolesAreUsedBy(self, permission):
         "used by management screen"
@@ -290,9 +256,9 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
                 p=Permission(name,value,self)
                 roles=p.getRoles()
                 return type(roles) is ListType and 'CHECKED' or ''
-        
+
         raise 'Invalid Permission', (
-            "The permission <em>%s</em> is invalid." % permission)
+            "The permission <em>%s</em> is invalid." % escape(permission))
 
 
     # Local roles support
@@ -302,11 +268,18 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
     # of a particular object (and its children). When a user is given
     # extra roles in a particular object, an entry for that user is made
     # in the __ac_local_roles__ dict containing the extra roles.
-    
+
     __ac_local_roles__=None
 
-    manage_listLocalRoles=HTMLFile('listLocalRoles', globals())
-    manage_editLocalRoles=HTMLFile('editLocalRoles', globals())
+    manage_listLocalRoles=DTMLFile('dtml/listLocalRoles', globals(),
+                                   management_view='Security',
+                                   help_topic='Security_Local-Roles.stx',
+                                   help_product='OFSP')
+
+    manage_editLocalRoles=DTMLFile('dtml/editLocalRoles', globals(),
+                                   management_view='Security',
+                                   help_topic='Security_User-Local-Roles.stx',
+                                   help_product='OFSP')
 
     def has_local_roles(self):
         dict=self.__ac_local_roles__ or {}
@@ -318,34 +291,52 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
         keys.sort()
         info=[]
         for key in keys:
-            info.append((key, dict[key]))
-        return info
+            value=tuple(dict[key])
+            info.append((key, value))
+        return tuple(info)
+
+    def users_with_local_role(self, role):
+        got = {}
+        for user, roles in self.get_local_roles():
+            if role in roles:
+                got[user] = 1
+        return got.keys()
 
     def get_valid_userids(self):
         item=self
         dict={}
+        _notfound = []
         while 1:
-            if hasattr(aq_base(item), 'acl_users') and \
-               hasattr(item.acl_users, 'user_names'):
-                for name in item.acl_users.user_names():
-                    dict[name]=1
-            if not hasattr(item, 'aq_parent'):
+            aclu = getattr(aq_base(item), '__allow_groups__', _notfound)
+            if aclu is not _notfound:
+                mlu = getattr(aclu, 'maxlistusers', _notfound)
+                if type(mlu) != type(1): mlu = DEFAULTMAXLISTUSERS
+                if mlu < 0: raise OverflowError
+                un = getattr(aclu, 'user_names', _notfound)
+                if un is not _notfound:
+                    unl = un()
+                    # maxlistusers of 0 is list all
+                    if len(unl) > mlu and mlu != 0:
+                        raise OverflowError
+                    for name in un():
+                        dict[name]=1
+            item = getattr(item, 'aq_parent', _notfound)
+            if item is _notfound:
                 break
-            item=item.aq_parent
         keys=dict.keys()
         keys.sort()
-        return keys
+        return tuple(keys)
 
     def get_local_roles_for_userid(self, userid):
         dict=self.__ac_local_roles__ or {}
-        return dict.get(userid, [])
+        return tuple(dict.get(userid, []))
 
     def manage_addLocalRoles(self, userid, roles, REQUEST=None):
         """Set local roles for a user."""
         if not roles:
             raise ValueError, 'One or more roles must be given!'
         dict=self.__ac_local_roles__ or {}
-        local_roles = dict.get(userid, [])
+        local_roles = list(dict.get(userid, []))
         for r in roles:
             if r not in local_roles:
                 local_roles.append(r)
@@ -382,6 +373,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
 
     #------------------------------------------------------------
 
+    access_debug_info__roles__=()
     def access_debug_info(self):
         "Return debug info"
         clas=class_attrs(self)
@@ -389,16 +381,16 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
         data=[]
         _add=data.append
         for key, value in inst.items():
-            if find(key,'__roles__') >= 0:
+            if key.find('__roles__') >= 0:
                 _add({'name': key, 'value': value, 'class': 0})
             if hasattr(value, '__roles__'):
-                _add({'name': '%s.__roles__' % key, 'value': value.__roles__, 
+                _add({'name': '%s.__roles__' % key, 'value': value.__roles__,
                       'class': 0})
         for key, value in clas.items():
-            if find(key,'__roles__') >= 0:
+            if key.find('__roles__') >= 0:
                 _add({'name': key, 'value': value, 'class' : 1})
             if hasattr(value, '__roles__'):
-                _add({'name': '%s.__roles__' % key, 'value': value.__roles__, 
+                _add({'name': '%s.__roles__' % key, 'value': value.__roles__,
                       'class': 1})
         return data
 
@@ -420,7 +412,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
             x=x+1
         roles=dict.keys()
         roles.sort()
-        return roles
+        return tuple(roles)
 
     def validate_roles(self, roles):
         "Return true if all given roles are valid"
@@ -436,7 +428,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
         for role in classattr(self.__class__,'__ac_roles__'):
             try:    roles.remove(role)
             except: pass
-        return roles
+        return tuple(roles)
 
 
     def manage_defined_roles(self,submit=None,REQUEST=None):
@@ -450,24 +442,24 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
             roles=reqattr(REQUEST, 'roles')
             return self._delRoles(roles, REQUEST)
 
-        return self.manage_access(self,REQUEST)
+        return self.manage_access(REQUEST)
 
     def _addRole(self, role, REQUEST=None):
         if not role:
             return MessageDialog(
                    title  ='Incomplete',
                    message='You must specify a role name',
-                   action ='manage_changeAccess')
+                   action ='manage_access')
         if role in self.__ac_roles__:
             return MessageDialog(
                    title  ='Role Exists',
                    message='The given role is already defined',
-                   action ='manage_changeAccess')
+                   action ='manage_access')
         data=list(self.__ac_roles__)
         data.append(role)
         self.__ac_roles__=tuple(data)
         if REQUEST is not None:
-            return self.manage_access(self, REQUEST)
+            return self.manage_access(REQUEST)
 
 
     def _delRoles(self, roles, REQUEST):
@@ -475,19 +467,19 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
             return MessageDialog(
                    title  ='Incomplete',
                    message='You must specify a role name',
-                   action ='manage_changeAccess')
+                   action ='manage_access')
         data=list(self.__ac_roles__)
         for role in roles:
             try:    data.remove(role)
             except: pass
         self.__ac_roles__=tuple(data)
         if REQUEST is not None:
-            return self.manage_access(self, REQUEST)
+            return self.manage_access(REQUEST)
 
 
     def _has_user_defined_role(self, role):
         return role in self.__ac_roles__
-        
+
 
     # Compatibility names only!!
 
@@ -513,7 +505,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
 
         d=d.keys()
         d.sort()
-        
+
         return d
 
 
