@@ -1,18 +1,18 @@
 ##############################################################################
 #
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
-# 
+#
 ##############################################################################
 """Access control package"""
 
-__version__='$Revision: 1.166.10.2 $'[11:-2]
+__version__='$Revision: 1.170.6.7 $'[11:-2]
 
 import Globals, socket, SpecialUsers,re
 import os
@@ -32,7 +32,6 @@ from AccessControl.SecurityManagement import noSecurityManager
 from AccessControl.ZopeSecurityPolicy import _noroles
 
 ListType=type([])
-NotImplemented='NotImplemented'
 
 _marker=[]
 
@@ -42,7 +41,7 @@ class BasicUser(Implicit):
     # ----------------------------
     # Public User object interface
     # ----------------------------
-    
+
     # Maybe allow access to unprotected attributes. Note that this is
     # temporary to avoid exposing information but without breaking
     # everyone's current code. In the future the security will be
@@ -59,33 +58,33 @@ class BasicUser(Implicit):
         if name in deny_names:
             return 0
         return 1
-        
+
     def __init__(self,name,password,roles,domains):
-        raise NotImplemented
+        raise NotImplementedError
 
     def getUserName(self):
         """Return the username of a user"""
-        raise NotImplemented
+        raise NotImplementedError
 
     def getId(self):
         """Get the ID of the user. The ID can be used, at least from
         Python, to get the user from the user's
         UserDatabase"""
-        return self.getUserName()        
+        return self.getUserName()
 
     def _getPassword(self):
         """Return the password of the user."""
-        raise NotImplemented
+        raise NotImplementedError
 
     def getRoles(self):
         """Return the list of roles assigned to a user."""
-        raise NotImplemented
+        raise NotImplementedError
 
     def getRolesInContext(self, object):
         """Return the list of roles assigned to the user,
            including local roles assigned in context of
            the passed in object."""
-        name=self.getUserName()
+        userid=self.getId()
         roles=self.getRoles()
         local={}
         object=getattr(object, 'aq_inner', object)
@@ -95,7 +94,7 @@ class BasicUser(Implicit):
                 if callable(local_roles):
                     local_roles=local_roles()
                 dict=local_roles or {}
-                for r in dict.get(name, []):
+                for r in dict.get(userid, []):
                     local[r]=1
             inner = getattr(object, 'aq_inner', object)
             parent = getattr(inner, 'aq_parent', None)
@@ -112,12 +111,12 @@ class BasicUser(Implicit):
 
     def getDomains(self):
         """Return the list of domain restrictions for a user"""
-        raise NotImplemented
+        raise NotImplementedError
 
     # ------------------------------
     # Internal User object interface
     # ------------------------------
-    
+
     def authenticate(self, password, request):
         passwrd=self._getPassword()
         result = AuthEncoding.pw_validate(passwrd, password)
@@ -126,25 +125,25 @@ class BasicUser(Implicit):
             return result and domainSpecMatch(domains, request)
         return result
 
-    
+
     def _shared_roles(self, parent):
-          r=[]
-          while 1:
-              if hasattr(parent,'__roles__'):
-                  roles=parent.__roles__
-                  if roles is None: return 'Anonymous',
-                  if 'Shared' in roles:
-                      roles=list(roles)
-                      roles.remove('Shared')
-                      r=r+roles
-                  else:
-                      try: return r+list(roles)
-                      except: return r
-              if hasattr(parent, 'aq_parent'):
-                  while hasattr(parent.aq_self,'aq_self'):
-                      parent=parent.aq_self
-                  parent=parent.aq_parent
-              else: return r
+        r=[]
+        while 1:
+            if hasattr(parent,'__roles__'):
+                roles=parent.__roles__
+                if roles is None: return 'Anonymous',
+                if 'Shared' in roles:
+                    roles=list(roles)
+                    roles.remove('Shared')
+                    r=r+roles
+                else:
+                    try: return r+list(roles)
+                    except: return r
+            if hasattr(parent, 'aq_parent'):
+                while hasattr(parent.aq_self,'aq_self'):
+                    parent=parent.aq_self
+                parent=parent.aq_parent
+            else: return r
 
     def _check_context(self, object):
         # Check that 'object' exists in the acquisition context of
@@ -207,14 +206,14 @@ class BasicUser(Implicit):
         # this manually rather than call getRolesInContext so that
         # we can incur only the overhead required to find a match.
         inner_obj = getattr(object, 'aq_inner', object)
-        user_name = self.getUserName()
+        userid = self.getId()
         while 1:
             local_roles = getattr(inner_obj, '__ac_local_roles__', None)
             if local_roles:
                 if callable(local_roles):
                     local_roles = local_roles()
                 dict = local_roles or {}
-                local_roles = dict.get(user_name, [])
+                local_roles = dict.get(userid, [])
                 for role in object_roles:
                     if role in local_roles:
                         if self._check_context(object):
@@ -234,7 +233,7 @@ class BasicUser(Implicit):
 
     def hasRole(self, *args, **kw):
         """hasRole is an alias for 'allowed' and has been deprecated.
-        
+
         Code still using this method should convert to either 'has_role' or
         'allowed', depending on the intended behaviour.
 
@@ -243,10 +242,10 @@ class BasicUser(Implicit):
         warnings.warn('BasicUser.hasRole is deprecated, please use '
             'BasicUser.allowed instead; hasRole was an alias for allowed, but '
             'you may have ment to use has_role.', DeprecationWarning)
-        self.allowed(*args, **kw)
+        return self.allowed(*args, **kw)
 
     domains=[]
-    
+
     def has_role(self, roles, object=None):
         """Check to see if a user has a given role or roles."""
         if type(roles)==type('s'):
@@ -263,7 +262,10 @@ class BasicUser(Implicit):
 
     def has_permission(self, permission, object):
         """Check to see if a user has a given permission on an object."""
-        return getSecurityManager().checkPermission(permission, object)
+        roles=rolesForPermissionOn(permission, object)
+        if type(roles) is type(''):
+            roles=[roles]
+        return self.allowed(object, roles)
 
     def __len__(self): return 1
     def __str__(self): return self.getUserName()
@@ -314,7 +316,7 @@ class UnrestrictedUser(SpecialUser):
 
     def hasRole(self, *args, **kw):
         """hasRole is an alias for 'allowed' and has been deprecated.
-        
+
         Code still using this method should convert to either 'has_role' or
         'allowed', depending on the intended behaviour.
 
@@ -366,7 +368,7 @@ class NullUnrestrictedUser(SpecialUser):
 
     def hasRole(self, *args, **kw):
         """hasRole is an alias for 'allowed' and has been deprecated.
-        
+
         Code still using this method should convert to either 'has_role' or
         'allowed', depending on the intended behaviour.
 
@@ -384,7 +386,7 @@ class NullUnrestrictedUser(SpecialUser):
     def has_permission(self, permission, object):
         return 0
 
-    
+
 
 def readUserAccessFile(filename):
     '''Reads an access file from INSTANCE_HOME.
@@ -471,18 +473,18 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
     # ----------------------------------
     # Public UserFolder object interface
     # ----------------------------------
-    
+
     def getUserNames(self):
         """Return a list of usernames"""
-        raise NotImplemented
+        raise NotImplementedError
 
     def getUsers(self):
         """Return a list of user objects"""
-        raise NotImplemented
+        raise NotImplementedError
 
     def getUser(self, name):
         """Return the named user object or None"""
-        raise NotImplemented
+        raise NotImplementedError
 
     def getUserById(self, id, default=_marker):
         """Return the user corresponding to the given id.
@@ -492,27 +494,27 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
         try:
             return self.getUser(id)
         except:
-           if default is _marker: raise
-           return default
+            if default is _marker: raise
+            return default
 
     def _doAddUser(self, name, password, roles, domains, **kw):
         """Create a new user. This should be implemented by subclasses to
            do the actual adding of a user. The 'password' will be the
            original input password, unencrypted. The implementation of this
            method is responsible for performing any needed encryption."""
-        raise NotImplemented
+        raise NotImplementedError
 
     def _doChangeUser(self, name, password, roles, domains, **kw):
-        """Modify an existing user. This should be implemented by subclasses 
+        """Modify an existing user. This should be implemented by subclasses
            to make the actual changes to a user. The 'password' will be the
            original input password, unencrypted. The implementation of this
            method is responsible for performing any needed encryption."""
-        raise NotImplemented
+        raise NotImplementedError
 
     def _doDelUsers(self, names):
         """Delete one or more users. This should be implemented by subclasses
            to do the actual deleting of users."""
-        raise NotImplemented
+        raise NotImplementedError
 
     # As of Zope 2.5, userFolderAddUser, userFolderEditUser and
     # userFolderDelUsers offer aliases for the the _doAddUser, _doChangeUser
@@ -529,7 +531,7 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
            objects."""
         if hasattr(self, '_doAddUser'):
             return self._doAddUser(name, password, roles, domains, **kw)
-        raise NotImplemented
+        raise NotImplementedError
 
     def userFolderEditUser(self, name, password, roles, domains, **kw):
         """API method for changing user object attributes. Note that not
@@ -537,14 +539,14 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
            attributes."""
         if hasattr(self, '_doChangeUser'):
             return self._doChangeUser(name, password, roles, domains, **kw)
-        raise NotImplemented
+        raise NotImplementedError
 
     def userFolderDelUsers(self, names):
         """API method for deleting one or more user objects. Note that not
            all user folder implementations support deletion of user objects."""
         if hasattr(self, '_doDelUsers'):
             return self._doDelUsers(names)
-        raise NotImplemented
+        raise NotImplementedError
 
 
     # -----------------------------------
@@ -588,8 +590,17 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
         security = getSecurityManager()
         try:
             try:
-                if security.validate(accessed, container, name, value, roles):
-                    return 1
+                # This is evil: we cannot pass _noroles directly because
+                # it is a special marker, and that special marker is not
+                # the same between the C and Python policy implementations.
+                # We __really__ need to stop using this marker pattern!
+                if roles is _noroles:
+                    if security.validate(accessed, container, name, value):
+                        return 1
+                else:
+                    if security.validate(accessed, container, name, value,
+                                         roles):
+                        return 1
             except:
                 noSecurityManager()
                 raise
@@ -658,9 +669,9 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
                 # we can't authorize the user, and we either can't authorize
                 # nobody against the published object or we're not top-level
                 return None
-            
+
     if _remote_user_mode:
-        
+
         def validate(self, request, auth='', roles=_noroles):
             v = request['PUBLISHED']
             a, c, n, v = self._getobcontext(v, request)
@@ -720,7 +731,7 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
         c is the physical container of the object
         """
         if len(request.steps) == 0: # someone deleted root index_html
-            request.RESPONSE.notFoundError('no default view (root index_html'
+            request.RESPONSE.notFoundError('no default view (root default view'
                                            ' was probably deleted)')
         n = request.steps[-1]
         # default to accessed and container as v.aq_parent
@@ -742,7 +753,7 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
             a = request['PARENTS'][-1]
         if c is request_container:
             c = request['PARENTS'][-1]
-            
+
         return a, c, n, v
 
     def _isTop(self):
@@ -830,33 +841,33 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
     def _addUser(self,name,password,confirm,roles,domains,REQUEST=None):
         if not name:
             return MessageDialog(
-                   title  ='Illegal value', 
+                   title  ='Illegal value',
                    message='A username must be specified',
                    action ='manage_main')
         if not password or not confirm:
             if not domains:
                 return MessageDialog(
-                   title  ='Illegal value', 
+                   title  ='Illegal value',
                    message='Password and confirmation must be specified',
                    action ='manage_main')
         if self.getUser(name) or (self._emergency_user and
                                   name == self._emergency_user.getUserName()):
             return MessageDialog(
-                   title  ='Illegal value', 
+                   title  ='Illegal value',
                    message='A user with the specified name already exists',
                    action ='manage_main')
         if (password or confirm) and (password != confirm):
             return MessageDialog(
-                   title  ='Illegal value', 
+                   title  ='Illegal value',
                    message='Password and confirmation do not match',
                    action ='manage_main')
-        
+
         if not roles: roles=[]
         if not domains: domains=[]
 
         if domains and not self.domainSpecValidate(domains):
             return MessageDialog(
-                   title  ='Illegal value', 
+                   title  ='Illegal value',
                    message='Illegal domain specification',
                    action ='manage_main')
         self._doAddUser(name, password, roles, domains)
@@ -869,23 +880,23 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
             password = confirm = None
         if not name:
             return MessageDialog(
-                   title  ='Illegal value', 
+                   title  ='Illegal value',
                    message='A username must be specified',
                    action ='manage_main')
         if password == confirm == '':
             if not domains:
                 return MessageDialog(
-                   title  ='Illegal value', 
+                   title  ='Illegal value',
                    message='Password and confirmation must be specified',
                    action ='manage_main')
         if not self.getUser(name):
             return MessageDialog(
-                   title  ='Illegal value', 
+                   title  ='Illegal value',
                    message='Unknown user',
                    action ='manage_main')
         if (password or confirm) and (password != confirm):
             return MessageDialog(
-                   title  ='Illegal value', 
+                   title  ='Illegal value',
                    message='Password and confirmation do not match',
                    action ='manage_main')
 
@@ -894,7 +905,7 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
 
         if domains and not self.domainSpecValidate(domains):
             return MessageDialog(
-                   title  ='Illegal value', 
+                   title  ='Illegal value',
                    message='Illegal domain specification',
                    action ='manage_main')
         self._doChangeUser(name, password, roles, domains)
@@ -903,7 +914,7 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
     def _delUsers(self,names,REQUEST=None):
         if not names:
             return MessageDialog(
-                   title  ='Illegal value', 
+                   title  ='Illegal value',
                    message='No users specified',
                    action ='manage_main')
         self._doDelUsers(names)
@@ -1034,7 +1045,7 @@ class UserFolder(BasicUserFolder):
     def _doChangeUser(self, name, password, roles, domains, **kw):
         user=self.data[name]
         if password is not None:
-            if self.encrypt_passwords:
+            if self.encrypt_passwords and not self._isPasswordEncrypted(password):
                 password = self._encryptPassword(password)
             user.__=password
         user.roles=roles
@@ -1131,7 +1142,7 @@ def domainSpecMatch(spec, request):
     _addr=addr.split('.')
     _hlen=len(_host)
     _alen=len(_addr)
-    
+
     for ob in spec:
         sz=len(ob)
         _ob=ob.split('.')
@@ -1139,7 +1150,7 @@ def domainSpecMatch(spec, request):
 
         mo = addr_match(ob)
         if mo is not None:
-            if mo.end(0)==sz: 
+            if mo.end(0)==sz:
                 fail=0
                 for i in range(_sz):
                     a=_addr[i]
