@@ -36,7 +36,7 @@
   USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
   DAMAGE.
 
-  $Id: cAccessControl.c,v 1.13.2.2 2002/04/12 15:18:44 matt Exp $
+  $Id: cAccessControl.c,v 1.17.6.7 2004/01/27 17:10:54 Brian Exp $
 
   If you have questions regarding this software,
   contact:
@@ -48,11 +48,12 @@
 
 */
 
+#include "ExtensionClass.h"
+#include "Acquisition.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 
-#include "ExtensionClass.h"
-#include "Acquisition.h"
 
 
 static void
@@ -636,6 +637,7 @@ static PyExtensionClass imPermissionRoleType = {
 */
 
 static PyObject *Containers = NULL;
+static PyObject *ContainerAssertions = NULL;
 static PyObject *Unauthorized = NULL;
 static PyObject *LOG = NULL;
 static PyObject *PROBLEM = NULL;
@@ -654,9 +656,15 @@ static PyObject *validate_str = NULL;
 static PyObject *_proxy_roles_str = NULL;
 static PyObject *allowed_str = NULL;
 static PyObject *getOwner_str = NULL;
+static PyObject *getPhysicalRoot_str = NULL;
 static PyObject *checkPermission_str = NULL;
 static PyObject *getSecurityManager = NULL;
+static PyObject *unrestrictedTraverse_str = NULL;
 static PyObject *aq_validate = NULL;
+static PyObject *aq_parent_str = NULL;
+static PyObject *_check_context_str = NULL;
+static PyObject *getWrappedOwner_str = NULL;
+
 static int ownerous = 1;
 static int authenticated = 1;
 
@@ -691,6 +699,17 @@ ZopeSecurityPolicy_setup(void) {
           return -1;
 	UNLESS (allowed_str = PyString_FromString("allowed")) return -1;
 	UNLESS (getOwner_str = PyString_FromString("getOwner")) return -1;
+	UNLESS (getWrappedOwner_str = PyString_FromString("getWrappedOwner")) 
+	  return -1;
+	UNLESS (getPhysicalRoot_str = PyString_FromString("getPhysicalRoot")) 
+	  return -1;
+	UNLESS (aq_parent_str = PyString_FromString("aq_parent")) return -1;
+	UNLESS (_check_context_str = PyString_FromString("_check_context")) 
+	  return -1;
+	UNLESS (unrestrictedTraverse_str = PyString_FromString(
+					   "unrestrictedTraverse")) 
+	  return -1;
+
 	UNLESS (checkPermission_str = PyString_FromString("checkPermission")) 
           return -1;
         UNLESS (__allow_access_to_unprotected_subobjects__ = 
@@ -742,8 +761,14 @@ static PyObject *ZopeSecurityPolicy_validate(PyObject *self, PyObject *args) {
 	PyObject *rval = NULL;
 	PyObject *stack = NULL;
 	PyObject *user = NULL;
+
+	PyObject *method = NULL;
+	PyObject *tmp = NULL;
+
 	char *sname;
 
+	int i, l, contains;
+	PyObject *r;
 
 	/*| def validate(self, accessed, container, name, value, context
 	**|	roles=_noroles ...
@@ -774,7 +799,11 @@ static PyObject *ZopeSecurityPolicy_validate(PyObject *self, PyObject *args) {
 	Py_XINCREF(roles);	/* Convert the borrowed ref to a real one */
 
 	/*| containerbase = aq_base(container)
-	**| accessedbase = getattr(accessed, 'aq_base', container)
+	**| accessedbase = aq_base(accessed)
+	**| if accessedbase is accessed:
+        **|     # accessed is not a wrapper, so assume that the
+        **|     # value could not have been acquired.
+	**|     accessedbase = container
 	*/
 
 	containerbase = aq_base(container);
@@ -822,13 +851,12 @@ static PyObject *ZopeSecurityPolicy_validate(PyObject *self, PyObject *args) {
 
 		/*| roles = getattr(container, "__roles__", _noroles)
 		**| if roles is _noroles:
-		**|    aq = getattr(container, 'aq_acquire', None)
-		**|    if aq is None:
+		**|    if containerbase is container:
 		**|       roles = _noroles
 		**|       if containerbase is not accessedbase: return 0
 		**|    else:
 		**|       # Try to acquire roles
-		**|      try: roles = aq('__roles__')
+		**|      try: roles = container.aq_aquire('__roles__')
 		**|      except AttributeError:
 		**|	    roles = _noroles
 		**|         if containerbase is not accessedbase: return 0
@@ -890,24 +918,38 @@ static PyObject *ZopeSecurityPolicy_validate(PyObject *self, PyObject *args) {
 		**|    tp = type(p)
 		**|    if tp is not IntType:
 		**|       if tp is DictType:
-		**|          p = p.get(name, None)
+                **|         if (isinstance(name, StringType) or
+                **|             isinstance(name, UnicodeType)):
+                **|             p=p.get(name, None)
+                **|         else:
+                **|             p = 1
 		**|       else:
 		**|          p = p(name, value)
 		*/
 
-		if (p) {
-			if (! PyInt_Check(p)) {
-				if (PyDict_Check(p)) {
-                                        ASSIGN(p, PyObject_GetItem(p, name));
-                                        if (p == NULL)
-                                          PyErr_Clear();
-				} else {
-                                  ASSIGN(p, callfunction2(p, name, value));
-                                  if (p == NULL)
-                                    goto err;
-				}
-			}
-		}
+		if (p) 
+                  {
+                    if (! PyInt_Check(p)) 
+                      {
+                        if (PyDict_Check(p)) 
+                          {
+                            if (PyString_Check(name) || PyUnicode_Check(name))
+                              {
+                                ASSIGN(p, PyObject_GetItem(p, name));
+                                if (p == NULL)
+                                  PyErr_Clear();
+                              }
+                            else
+                              p = PyInt_FromLong(1);
+                          } 
+                        else 
+                          {
+                            ASSIGN(p, callfunction2(p, name, value));
+                            if (p == NULL)
+                              goto err;
+                          }
+                      }
+                  }
 
 		/*| if not p:
 		**|    if containerbase is accessedbase:
@@ -1028,7 +1070,7 @@ static PyObject *ZopeSecurityPolicy_validate(PyObject *self, PyObject *args) {
                   }
 
 		if (owner != Py_None) {
-                  owner = PyObject_GetAttr(owner, allowed_str);
+                  ASSIGN(owner,PyObject_GetAttr(owner, allowed_str));
                   if (owner)
                     ASSIGN(owner, callfunction2(owner, value, roles));
                   if (owner == NULL)
@@ -1055,26 +1097,85 @@ static PyObject *ZopeSecurityPolicy_validate(PyObject *self, PyObject *args) {
 	/*|    # Proxy roles, which are a lot safer now
 	**|    proxy_roles = getattr(eo, "_proxy_roles", None)
 	**|    if proxy_roles:
+	**|        # Verify that the owner actually can state the proxy role
+	**|        # in the context of the accessed item; users in subfolders
+	**|        # should not be able to use proxy roles to access items 
+	**|        # above their subfolder!
+	**|        owner = eo.getWrappedOwner()
+	**|                        
+	**|        if owner is not None:
+	**|            if container is not containerbase:
+        **|                # Unwrapped objects don't need checking
+	**|                if not owner._check_context(container):
+	**|                    # container is higher up than the owner, 
+	**|                    # deny access
+	**|                    raise Unauthorized(name, value)
+	**|
 	**|       for r in proxy_roles:
 	**|          if r in roles: return 1
 	**|
 	**|       # proxy roles actually limit access!
 	**|       if accessedbase is containerbase:
-	**|	     raise Unauthorized, ('You are not authorized to access'
-	**|		'<em>%s</em>.' % cleanupName(name, value))
+	**|	     raise Unauthorized(name, value)
 	**|
 	**|	  return 0
 	*/
+
 		proxy_roles = PyObject_GetAttr(eo, _proxy_roles_str);
-                Py_DECREF(eo);
+
 		if (proxy_roles == NULL) 
                   {
+		    Py_DECREF(eo);
                     PyErr_Clear();
                   }
                 else if (PyObject_IsTrue(proxy_roles)) 
                   {
-                    int i, l, contains=0;
-                    PyObject *r;
+
+		    method = PyObject_GetAttr(eo, getWrappedOwner_str);
+		    if (method == NULL) {
+		      Py_DECREF(eo);
+		      Py_DECREF(proxy_roles);
+		      goto err;
+		    }
+
+		    owner = PyObject_CallObject(method, NULL);
+		    Py_DECREF(method);
+		    if (owner == NULL) {
+		      Py_DECREF(eo);
+		      Py_DECREF(proxy_roles);
+		      goto err;
+		    }
+
+		    Py_DECREF(eo);
+
+		    if (owner != Py_None) {
+
+		      if (containerbase != container) {
+
+			tmp = callmethod1(owner,_check_context_str,
+					  container
+					  );
+			if (tmp == NULL) {
+			  Py_DECREF(proxy_roles);
+			  Py_DECREF(owner);
+			  goto err;
+			}
+
+			if (!PyObject_IsTrue(tmp)) {
+			  Py_DECREF(proxy_roles);
+			  Py_DECREF(owner);
+			  Py_DECREF(tmp);
+			  unauthErr(name, value);
+			  goto err;
+			}
+			Py_DECREF(tmp);
+		      }
+
+		      Py_DECREF(owner);
+		    }
+
+
+                    contains = 0;
                     if (PyTuple_Check(proxy_roles)) 
                       {
                         l=PyTuple_GET_SIZE(proxy_roles);
@@ -1160,11 +1261,10 @@ static PyObject *ZopeSecurityPolicy_validate(PyObject *self, PyObject *args) {
         else
           rval = PyInt_FromLong(0);        
   err:
-
-	Py_XDECREF(stack);
-	Py_XDECREF(roles);
 	Py_XDECREF(containerbase);
 	Py_XDECREF(accessedbase);
+	Py_XDECREF(stack);
+	Py_XDECREF(roles);
 
 	return rval;
 }
@@ -1955,16 +2055,38 @@ guarded_getattr(PyObject *inst, PyObject *name, PyObject *default_,
         }
 
       /*
-        if Containers(type(inst)):
-            # Simple type.  Short circuit.
+
+        assertion = Containers(type(inst))
+        if assertion:
+            if callable(assertion):
+                factory = assertion(name, v)
+                if callable(factory):
+                    return factory(inst, name)
+                assert factory == 1
+            assert callable == 1
             return v
+
       */
-      t=callfunction1(Containers, OBJECT(inst->ob_type));
-      if (t==NULL) goto err;
-      i=PyObject_IsTrue(t);
-      if (i < 0) goto err;
-      Py_DECREF(t);
-      if (i) return v;
+      t = PyDict_GetItem(ContainerAssertions, OBJECT(inst->ob_type));
+      if (t != NULL)
+        {
+          if (PyCallable_Check(t))
+            {
+              PyObject *factory;
+
+              factory = callfunction2(t, name, v);
+              if (factory == NULL) 
+                goto err;
+
+              if (PyCallable_Check(factory))
+                {
+                  Py_DECREF(v);
+                  v = callfunction2(factory, inst, name);
+                }
+              Py_DECREF(factory);
+            }
+          return v;
+        }
 
       /*
         # Filter out the objects we can't access.
@@ -2077,7 +2199,6 @@ static struct PyMethodDef dtml_methods[] = {
 void initcAccessControl(void) {
 	PyObject *module;
 	PyObject *dict;
-	char *rev = "$Revision: 1.13.2.2 $";
         PURE_MIXIN_CLASS(RestrictedDTMLMixin,
                          "A mix-in for derivatives of DT_String.String "
                          "that adds Zope security."
@@ -2098,16 +2219,13 @@ void initcAccessControl(void) {
 
 	module = Py_InitModule3("cAccessControl",
 		cAccessControl_methods,
-		"$Id: cAccessControl.c,v 1.13.2.2 2002/04/12 15:18:44 matt Exp $\n");
+		"$Id: cAccessControl.c,v 1.17.6.7 2004/01/27 17:10:54 Brian Exp $\n");
 
 	aq_init(); /* For Python <= 2.1.1, aq_init() should be after
                       Py_InitModule(). */
 
 	dict = PyModule_GetDict(module);
 
-	PyDict_SetItemString(dict, "__version__",
-		PyString_FromStringAndSize(rev+11,strlen(rev+11)-2));
-
 	PyDict_SetItemString(dict, "_what_not_even_god_should_do",
 		_what_not_even_god_should_do);
 
@@ -2136,6 +2254,7 @@ void initcAccessControl(void) {
 
 	IMPORT(module, "AccessControl.SimpleObjectPolicies");
 	GETATTR(module, Containers);
+	GETATTR(module, ContainerAssertions);
 	Py_DECREF(module);
 	module = NULL;
 
