@@ -1,20 +1,20 @@
 ##############################################################################
 #
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
-# 
+#
 ##############################################################################
 __doc__="""System management components"""
-__version__='$Revision: 1.77 $'[11:-2]
+__version__='$Revision: 1.81.6.4 $'[11:-2]
 
 
-import sys,os,time,string,Globals, Acquisition, os, Undo
+import sys,os,time,Globals, Acquisition, os, Undo
 from Globals import DTMLFile
 from OFS.ObjectManager import ObjectManager
 from OFS.Folder import Folder
@@ -25,8 +25,9 @@ from OFS import SimpleItem
 from App.Dialogs import MessageDialog
 from Product import ProductFolder
 from version_txt import version_txt
-from StringIO import StringIO
+from cStringIO import StringIO
 from AccessControl import getSecurityManager
+from cgi import escape
 import zLOG
 
 try: import thread
@@ -49,6 +50,8 @@ class DatabaseManager(Fake, SimpleItem.Item, Acquisition.Implicit):
         (
         {'label':'Database', 'action':'manage_main',
          'help':('OFSP','Database-Management_Database.stx')},
+        {'label':'Activity', 'action':'manage_activity',
+         'help':('OFSP','Database-Management_Activity.stx')},
         {'label':'Cache Parameters', 'action':'manage_cacheParameters',
          'help':('OFSP','Database-Management_Cache-Parameters.stx')},
         {'label':'Flush Cache', 'action':'manage_cacheGC',
@@ -58,6 +61,7 @@ class DatabaseManager(Fake, SimpleItem.Item, Acquisition.Implicit):
 
     # These need to be here rather to make tabs work correctly. This
     # needs to be revisited.
+    manage_activity=Globals.DTMLFile('dtml/activity', globals())
     manage_cacheParameters=Globals.DTMLFile('dtml/cacheParameters', globals())
     manage_cacheGC=Globals.DTMLFile('dtml/cacheGC', globals())
 
@@ -79,7 +83,7 @@ class VersionManager(Fake, SimpleItem.Item, Acquisition.Implicit):
          'help':('OFSP','Version-Management_Version.stx')},
         )
         )
-        
+
 Globals.default__class_init__(VersionManager)
 
 
@@ -107,7 +111,7 @@ class DebugManager(Fake, SimpleItem.Item, Acquisition.Implicit):
         )
 
     manage_debug=DTMLFile('dtml/debug', globals())
-    
+
     def refcount(self, n=None, t=(type(Fake), type(Acquisition.Implicit))):
         # return class reference info
         dict={}
@@ -209,7 +213,7 @@ class ApplicationManager(Folder,CacheManager):
     def version_txt(self):
         if not hasattr(self, '_v_version_txt'):
             self._v_version_txt=version_txt()
-                
+
         return self._v_version_txt
 
     def sys_version(self): return sys.version
@@ -235,7 +239,7 @@ class ApplicationManager(Folder,CacheManager):
         )
         +Undo.UndoSupport.manage_options
         )
-    
+
     id        ='Control_Panel'
     name=title='Control Panel'
     meta_type ='Control Panel'
@@ -281,13 +285,13 @@ class ApplicationManager(Folder,CacheManager):
         raise 'Redirect', URL2+'/manage'
 
     def process_time(self):
-        s=int(time.time())-self.process_start   
+        s=int(time.time())-self.process_start
         d=int(s/86400)
         s=s-(d*86400)
         h=int(s/3600)
         s=s-(h*3600)
         m=int(s/60)
-        s=s-(m*60)      
+        s=s-(m*60)
         d=d and ('%d day%s'  % (d, (d != 1 and 's' or ''))) or ''
         h=h and ('%d hour%s' % (h, (h != 1 and 's' or ''))) or ''
         m=m and ('%d min' % m) or ''
@@ -312,7 +316,7 @@ class ApplicationManager(Folder,CacheManager):
 
 
     if hasattr(sys, 'ZMANAGED'):
-        
+
         manage_restartable=1
         def manage_restart(self, URL1):
             """Shut down the application"""
@@ -327,7 +331,7 @@ class ApplicationManager(Folder,CacheManager):
             <head><meta HTTP-EQUIV=REFRESH CONTENT="5; URL=%s/manage_main">
             </head>
             <body>Zope is restarting</body></html>
-            """ % URL1
+            """ % escape(URL1, 1)
             sys.exit(1)
 
     def manage_shutdown(self):
@@ -360,7 +364,6 @@ class ApplicationManager(Folder,CacheManager):
         path_join=os.path.join
         isdir=os.path.isdir
         exists=os.path.exists
-        strip=string.strip
 
         product_dir=path_join(SOFTWARE_HOME,'Products')
         product_names=os.listdir(product_dir)
@@ -380,7 +383,7 @@ class ApplicationManager(Folder,CacheManager):
                 file=open(version_txt, 'r')
                 data=file.readline()
                 file.close()
-                info.append(strip(data))
+                info.append(data.strip())
         return info
 
 
@@ -410,10 +413,13 @@ class ApplicationManager(Folder,CacheManager):
             db.abortVersion(v)
         if REQUEST is not None:
             REQUEST['RESPONSE'].redirect(REQUEST['URL1']+'/manage_main')
-            
+
     def getSOFTWARE_HOME(self):
         return SOFTWARE_HOME
 
+    def getZOPE_HOME(self):
+        return ZOPE_HOME
+
     def getINSTANCE_HOME(self):
         return INSTANCE_HOME
 
@@ -436,4 +442,3 @@ class ApplicationManager(Folder,CacheManager):
             self._objects = tuple(lst)
 
         return Folder.objectIds(self, spec)
-
