@@ -1,22 +1,23 @@
 ##############################################################################
 #
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
-# 
+#
 ##############################################################################
 __doc__='''Cache management support
 
 
-$Id: CacheManager.py,v 1.22 2001/11/28 15:50:52 matt Exp $'''
-__version__='$Revision: 1.22 $'[11:-2]
+$Id: CacheManager.py,v 1.26.6.1 2002/10/09 15:26:26 shane Exp $'''
+__version__='$Revision: 1.26.6.1 $'[11:-2]
 
-import Globals, time, sys, string
+import Globals, time, sys
+from DateTime import DateTime
 
 class CacheManager:
     """Cache management mix-in
@@ -25,10 +26,15 @@ class CacheManager:
     _cache_size=400
     _vcache_age=60
     _vcache_size=400
+    _history_length = 3600  # Seconds
 
     manage_cacheParameters=Globals.DTMLFile('dtml/cacheParameters', globals())
     manage_cacheGC=Globals.DTMLFile('dtml/cacheGC', globals())
 
+    transparent_bar = Globals.ImageFile('www/transparent_bar.gif', globals())
+    store_bar = Globals.ImageFile('www/store_bar.gif', globals())
+    load_bar = Globals.ImageFile('www/load_bar.gif', globals())
+
     def cache_length(self):
         try: db=self._p_jar.db()
         except:
@@ -36,6 +42,12 @@ class CacheManager:
             return len(Globals.Bobobase._jar.cache)
         else: return db.cacheSize()
 
+    def cache_detail_length(self):
+        try: db=self._p_jar.db()
+        except:
+            return ()
+        else: return db.cacheDetailSize()
+
     def database_size(self):
         try: db=self._p_jar.db()
         except:
@@ -87,7 +99,7 @@ class CacheManager:
         "set cache size"
         try:
             v=self._p_jar.getVersion()
-        except: 
+        except:
             # BoboPOS2:
             if self._p_jar.db is not Globals.Bobobase._jar.db:
                 raise 'Version Error', (
@@ -125,7 +137,7 @@ class CacheManager:
                  ))
              ),
             )
-        
+
 
     # BoboPOS 2
     def cache_mean_age(self):
@@ -156,7 +168,7 @@ class CacheManager:
             response=REQUEST['RESPONSE']
             response.redirect(REQUEST['URL1']+'/manage_cacheGC')
 
-    def manage_minimize(self,value,REQUEST):
+    def manage_minimize(self,value=1,REQUEST=None):
         "Perform a full sweep through the cache"
         try: db=self._p_jar.db()
         except:
@@ -179,6 +191,9 @@ class CacheManager:
             db.setCacheDeactivateAfter(self._cache_age)
             db.setVersionCacheSize(self._vcache_size)
             db.setVersionCacheDeactivateAfter(self._vcache_age)
+            am = self._getActivityMonitor()
+            if am is not None:
+                am.setHistoryLength(self._history_length)
 
     def cache_detail(self, REQUEST=None):
         """
@@ -190,8 +205,8 @@ class CacheManager:
         if REQUEST is not None:
             # format as text
             REQUEST.RESPONSE.setHeader('Content-Type', 'text/plain')
-            return string.join(map(lambda (name, count): '%6d %s' %
-                                   (count, name), detail), '\n')
+            return '\n'.join(map(lambda (name, count): '%6d %s' %
+                                   (count, name), detail))
         else:
             # raw
             return detail
@@ -206,7 +221,6 @@ class CacheManager:
             # sort the list.
             lst = map(lambda dict: ((dict['conn_no'], dict['oid']), dict),
                       detail)
-            lst.sort()
             # format as text.
             res = [
                 '# Table shows connection number, oid, refcount, state, '
@@ -229,10 +243,126 @@ class CacheManager:
                     dict['conn_no'], `dict['oid']`, dict['rc'],
                     state, dict['klass'], idinfo))
             REQUEST.RESPONSE.setHeader('Content-Type', 'text/plain')
-            return string.join(res, '\n')
+            return '\n'.join(res)
         else:
             # raw
             return detail
 
-Globals.default__class_init__(CacheManager)
+    def _getActivityMonitor(self):
+        db = self._p_jar.db()
+        if not hasattr(db, 'getActivityMonitor'):
+            return None
+        am = db.getActivityMonitor()
+        if am is None:
+            return None
+        return am
 
+    def getHistoryLength(self):
+        am = self._getActivityMonitor()
+        if am is None:
+            return 0
+        return am.getHistoryLength()
+
+    def manage_setHistoryLength(self, length, REQUEST=None):
+        """Change the length of the activity monitor history.
+        """
+        am = self._getActivityMonitor()
+        length = int(length)
+        if length < 0:
+            raise ValueError, 'length can not be negative'
+        if am is not None:
+            am.setHistoryLength(length)
+        self._history_length = length  # Restore on startup
+
+        if REQUEST is not None:
+            response = REQUEST['RESPONSE']
+            response.redirect(REQUEST['URL1'] + '/manage_activity')
+
+    def getActivityChartData(self, segment_height, REQUEST=None):
+        """Returns information for generating an activity chart.
+        """
+        am = self._getActivityMonitor()
+        if am is None:
+            return None
+
+        if REQUEST is not None:
+            start = float(REQUEST.get('chart_start', 0))
+            end = float(REQUEST.get('chart_end', 0))
+            divisions = int(REQUEST.get('chart_divisions', 10))
+            analysis = am.getActivityAnalysis(start, end, divisions)
+        else:
+            analysis = am.getActivityAnalysis()
+
+        total_load_count = 0
+        total_store_count = 0
+        total_connections = 0
+        limit = 0
+        divs = []
+        for div in analysis:
+            total_store_count = total_store_count + div['stores']
+            total_load_count = total_load_count + div['loads']
+            total_connections = total_connections + div['connections']
+            sum = div['stores'] + div['loads']
+            if sum > limit:
+                limit = sum
+
+        if analysis:
+            segment_time = analysis[0]['end'] - analysis[0]['start']
+        else:
+            segment_time = 0
+
+        for div in analysis:
+            stores = div['stores']
+            if stores > 0:
+                store_len = max(int(segment_height * stores / limit), 1)
+            else:
+                store_len = 0
+            loads = div['loads']
+            if loads > 0:
+                load_len = max(int(segment_height * loads / limit), 1)
+            else:
+                load_len = 0
+
+            t = div['end'] - analysis[-1]['end']  # Show negative numbers.
+            if segment_time >= 3600:
+                # Show hours.
+                time_offset = '%dh' % (t / 3600)
+            elif segment_time >= 60:
+                # Show minutes.
+                time_offset = '%dm' % (t / 60)
+            elif segment_time >= 1:
+                # Show seconds.
+                time_offset = '%ds' % t
+            else:
+                # Show fractions.
+                time_offset = '%.2fs' % t
+            divs.append({
+                'store_len': store_len,
+                'load_len': load_len,
+                'trans_len': max(segment_height - store_len - load_len, 0),
+                'store_count': div['stores'],
+                'load_count': div['loads'],
+                'connections': div['connections'],
+                'start': div['start'],
+                'end': div['end'],
+                'time_offset': time_offset,
+                })
+
+        if analysis:
+            start_time = DateTime(divs[0]['start']).aCommonZ()
+            end_time = DateTime(divs[-1]['end']).aCommonZ()
+        else:
+            start_time = ''
+            end_time = ''
+
+        res = {'start_time': start_time,
+               'end_time': end_time,
+               'divs': divs,
+               'total_store_count': total_store_count,
+               'total_load_count': total_load_count,
+               'total_connections': total_connections,
+               }
+        return res
+
+
+Globals.default__class_init__(CacheManager)
