@@ -1,14 +1,14 @@
 ##############################################################################
 #
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
-# 
+#
 ##############################################################################
 """Objects providing context for product initialization
 """
@@ -19,11 +19,12 @@ from HelpSys import HelpTopic, APIHelpTopic
 from HelpSys.HelpSys import ProductHelp
 from FactoryDispatcher import FactoryDispatcher
 from zLOG import LOG, WARNING
-import string, os.path, re
+import os.path, re
 import stat
 from DateTime import DateTime
 from types import ListType, TupleType
-from Interface import instancesOfObjectImplements
+from Interface.Implements import instancesOfObjectImplements
+from App.Product import doInstall
 
 import ZClasses # to enable 'PC.registerBaseClass()'
 
@@ -41,17 +42,18 @@ class ProductContext:
         self.__app=app
         self.__pack=package
 
-    def registerClass(self, instance_class=None, meta_type='', 
+    def registerClass(self, instance_class=None, meta_type='',
                       permission=None, constructors=(),
                       icon=None, permissions=None, legacy=(),
-                      visibility="Global",interfaces=_marker
+                      visibility="Global",interfaces=_marker,
+                      container_filter=None
         ):
         """Register a constructor
 
         Keyword arguments are used to provide meta data:
 
         instance_class -- The class of the object that will be created.
-        
+
           This is not currently used, but may be used in the future to
           increase object mobility.
 
@@ -80,7 +82,7 @@ class ProductContext:
         permissions -- Additional permissions to be registered
            If not provided, then permissions defined in the
            class will be registered.
-        
+
         legacy -- A list of legacy methods to be added to ObjectManager
                   for backward compatibility
 
@@ -88,6 +90,13 @@ class ProductContext:
 
         interfaces -- a list of the interfaces the object supports
 
+        container_filter -- function that is called with an ObjectManager
+           object as the only parameter, which should return a true object
+           if the object is happy to be created in that container. The
+           filter is called before showing ObjectManager's Add list,
+           and before pasting (after object copy or cut), but not
+           before calling an object's constructor.
+
         """
         app=self.__app
         pack=self.__pack
@@ -161,7 +170,10 @@ class ProductContext:
         m=pack._m
 
         if interfaces is _marker:
-            interfaces = instancesOfObjectImplements(instance_class)
+            if instance_class is None:
+                interfaces = ()
+            else:
+                interfaces = instancesOfObjectImplements(instance_class)
 
         Products.meta_types=Products.meta_types+(
             { 'name': meta_type or instance_class.meta_type,
@@ -171,6 +183,7 @@ class ProductContext:
               'visibility': visibility,
               'interfaces': interfaces,
               'instance': instance_class,
+              'container_filter': container_filter
               },)
 
         m[name]=initial
@@ -207,18 +220,18 @@ class ProductContext:
 
         module=base_class.__module__
         name=base_class.__name__
-            
+
         key="%s/%s" % (module, name)
 
-        if module[:9]=='Products.': module=string.split(module,'.')[1]
-        else: module=string.split(module,'.')[0]
-            
+        if module[:9]=='Products.': module=module.split('.')[1]
+        else: module=module.split('.')[0]
+
         info="%s: %s" % (module, name)
 
         Products.meta_class_info[key]=info # meta_type
         Products.meta_classes[key]=Z
-        
-        
+
+
 
     def registerBaseClass(self, base_class, meta_type=None):
         #
@@ -265,11 +278,15 @@ class ProductContext:
         HelpTopics are created for these kind of files
 
         .dtml            -- DTMLHelpTopic
-        .html .htm       -- TextHelpTopic        
+        .html .htm       -- TextHelpTopic
         .stx .txt        -- STXHelpTopic
         .jpg .png .gif   -- ImageHelpTopic
         .py              -- APIHelpTopic
         """
+
+        if not doInstall():
+            return
+
         help=self.getProductHelp()
         path=os.path.join(Globals.package_home(self.__pack.__dict__),
                           directory)
@@ -293,7 +310,7 @@ class ProductContext:
 
         for file in os.listdir(path):
             ext=os.path.splitext(file)[1]
-            ext=string.lower(ext)
+            ext=ext.lower()
             if ext in ('.dtml',):
                 contents = open(os.path.join(path,file),'rb').read()
                 m = title_re.search(contents)
@@ -313,7 +330,7 @@ class ProductContext:
                 ht=HelpTopic.TextTopic(file, title, os.path.join(path,file))
                 self.registerHelpTopic(file, ht)
             elif ext in ('.stx', '.txt'):
-                title=string.split(open(os.path.join(path,file),'rb').readline(), ':')[0]
+                title=(open(os.path.join(path,file),'rb').readline()).split(':')[0]
                 ht=HelpTopic.STXTopic(file, title, os.path.join(path, file))
                 self.registerHelpTopic(file, ht)
             elif ext in ('.jpg', '.gif', '.png'):
@@ -324,4 +341,3 @@ class ProductContext:
                     continue
                 ht=APIHelpTopic.APIHelpTopic(file, '', os.path.join(path, file))
                 self.registerHelpTopic(file, ht)
-            
