@@ -1,26 +1,26 @@
 ##############################################################################
 #
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
-# 
+#
 ##############################################################################
-__doc__='''short description
+"""
+$Id: Undo.py,v 1.31.6.1 2002/11/15 17:47:43 jeremy Exp $"""
+__version__='$Revision: 1.31.6.1 $'[11:-2]
 
-$Id: Undo.py,v 1.27 2001/11/28 15:50:52 matt Exp $'''
-__version__='$Revision: 1.27 $'[11:-2]
-
-import Globals, ExtensionClass
-from DateTime import DateTime
-from string import atof, find, atoi, split, rfind, join
-from AccessControl import getSecurityManager
 import base64
 
+from AccessControl import getSecurityManager
+from DateTime import DateTime
+import Globals, ExtensionClass
+from ZopeUndo.Prefix import Prefix
+
 class UndoSupport(ExtensionClass.Base):
 
     __ac_permissions__=(
@@ -75,19 +75,19 @@ class UndoSupport(ExtensionClass.Base):
                 first_transaction+PrincipiaUndoBatchSize)
 
         spec={}
-        
+
         # A user is allowed to undo transactions that were initiated
         # by any member of a user folder in the place where the user
-        # is defined.        
+        # is defined.
         user = getSecurityManager().getUser()
         if hasattr(user, 'aq_parent'):
-            path = join(user.aq_parent.getPhysicalPath()[1:-1], '/')
+            path = '/'.join(user.aq_parent.getPhysicalPath()[1:-1])
         else:
             path=''
         if path: spec['user_name']=Prefix(path)
 
         # We also only want to undo things done here
-        opath=join(self.getPhysicalPath(),'/')
+        opath='/'.join(self.getPhysicalPath())
         if opath: spec['description']=Prefix(opath)
 
         r=Globals.UndoManager.undoInfo(
@@ -99,9 +99,9 @@ class UndoSupport(ExtensionClass.Base):
             desc = d['description']
             tid=d['id']
             if desc:
-                desc = split(desc)
+                desc = desc.split()
                 d1=desc[0]
-                desc = join(desc[1:])
+                desc = ''.join(desc[1:])
                 if len(desc) > 60: desc = desc[:56]+' ...'
                 tid = "%s %s %s %s" % (encode64(tid), t, d1, desc)
             else:
@@ -110,39 +110,24 @@ class UndoSupport(ExtensionClass.Base):
 
 
         return r
-    
+
     def manage_undo_transactions(self, transaction_info=(), REQUEST=None):
         """
         """
         undo=Globals.UndoManager.undo
         for tid in transaction_info:
-            tid=split(tid)
+            tid=tid.split()
             if tid:
-                get_transaction().note("Undo %s" % join(tid[1:]))
+                get_transaction().note("Undo %s" % ''.join(tid[1:]))
                 tid=decode64(tid[0])
                 undo(tid)
-            
+
         if REQUEST is None: return
         REQUEST['RESPONSE'].redirect("%s/manage_UndoForm" % REQUEST['URL1'])
         return ''
-                 
+
 Globals.default__class_init__(UndoSupport)
 
-
-class Prefix:
-    
-    __no_side_effects__=1
-
-    def __init__(self, path):
-        self.value = len(path), path
-        
-    def __cmp__(self, o):
-        l,v = self.value
-        rval = cmp(o[:l],v)
-        return rval
-            
-
-
 ########################################################################
 # Blech, need this cause binascii.b2a_base64 is too pickly
 
@@ -153,7 +138,7 @@ def encode64(s, b2a=binascii.b2a_base64):
     r=[]; a=r.append
     for i in range(0, len(s), 57):
         a(b2a(s[i:i+57])[:-1])
-    return join(r,'')
+    return ''.join(r)
 
 def decode64(s, a2b=binascii.a2b_base64):
     __traceback_info__=len(s), `s`
