@@ -1,28 +1,26 @@
 ##############################################################################
 #
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
-# 
+#
 ##############################################################################
 __doc__='''Simple module for writing tar files
 
-$Id: tar.py,v 1.4 2001/11/28 15:50:53 matt Exp $'''
-__version__='$Revision: 1.4 $'[11:-2]
+$Id: tar.py,v 1.6.6.1 2004/01/19 16:24:40 Brian Exp $'''
+__version__='$Revision: 1.6.6.1 $'[11:-2]
 
 import sys, time, zlib
 try:
     from newstruct import pack
 except:
     from struct import pack
-    
 
-from string import find, join
 
 def oct8(i):
     i=oct(i)
@@ -30,8 +28,15 @@ def oct8(i):
 
 def oct12(i):
     i=oct(i)
-    return '0'*(11-len(i))+i+' '
-    
+    v = '0'*(11-len(i))+i+' '
+    if len(v) > 12:
+        left = v[:-12]
+        for c in left:
+            if c != '0':
+                raise ValueError, 'value too large for oct12'
+        return v[-12:]
+    return v
+
 def pad(s,l):
     ls=len(s)
     if ls >= l: raise ValueError, 'value, %s, too wide for field (%d)' % (s,l)
@@ -45,7 +50,7 @@ class TarEntry:
         "Initialize a Tar archive entry"
         self.data=data
         if mtime is None: mtime=int(time.time())
-        header=join([
+        header=''.join([
             pad(path,      100),
             oct8(mode),
             oct8(uid),
@@ -63,7 +68,7 @@ class TarEntry:
             '000000 \0',
             pad(prefix,    155),
             '\0'*12,
-            ], '')
+            ])
         if len(header) != 512: raise 'Bad Header Length', len(header)
         header=(header[:148]+
                 oct8(reduce(lambda a,b: a+b, map(ord,header)))+
@@ -75,14 +80,14 @@ class TarEntry:
         l=len(data)
         if l%512: data=data+'\0'*(512-l%512)
         return self.header+data
-            
+
 def tar(entries):
     r=[]
     ra=r.append
     for name, data in entries:
         ra(str(TarEntry(name,data)))
     ra('\0'*1024)
-    return join(r,'')
+    return ''.join(r)
 
 def tgz(entries):
     c=zlib.compressobj()
@@ -93,7 +98,7 @@ def tgz(entries):
         ra(compress(str(TarEntry(name,data))))
     ra(compress('\0'*1024))
     ra(c.flush())
-    return join(r,'')
+    return ''.join(r)
 
 class tgzarchive:
 
@@ -105,7 +110,7 @@ class tgzarchive:
 
     def finish(self):
         self._f.write('\0'*1024)
-        
+
     def __str__(self):
         return self._f.getdata()
 
@@ -135,4 +140,4 @@ class gzFile:
         append(self._c.flush())
         append(pack("<i", self._crc))
         append(pack("<i", self._l))
-        return join(r,'')
+        return ''.join(r)
