@@ -2,14 +2,14 @@
 #
 # Copyright (c) 2001, 2002 Zope Corporation and Contributors.
 # All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
-# 
+#
 ##############################################################################
 import sys, os, time, random
 import os, sys
@@ -18,10 +18,15 @@ from BTrees.OOBTree import OOBTree, OOBucket, OOSet, OOTreeSet
 from BTrees.IOBTree import IOBTree, IOBucket, IOSet, IOTreeSet
 from BTrees.IIBTree import IIBTree, IIBucket, IISet, IITreeSet
 from BTrees.OIBTree import OIBTree, OIBucket, OISet, OITreeSet
+
+from BTrees.check import check
+
 from unittest import TestCase, TestSuite, TextTestRunner, makeSuite
 
 from glob import glob
 
+from ZODB.tests.StorageTestBase import removefs
+
 class Base:
     """ Tests common to all types: sets, buckets, and BTrees """
     def tearDown(self):
@@ -31,24 +36,24 @@ class Base:
     def _getRoot(self):
         from ZODB.FileStorage import FileStorage
         from ZODB.DB import DB
-        n = 'fs_tmp__%s' % os.getpid()
-        s = FileStorage(n)
+        self._fsname = 'fs_tmp__%s' % os.getpid()
+        s = FileStorage(self._fsname)
         db = DB(s)
         root = db.open().root()
         return root
 
     def _closeDB(self, root):
-        root._p_jar._db.close()
-        root = None
+        if root is not None:
+            root._p_jar._db.close()
 
     def _delDB(self):
-        for file in glob('fs_tmp__*'):
-            os.remove(file)
-        
+        removefs(self._fsname)
+
     def testLoadAndStore(self):
         for i in 0, 10, 1000:
             t = self.t.__class__()
             self._populate(t, i)
+            root = None
             try:
                 root = self._getRoot()
                 root[i] = t
@@ -60,6 +65,7 @@ class Base:
 
             self._closeDB(root)
 
+            root = None
             try:
                 root = self._getRoot()
                 #XXX BTree stuff doesn't implement comparison
@@ -70,11 +76,12 @@ class Base:
             finally:
                 self._closeDB(root)
                 self._delDB()
-            
+
     def testGhostUnghost(self):
         for i in 0, 10, 1000:
             t = self.t.__class__()
             self._populate(t, i)
+            root = None
             try:
                 root = self._getRoot()
                 root[i] = t
@@ -86,6 +93,7 @@ class Base:
 
             self._closeDB(root)
 
+            root = None
             try:
                 root = self._getRoot()
                 root[i]._p_changed = None
@@ -104,7 +112,7 @@ class MappingBase(Base):
     def _populate(self, t, l):
         # Make some data
         for i in range(l): t[i]=i
-    
+
     def testGetItemFails(self):
         self.assertRaises(KeyError, self._getitemfail)
 
@@ -114,7 +122,7 @@ class MappingBase(Base):
     def testGetReturnsDefault(self):
         self.assertEqual(self.t.get(1) , None)
         self.assertEqual(self.t.get(1, 'foo') , 'foo')
-        
+
     def testSetItemGetItemWorks(self):
         self.t[1] = 1
         a = self.t[1]
@@ -150,14 +158,14 @@ class MappingBase(Base):
     def testValuesWorks1(self):
 
         for x in range(100):
-            self.t[99-x] = x 
+            self.t[99-x] = x
 
         for x in range(40):
             lst = list(self.t.values(0+x,99-x))
             lst.sort()
             self.assertEqual(lst,range(0+x,99-x+1))
 
-            
+
     def testKeysWorks(self):
         for x in range(100):
             self.t[x] = x
@@ -219,14 +227,13 @@ class MappingBase(Base):
         self.assertEqual(diff , [], diff)
 
     def testUpdate(self):
-        "mapping update"
         d={}
         l=[]
         for i in range(10000):
             k=random.randrange(-2000, 2001)
             d[k]=i
             l.append((k, i))
-            
+
         items=d.items()
         items.sort()
 
@@ -240,19 +247,125 @@ class MappingBase(Base):
         self.assertEqual(list(self.t.items()) , items)
 
     def testEmptyRangeSearches(self):
-        t=self.t
-        t.update([(1,1),(5,5),(9,9)])
-        self.assertEqual(list(t.keys(-6,-4)),[], list(t.keys(-6,-4)))
-        self.assertEqual(list(t.keys(2,4)),[], list(t.keys(2,4)))
-        self.assertEqual(list(t.keys(6,8)),[], list(t.keys(6,8)))
-        self.assertEqual(list(t.keys(10,12)),[], list(t.keys(10,12)))
-        
+        t = self.t
+        t.update([(1,1), (5,5), (9,9)])
+        self.assertEqual(list(t.keys(-6,-4)), [], list(t.keys(-6,-4)))
+        self.assertEqual(list(t.keys(2,4)), [], list(t.keys(2,4)))
+        self.assertEqual(list(t.keys(6,8)), [], list(t.keys(6,8)))
+        self.assertEqual(list(t.keys(10,12)), [], list(t.keys(10,12)))
+        self.assertEqual(list(t.keys(9, 1)), [], list(t.keys(9, 1)))
+
+        # For IITreeSets, this one was returning 31 for len(keys), and
+        # list(keys) produced a list with 100 elements.
+        t.clear()
+        t.update(zip(range(300), range(300)))
+        keys = t.keys(200, 50)
+        self.assertEqual(len(keys), 0)
+        self.assertEqual(list(keys), [])
+
+    def testSlicing(self):
+        # Test that slicing of .keys()/.values()/.items() works exactly the
+        # same way as slicing a Python list with the same contents.
+        # This tests fixes to several bugs in this area, starting with
+        # http://collector.zope.org/Zope/419,
+        # "BTreeItems slice contains 1 too many elements".
+
+        t = self.t
+        for n in range(10):
+            t.clear()
+            self.assertEqual(len(t), 0)
+
+            keys = []
+            values = []
+            items = []
+            for key in range(n):
+                value = -2 * key
+                t[key] = value
+                keys.append(key)
+                values.append(value)
+                items.append((key, value))
+            self.assertEqual(len(t), n)
+
+            kslice = t.keys()
+            vslice = t.values()
+            islice = t.items()
+            self.assertEqual(len(kslice), n)
+            self.assertEqual(len(vslice), n)
+            self.assertEqual(len(islice), n)
+
+            # Test whole-structure slices.
+            x = kslice[:]
+            self.assertEqual(list(x), keys[:])
+
+            x = vslice[:]
+            self.assertEqual(list(x), values[:])
+
+            x = islice[:]
+            self.assertEqual(list(x), items[:])
+
+            for lo in range(-2*n, 2*n+1):
+                # Test one-sided slices.
+                x = kslice[:lo]
+                self.assertEqual(list(x), keys[:lo])
+                x = kslice[lo:]
+                self.assertEqual(list(x), keys[lo:])
+
+                x = vslice[:lo]
+                self.assertEqual(list(x), values[:lo])
+                x = vslice[lo:]
+                self.assertEqual(list(x), values[lo:])
+
+                x = islice[:lo]
+                self.assertEqual(list(x), items[:lo])
+                x = islice[lo:]
+                self.assertEqual(list(x), items[lo:])
+
+                for hi in range(-2*n, 2*n+1):
+                    # Test two-sided slices.
+                    x = kslice[lo:hi]
+                    self.assertEqual(list(x), keys[lo:hi])
+
+                    x = vslice[lo:hi]
+                    self.assertEqual(list(x), values[lo:hi])
+
+                    x = islice[lo:hi]
+                    self.assertEqual(list(x), items[lo:hi])
+
+        # The specific test case from Zope collector 419.
+        t.clear()
+        for i in xrange(100):
+            t[i] = 1
+        tslice = t.items()[20:80]
+        self.assertEqual(len(tslice), 60)
+        self.assertEqual(list(tslice), zip(range(20, 80), [1]*60))
+
+    def testBadUpdateTupleSize(self):
+        # This one silently ignored the excess in Zope3.
+        try:
+            self.t.update([(1, 2, 3)])
+        except TypeError:
+            pass
+        else:
+            self.fail("update() with 3-tuple didn't complain")
+
+        # This one dumped core in Zope3.
+        try:
+            self.t.update([(1,)])
+        except TypeError:
+            pass
+        else:
+            self.fail("update() with 1-tuple didn't complain")
+
+        # This one should simply succeed.
+        self.t.update([(1, 2)])
+        self.assertEqual(list(self.t.items()), [(1, 2)])
+
 
 class NormalSetTests(Base):
     """ Test common to all set types """
 
 
-    def _populate(self, t, l): 
+    def _populate(self, t, l):
         # Make some data
         t.update(range(l))
 
@@ -265,7 +378,7 @@ class NormalSetTests(Base):
         t = self.t
         t.insert(5)
         self.assertEqual(t.insert(5) , 0)
-        
+
     def testInsert(self):
         t = self.t
         t.insert(1)
@@ -328,14 +441,13 @@ class NormalSetTests(Base):
         self.assertEqual(t.minKey(9) , 10)
 
     def testUpdate(self):
-        "mapping update"
         d={}
         l=[]
         for i in range(10000):
             k=random.randrange(-2000, 2001)
             d[k]=i
             l.append(k)
-            
+
         items=d.keys()
         items.sort()
 
@@ -343,12 +455,54 @@ class NormalSetTests(Base):
         self.assertEqual(list(self.t.keys()) , items)
 
     def testEmptyRangeSearches(self):
-        t=self.t
-        t.update([1,5,9])
-        self.assertEqual(list(t.keys(-6,-4)),[], list(t.keys(-6,-4)))
-        self.assertEqual(list(t.keys(2,4)),[], list(t.keys(2,4)))
-        self.assertEqual(list(t.keys(6,8)),[], list(t.keys(6,8)))
-        self.assertEqual(list(t.keys(10,12)),[], list(t.keys(10,12)))
+        t = self.t
+        t.update([1, 5, 9])
+        self.assertEqual(list(t.keys(-6,-4)), [], list(t.keys(-6,-4)))
+        self.assertEqual(list(t.keys(2,4)), [], list(t.keys(2,4)))
+        self.assertEqual(list(t.keys(6,8)), [], list(t.keys(6,8)))
+        self.assertEqual(list(t.keys(10,12)), [], list(t.keys(10,12)))
+        self.assertEqual(list(t.keys(9,1)), [], list(t.keys(9,1)))
+
+        # For IITreeSets, this one was returning 31 for len(keys), and
+        # list(keys) produced a list with 100 elements.
+        t.clear()
+        t.update(range(300))
+        keys = t.keys(200, 50)
+        self.assertEqual(len(keys), 0)
+        self.assertEqual(list(keys), [])
+
+    def testSlicing(self):
+        # Test that slicing of .keys() works exactly the same way as slicing
+        # a Python list with the same contents.
+
+        t = self.t
+        for n in range(10):
+            t.clear()
+            self.assertEqual(len(t), 0)
+
+            keys = range(10*n, 11*n)
+            t.update(keys)
+            self.assertEqual(len(t), n)
+
+            kslice = t.keys()
+            self.assertEqual(len(kslice), n)
+
+            # Test whole-structure slices.
+            x = kslice[:]
+            self.assertEqual(list(x), keys[:])
+
+            for lo in range(-2*n, 2*n+1):
+                # Test one-sided slices.
+                x = kslice[:lo]
+                self.assertEqual(list(x), keys[:lo])
+                x = kslice[lo:]
+                self.assertEqual(list(x), keys[lo:])
+
+                for hi in range(-2*n, 2*n+1):
+                    # Test two-sided slices.
+                    x = kslice[lo:hi]
+                    self.assertEqual(list(x), keys[lo:hi])
+
 
 class ExtendedSetTests(NormalSetTests):
     def testLen(self):
@@ -363,13 +517,19 @@ class ExtendedSetTests(NormalSetTests):
         for x in r: t.insert(x)
         for x in r:
             self.assertEqual(t[x] , x)
-        
+
 class BucketTests(MappingBase):
     """ Tests common to all buckets """
     pass
 
 class BTreeTests(MappingBase):
     """ Tests common to all BTrees """
+
+    def tearDown(self):
+        self.t._check()
+        check(self.t)
+        MappingBase.tearDown(self)
+
     def testDeleteNoChildrenWorks(self):
         self.t[5] = 6
         self.t[2] = 10
@@ -405,9 +565,9 @@ class BTreeTests(MappingBase):
         del self.t[2]
         diff = lsubtract(self.t.keys(), [1,3,4,5,6,10])
         self.assertEqual(diff , [], diff)
-        
+
     def testDeleteTwoChildrenInorderSuccessorWorks(self):
-        """ 7, 3, 8, 1, 5, 10, 6, 4 -- del 3 """
+        # 7, 3, 8, 1, 5, 10, 6, 4 -- del 3
         self.t[7] = 6
         self.t[3] = 10
         self.t[8] = 12
@@ -421,7 +581,7 @@ class BTreeTests(MappingBase):
         self.assertEqual(diff , [], diff)
 
     def testDeleteRootWorks(self):
-        """ 7, 3, 8, 1, 5, 10, 6, 4 -- del 7 """
+        # 7, 3, 8, 1, 5, 10, 6, 4 -- del 7
         self.t[7] = 6
         self.t[3] = 10
         self.t[8] = 12
@@ -491,7 +651,7 @@ class BTreeTests(MappingBase):
             except KeyError:
                 pass
         self.assertEqual(realseq(self.t.keys()) , [], realseq(self.t.keys()))
-        
+
     def testPathologicalRightBranching(self):
         r = range(1000)
         for x in r:
@@ -576,6 +736,33 @@ class BTreeTests(MappingBase):
         diff = lsubtract(list(self.t.keys(0, 100)), a.keys())
         self.assertEqual(diff , [], diff)
 
+    def testPathologicalRangeSearch(self):
+        t = self.t
+        # Build a 2-level tree with at least two buckets.
+        for i in range(200):
+            t[i] = i
+        items, dummy = t.__getstate__()
+        self.assert_(len(items) > 2)   # at least two buckets and a key
+        # All values in the first bucket are < firstkey.  All in the
+        # second bucket are >= firstkey, and firstkey is the first key in
+        # the second bucket.
+        firstkey = items[1]
+        therange = t.keys(-1, firstkey)
+        self.assertEqual(len(therange), firstkey + 1)
+        self.assertEqual(list(therange), range(firstkey + 1))
+        # Now for the tricky part.  If we delete firstkey, the second bucket
+        # loses its smallest key, but firstkey remains in the BTree node.
+        # If we then do a high-end range search on firstkey, the BTree node
+        # directs us to look in the second bucket, but there's no longer any
+        # key <= firstkey in that bucket.  The correct answer points to the
+        # end of the *first* bucket.  The algorithm has to be smart enough
+        # to "go backwards" in the BTree then; if it doesn't, it will
+        # erroneously claim that the range is empty.
+        del t[firstkey]
+        therange = t.keys(-1, firstkey)
+        self.assertEqual(len(therange), firstkey)
+        self.assertEqual(list(therange), range(firstkey))
+
     def testInsertMethod(self):
         t = self.t
         t[0] = 1
@@ -583,6 +770,30 @@ class BTreeTests(MappingBase):
         self.assertEqual(t.insert(1, 1) , 1)
         self.assertEqual(lsubtract(list(t.keys()), [0,1]) , [])
 
+    def testDamagedIterator(self):
+        # A cute one from Steve Alexander.  This caused the BTreeItems
+        # object to go insane, accessing memory beyond the allocated part
+        # of the bucket.  If it fails, the symptom is either a C-level
+        # assertion error (if the BTree code was compiled without NDEBUG),
+        # or most likely a segfault (if the BTree code was compiled with
+        # NDEBUG).
+
+        t = self.t.__class__()
+        self._populate(t, 10)
+        # In order for this to fail, it's important that k be a "lazy"
+        # iterator, referring to the BTree by indirect position (index)
+        # instead of a fully materialized list.  Then the position can
+        # end up pointing into trash memory, if the bucket pointed to
+        # shrinks.
+        k = t.keys()
+        for dummy in range(20):
+            try:
+                del t[k[0]]
+            except RuntimeError, detail:
+                self.assertEqual(str(detail), "the bucket being iterated "
+                                              "changed size")
+                break
+
 ## BTree tests
 
 class TestIOBTrees(BTreeTests, TestCase):
@@ -603,6 +814,16 @@ class TestIOBTrees(BTreeTests, TestCase):
     def _noneraises(self):
         self.t[None] = 1
 
+    def testEmptyFirstBucketReportedByGuido(self):
+        b = self.t
+        for i in xrange(29972): # reduce to 29971 and it works
+            b[i] = i
+        for i in xrange(30): # reduce to 29 and it works
+            del b[i]
+            b[i+40000] = i
+
+        self.assertEqual(b.keys()[0], 30)
+
 class TestOOBTrees(BTreeTests, TestCase):
     def setUp(self):
         self.t = OOBTree()
@@ -692,7 +913,7 @@ class TestOISets(ExtendedSetTests, TestCase):
 class TestIOTreeSets(NormalSetTests, TestCase):
     def setUp(self):
         self.t = IOTreeSet()
-        
+
 class TestOOTreeSets(NormalSetTests, TestCase):
     def setUp(self):
         self.t = OOTreeSet()
@@ -701,10 +922,170 @@ class TestIITreeSets(NormalSetTests, TestCase):
     def setUp(self):
         self.t = IITreeSet()
 
+    # Build a degenerate tree (set).  Boxes are BTree nodes.  There are
+    # 5 leaf buckets, each containing a single int.  Keys in the BTree
+    # nodes don't appear in the buckets.  Seven BTree nodes are purely
+    # indirection nodes (no keys).  Buckets aren't all at the same depth:
+    #
+    #     +------------------------+
+    #     |          4             |
+    #     +------------------------+
+    #         |              |
+    #         |              v
+    #         |             +-+
+    #         |             | |
+    #         |             +-+
+    #         |              |
+    #         v              v
+    #     +-------+   +-------------+
+    #     |   2   |   |   6     10  |
+    #     +-------+   +-------------+
+    #      |     |     |     |     |
+    #      v     v     v     v     v
+    #     +-+   +-+   +-+   +-+   +-+
+    #     | |   | |   | |   | |   | |
+    #     +-+   +-+   +-+   +-+   +-+
+    #      |     |     |     |     |
+    #      v     v     v     v     v
+    #      1     3    +-+    7     11
+    #                 | |
+    #                 +-+
+    #                  |
+    #                  v
+    #                  5
+    #
+    # This is nasty for many algorithms.  Consider a high-end range search
+    # for 4.  The BTree nodes direct it to the 5 bucket, but the correct
+    # answer is the 3 bucket, which requires going in a different direction
+    # at the very top node already.  Consider a low-end range search for
+    # 9.  The BTree nodes direct it to the 7 bucket, but the correct answer
+    # is the 11 bucket.  This is also a nasty-case tree for deletions.
+
+    def _build_degenerate_tree(self):
+        # Build the buckets and chain them together.
+        bucket11 = IISet([11])
+
+        bucket7 = IISet()
+        bucket7.__setstate__(((7,), bucket11))
+
+        bucket5 = IISet()
+        bucket5.__setstate__(((5,), bucket7))
+
+        bucket3 = IISet()
+        bucket3.__setstate__(((3,), bucket5))
+
+        bucket1 = IISet()
+        bucket1.__setstate__(((1,), bucket3))
+
+        # Build the deepest layers of indirection nodes.
+        ts = IITreeSet
+        tree1 = ts()
+        tree1.__setstate__(((bucket1,), bucket1))
+
+        tree3 = ts()
+        tree3.__setstate__(((bucket3,), bucket3))
+
+        tree5lower = ts()
+        tree5lower.__setstate__(((bucket5,), bucket5))
+        tree5 = ts()
+        tree5.__setstate__(((tree5lower,), bucket5))
+
+        tree7 = ts()
+        tree7.__setstate__(((bucket7,), bucket7))
+
+        tree11 = ts()
+        tree11.__setstate__(((bucket11,), bucket11))
+
+        # Paste together the middle layers.
+        tree13 = ts()
+        tree13.__setstate__(((tree1, 2, tree3), bucket1))
+
+        tree5711lower = ts()
+        tree5711lower.__setstate__(((tree5, 6, tree7, 10, tree11), bucket5))
+        tree5711 = ts()
+        tree5711.__setstate__(((tree5711lower,), bucket5))
+
+        # One more.
+        t = ts()
+        t.__setstate__(((tree13, 4, tree5711), bucket1))
+        t._check()
+        check(t)
+        return t, [1, 3, 5, 7, 11]
+
+    def testDegenerateBasicOps(self):
+        t, keys = self._build_degenerate_tree()
+        self.assertEqual(len(t), len(keys))
+        self.assertEqual(list(t.keys()), keys)
+        # has_key actually returns the depth of a bucket.
+        self.assertEqual(t.has_key(1), 4)
+        self.assertEqual(t.has_key(3), 4)
+        self.assertEqual(t.has_key(5), 6)
+        self.assertEqual(t.has_key(7), 5)
+        self.assertEqual(t.has_key(11), 5)
+        for i in 0, 2, 4, 6, 8, 9, 10, 12:
+            self.assertEqual(t.has_key(i), 0)
+
+    def _checkRanges(self, tree, keys):
+        self.assertEqual(len(tree), len(keys))
+        sorted_keys = keys[:]
+        sorted_keys.sort()
+        self.assertEqual(list(tree.keys()), sorted_keys)
+        for k in keys:
+            self.assert_(tree.has_key(k))
+        if keys:
+            lokey = sorted_keys[0]
+            hikey = sorted_keys[-1]
+            self.assertEqual(lokey, tree.minKey())
+            self.assertEqual(hikey, tree.maxKey())
+        else:
+            lokey = hikey = 42
+
+        # Try all range searches.
+        for lo in range(lokey - 1, hikey + 2):
+            for hi in range(lo - 1, hikey + 2):
+                want = [k for k in keys if lo <= k <= hi]
+                got = list(tree.keys(lo, hi))
+                self.assertEqual(want, got)
+
+    def testRanges(self):
+        t, keys = self._build_degenerate_tree()
+        self._checkRanges(t, keys)
+
+    def testDeletes(self):
+        # Delete keys in all possible orders, checking each tree along
+        # the way.
+
+        # This is a tough test.  Previous failure modes included:
+        # 1. A variety of assertion failures in _checkRanges.
+        # 2. Assorted "Invalid firstbucket pointer" failures at
+        #    seemingly random times, coming out of the BTree destructor.
+        # 3. Under Python 2.3 CVS, some baffling
+        #      RuntimeWarning: tp_compare didn't return -1 or -2 for exception
+        #    warnings, possibly due to memory corruption after a BTree
+        #    goes insane.
+
+        t, keys = self._build_degenerate_tree()
+        for oneperm in permutations(keys):
+            t, keys = self._build_degenerate_tree()
+            for key in oneperm:
+                t.remove(key)
+                keys.remove(key)
+                t._check()
+                check(t)
+                self._checkRanges(t, keys)
+            # We removed all the keys, so the tree should be empty now.
+            self.assertEqual(t.__getstate__(), None)
+
+            # A damaged tree may trigger an "invalid firstbucket pointer"
+            # failure at the time its destructor is invoked.  Try to force
+            # that to happen now, so it doesn't look like a baffling failure
+            # at some unrelated line.
+            del t   # trigger destructor
+
 class TestOITreeSets(NormalSetTests, TestCase):
     def setUp(self):
         self.t = OITreeSet()
-        
+
 ## Bucket tests
 
 class TestIOBuckets(BucketTests, TestCase):
@@ -723,6 +1104,23 @@ class TestOIBuckets(BucketTests, TestCase):
     def setUp(self):
         self.t = OIBucket()
 
+# cmp error propagation tests
+
+class DoesntLikeBeingCompared:
+    def __cmp__(self,other):
+        raise ValueError('incomparable')
+
+class TestCmpError(TestCase):
+    def testFoo(self):
+        t = OOBTree()
+        t['hello world'] = None
+        try:
+            t[DoesntLikeBeingCompared()] = None
+        except ValueError,e:
+            assert str(e)=='incomparable'
+        else:
+            raise ValueError('incomarable objects should not be allowed into the tree')
+
 def test_suite():
     TIOBTree = makeSuite(TestIOBTrees, 'test')
     TOOBTree = makeSuite(TestOOBTrees, 'test')
@@ -731,23 +1129,25 @@ def test_suite():
 
     TIOSet = makeSuite(TestIOSets, 'test')
     TOOSet = makeSuite(TestOOSets, 'test')
-    TOISet = makeSuite(TestIOSets, 'test')
-    TIISet = makeSuite(TestOOSets, 'test')
+    TOISet = makeSuite(TestOISets, 'test')
+    TIISet = makeSuite(TestIISets, 'test')
 
     TIOTreeSet = makeSuite(TestIOTreeSets, 'test')
     TOOTreeSet = makeSuite(TestOOTreeSets, 'test')
-    TOITreeSet = makeSuite(TestIOTreeSets, 'test')
-    TIITreeSet = makeSuite(TestOOTreeSets, 'test')
+    TOITreeSet = makeSuite(TestOITreeSets, 'test')
+    TIITreeSet = makeSuite(TestIITreeSets, 'test')
 
     TIOBucket = makeSuite(TestIOBuckets, 'test')
     TOOBucket = makeSuite(TestOOBuckets, 'test')
     TOIBucket = makeSuite(TestOIBuckets, 'test')
     TIIBucket = makeSuite(TestIIBuckets, 'test')
-    
+
     alltests = TestSuite((TIOSet, TOOSet, TOISet, TIISet,
                           TIOTreeSet, TOOTreeSet, TOITreeSet, TIITreeSet,
                           TIOBucket, TOOBucket, TOIBucket, TIIBucket,
-                          TOOBTree, TIOBTree, TOIBTree, TIIBTree))
+                          TIOBTree, TOOBTree, TOIBTree, TIIBTree,
+                          makeSuite(TestCmpError),
+                         ))
 
     return alltests
 
@@ -756,19 +1156,32 @@ def test_suite():
 ## utility functions
 
 def lsubtract(l1, l2):
-   l1=list(l1)
-   l2=list(l2)
-   l = filter(lambda x, l1=l1: x not in l1, l2)
-   l = l + filter(lambda x, l2=l2: x not in l2, l1)
-   return l
+    l1=list(l1)
+    l2=list(l2)
+    l = filter(lambda x, l1=l1: x not in l1, l2)
+    l = l + filter(lambda x, l2=l2: x not in l2, l1)
+    return l
 
 def realseq(itemsob):
     return map(lambda x: x, itemsob)
 
+def permutations(x):
+    # Return a list of all permutations of list x.
+    n = len(x)
+    if n <= 1:
+        return [x]
+    result = []
+    x0 = x[0]
+    for i in range(n):
+        # Build the (n-1)! permutations with x[i] in the first position.
+        xcopy = x[:]
+        first, xcopy[i] = xcopy[i], x0
+        result.extend([[first] + p for p in permutations(xcopy[1:])])
+    return result
+
 
 def main():
     TextTestRunner().run(test_suite())
 
 if __name__ == '__main__':
     main()
-
