@@ -1,19 +1,19 @@
 ##############################################################################
 #
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
-# 
+#
 ##############################################################################
 __doc__='''Machinery to support through-the-web editing
 
-$Id: DT_UI.py,v 1.11 2001/11/28 15:50:55 matt Exp $''' 
-__version__='$Revision: 1.11 $'[11:-2]
+$Id: DT_UI.py,v 1.13.2.1 2004/01/08 21:13:03 tseaver Exp $'''
+__version__='$Revision: 1.13.2.1 $'[11:-2]
 
 from DT_HTML import HTML
 
@@ -23,8 +23,8 @@ HTML.document_template_edit_header='<h2>Edit Document</h2>'
 HTML.document_template_form_header=''
 HTML.document_template_edit_footer=(
     """<FONT SIZE="-1">
-    <I><A HREF="http://www.digicool.com/products/copyright.html">
-    &copy; 1997 Digital Creations, L.L.C.</A></I></FONT>""")
+    <I><A HREF="http://www.zope.com">
+    &copy; 2002 Zope Corporation</A></I></FONT>""")
 
 HTML.document_template_edit_width=58
 
@@ -35,54 +35,50 @@ HTML._manage_editForm = HTML(
     </HEAD>
     <BODY bgcolor="#FFFFFF">
     <!--#var document_template_edit_header-->
-    
-    <FORM name="editform" ACTION="<!--#var URL1-->/manage_edit" METHOD="POST">
+
+    <FORM name="editform" ACTION="&dtml-URL1;/manage_edit" METHOD="POST">
     <!--#var document_template_form_header-->
     Document template source:
     <center>
     <br>
-    <TEXTAREA NAME="data:text" cols="<!--#var document_template_edit_width-->" 
+    <TEXTAREA NAME="data:text" cols="&dtml-document_template_edit_width;"
                     rows="20"><!--#var __str__--></TEXTAREA>
 
     <br>
       <INPUT NAME=SUBMIT TYPE="SUBMIT" VALUE="Change">
       <INPUT NAME=SUBMIT TYPE="RESET"  VALUE="Reset">
-      <INPUT NAME="dt_edit_name" TYPE="HIDDEN"
-             VALUE="<!--#var URL1-->">
+      <INPUT NAME="dt_edit_name" TYPE="HIDDEN" VALUE="&dtml-URL1;">
       <!--#if FactoryDefaultString-->
-        <INPUT NAME=SUBMIT TYPE="SUBMIT" 
-         VALUE="<!--#var FactoryDefaultString-->">
+        <INPUT NAME=SUBMIT TYPE="SUBMIT" VALUE="&dtml-FactoryDefaultString;">
       <!--#/if FactoryDefaultString-->
       <INPUT NAME=SUBMIT TYPE="SUBMIT" VALUE="Cancel">
       <!--#if HTTP_REFERER-->
-         <INPUT NAME="CANCEL_ACTION" TYPE="HIDDEN" 
-                VALUE="<!--#var HTTP_REFERER-->">
+         <INPUT NAME="CANCEL_ACTION" TYPE="HIDDEN" VALUE="&dtml-HTTP_REFERER;">
       <!--#else HTTP_REFERER-->
          <!--#if URL1-->
-           <INPUT NAME="CANCEL_ACTION" TYPE="HIDDEN"
-                  VALUE="<!--#var URL1-->">
+           <INPUT NAME="CANCEL_ACTION" TYPE="HIDDEN" VALUE="&dtml-URL1;">
          <!--#/if URL1-->
       <!--#/if HTTP_REFERER-->
     </center>
     </FORM>
-    
+
     <BR CLEAR="ALL">
     <!--#var document_template_edit_footer-->
-    
+
     </BODY>
     </HTML>""",)
 
 HTML.editConfirmation=HTML(
     """<html><head><title>Change Successful</title></head><body>
     <!--#if CANCEL_ACTION-->
-      <form action="<!--#var CANCEL_ACTION-->" method="POST">
+      <form action="&dtml-CANCEL_ACTION;" method="POST">
         <center>
-           <em><!--#var dt_edit_name--></em><br>has been changed.<br><br>
+           <em>&dtml-dt_edit_name;</em><br>has been changed.<br><br>
            <input type=submit name="SUBMIT" value="OK">
         </center>
       </form></body></html>
     <!--#else CANCEL_ACTION-->
       <center>
-         <em><!--#var dt_edit_name--></em><br>has been changed.
+         <em>&dtml-dt_edit_name;</em><br>has been changed.
       </center>
     <!--#/if CANCEL_ACTION-->""")
