@@ -1,14 +1,14 @@
 ##############################################################################
 #
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
-# 
+#
 ##############################################################################
 """Provide a Main application for the Zope framework
 
@@ -27,7 +27,7 @@ The Job of this module is to:
 # versions of Persistent etc get registered.
 from BoboPOS import SimpleDB, TJar, SingleThreadedTransaction
 
-import sys, os, string, Globals, OFS.Application
+import sys, os,  Globals, OFS.Application
 Globals.BobobaseName = os.path.join(Globals.data_dir, 'Data.bbb')
 Globals.DatabaseVersion='2'
 
@@ -40,11 +40,11 @@ OFS.Application.import_products()
 revision=read_only=None
 if os.environ.has_key('ZOPE_READ_ONLY'):
     read_only=1
-    try: 
+    try:
         from DateTime import DateTime
         revision=DateTime(os.environ['ZOPE_READ_ONLY']).timeTime()
     except: pass
-        
+
 Bobobase=Globals.Bobobase=BoboPOS.PickleDictionary(
     Globals.BobobaseName, read_only=read_only, revision=revision)
 Globals.opened.append(Bobobase)
@@ -63,7 +63,7 @@ bobo_application=app
 OFS.Application.initialize(app)
 
 if os.environ.has_key('ZOPE_DATABASE_QUOTA'):
-    quota=string.atoi(os.environ['ZOPE_DATABASE_QUOTA'])
+    quota=int(os.environ['ZOPE_DATABASE_QUOTA'])
     Bobobase._jar.db.set_quota(
         lambda x, quota=quota, otherdb=VersionBase.TDB:
         x + otherdb.pos > quota)
