@@ -1,23 +1,22 @@
 ##############################################################################
 #
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
-# 
+#
 ##############################################################################
 __doc__="""Cacheable object and cache management base classes.
 
-$Id: Cache.py,v 1.8 2001/11/28 15:50:57 matt Exp $"""
+$Id: Cache.py,v 1.10 2002/08/14 21:42:56 mj Exp $"""
 
-__version__='$Revision: 1.8 $'[11:-2]
+__version__='$Revision: 1.10 $'[11:-2]
 
 import time, sys
-from string import join
 import Globals
 from Globals import DTMLFile
 from Acquisition import aq_get, aq_acquire, aq_inner, aq_parent, aq_base
@@ -373,7 +372,7 @@ def findCacheables(ob, manager_id, require_assoc, subfolders,
             icon = getattr(aq_base(subob), 'icon', '')
             info = {
                 'sortkey': subpath,
-                'path': join(subpath, '/'),
+                'path': '/'.join(subpath),
                 'title': getattr(aq_base(subob), 'title', ''),
                 'icon': icon,
                 'associated': associated,}
