@@ -1,19 +1,19 @@
 ##############################################################################
 #
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
-# 
+#
 ##############################################################################
 __doc__="""Copy interface"""
-__version__='$Revision: 1.77 $'[11:-2]
+__version__='$Revision: 1.82.6.3 $'[11:-2]
 
-import sys, string, Globals, Moniker, tempfile, ExtensionClass
+import sys,  Globals, Moniker, tempfile, ExtensionClass
 from marshal import loads, dumps
 from urllib import quote, unquote
 from zlib import compress, decompress
@@ -21,6 +21,8 @@ from App.Dialogs import MessageDialog
 from AccessControl import getSecurityManager
 from Acquisition import aq_base, aq_inner, aq_parent
 from zExceptions import Unauthorized
+from AccessControl import getSecurityManager
+from cgi import escape
 
 CopyError='Copy Error'
 
@@ -65,7 +67,7 @@ class CopyContainer(ExtensionClass.Base):
         for id in ids:
             ob=self._getOb(id)
             if not ob.cb_isMoveable():
-                raise CopyError, eNotSupported % id
+                raise CopyError, eNotSupported % escape(id)
             m=Moniker.Moniker(ob)
             oblist.append(m.dump())
         cp=(1, oblist)
@@ -76,7 +78,7 @@ class CopyContainer(ExtensionClass.Base):
             REQUEST['__cp'] = cp
             return self.manage_main(self, REQUEST)
         return cp
-    
+
     def manage_copyObjects(self, ids=None, REQUEST=None, RESPONSE=None):
         """Put a reference to the objects named in ids in the clip board"""
         if ids is None and REQUEST is not None:
@@ -90,7 +92,7 @@ class CopyContainer(ExtensionClass.Base):
         for id in ids:
             ob=self._getOb(id)
             if not ob.cb_isCopyable():
-                raise CopyError, eNotSupported % id
+                raise CopyError, eNotSupported % escape(id)
             m=Moniker.Moniker(ob)
             oblist.append(m.dump())
         cp=(0, oblist)
@@ -129,13 +131,14 @@ class CopyContainer(ExtensionClass.Base):
                 cp=REQUEST['__cp']
         if cp is None:
             raise CopyError, eNoData
-        
+
         try:    cp=_cb_decode(cp)
         except: raise CopyError, eInvalid
 
         oblist=[]
         op=cp[0]
         app = self.getPhysicalRoot()
+        result = []
 
         for mdata in cp[1]:
             m = Moniker.loadMoniker(mdata)
@@ -148,14 +151,16 @@ class CopyContainer(ExtensionClass.Base):
             # Copy operation
             for ob in oblist:
                 if not ob.cb_isCopyable():
-                    raise CopyError, eNotSupported % ob.getId()
+                    raise CopyError, eNotSupported % escape(ob.getId())
                 try:    ob._notifyOfCopyTo(self, op=0)
                 except: raise CopyError, MessageDialog(
                     title='Copy Error',
                     message=sys.exc_info()[1],
                     action ='manage_main')
                 ob=ob._getCopy(self)
+                orig_id=ob.getId()
                 id=self._get_id(ob.getId())
+                result.append({'id':orig_id, 'new_id':id})
                 ob._setId(id)
                 self._setObject(id, ob)
                 ob = self._getOb(id)
@@ -170,7 +175,7 @@ class CopyContainer(ExtensionClass.Base):
             for ob in oblist:
                 id=ob.getId()
                 if not ob.cb_isMoveable():
-                    raise CopyError, eNotSupported % id
+                    raise CopyError, eNotSupported % escape(id)
                 try:    ob._notifyOfCopyTo(self, op=1)
                 except: raise CopyError, MessageDialog(
                     title='Move Error',
@@ -182,10 +187,12 @@ class CopyContainer(ExtensionClass.Base):
                 # try to make ownership explicit so that it gets carried
                 # along to the new location if needed.
                 ob.manage_changeOwnershipType(explicit=1)
-                
+
                 aq_parent(aq_inner(ob))._delObject(id)
                 ob = aq_base(ob)
+                orig_id=id
                 id=self._get_id(id)
+                result.append({'id':orig_id, 'new_id':id })
                 ob._setId(id)
 
                 self._setObject(id, ob, set_owner=0)
@@ -201,7 +208,7 @@ class CopyContainer(ExtensionClass.Base):
                 REQUEST['__cp'] = None
                 return self.manage_main(self, REQUEST, update_menu=1,
                                         cb_dataValid=0)
-        return ''
+        return result
 
 
     manage_renameForm=Globals.DTMLFile('dtml/renameForm', globals())
@@ -226,7 +233,7 @@ class CopyContainer(ExtensionClass.Base):
                       action ='manage_main')
         ob=self._getOb(id)
         if not ob.cb_isMoveable():
-            raise CopyError, eNotSupported % id            
+            raise CopyError, eNotSupported % escape(id)
         self._verifyObjectPaste(ob)
         try:    ob._notifyOfCopyTo(self, op=1)
         except: raise CopyError, MessageDialog(
@@ -236,7 +243,7 @@ class CopyContainer(ExtensionClass.Base):
         self._delObject(id)
         ob = aq_base(ob)
         ob._setId(new_id)
-        
+
         # Note - because a rename always keeps the same context, we
         # can just leave the ownership info unchanged.
         self._setObject(new_id, ob, set_owner=0)
@@ -253,7 +260,7 @@ class CopyContainer(ExtensionClass.Base):
     def manage_clone(self, ob, id, REQUEST=None):
         # Clone an object, creating a new object with the given id.
         if not ob.cb_isCopyable():
-            raise CopyError, eNotSupported % ob.getId()
+            raise CopyError, eNotSupported % escape(ob.getId())
         try: self._checkId(id)
         except: raise CopyError, MessageDialog(
                       title='Invalid Id',
@@ -269,7 +276,7 @@ class CopyContainer(ExtensionClass.Base):
         ob._setId(id)
         self._setObject(id, ob)
         ob=ob.__of__(self)
-        #ob._postCopy(self, op=0)
+        ob.manage_afterClone(ob)
         return ob
 
     def cb_dataValid(self):
@@ -302,15 +309,15 @@ class CopyContainer(ExtensionClass.Base):
         #
         # Passing a false value for the validate_src argument will skip
         # checking the passed in object in its existing context. This is
-        # mainly useful for situations where the passed in object has no 
+        # mainly useful for situations where the passed in object has no
         # existing context, such as checking an object during an import
         # (the object will not yet have been connected to the acquisition
         # heirarchy).
         if not hasattr(object, 'meta_type'):
             raise CopyError, MessageDialog(
                   title='Not Supported',
-                  message='The object <EM>%s</EM> does not support this ' \
-                          'operation' % absattr(object.id),
+                  message='The object <EM>%s</EM> does not support this' \
+                          ' operation' % escape(absattr(object.id)),
                   action='manage_main')
         mt=object.meta_type
         if not hasattr(self, 'all_meta_types'):
@@ -369,16 +376,22 @@ class CopyContainer(ExtensionClass.Base):
         raise CopyError, MessageDialog(
               title='Not Supported',
               message='The object <EM>%s</EM> does not support this ' \
-                      'operation' % absattr(object.id),
+                      'operation.' % escape(absattr(object.id)),
               action='manage_main')
 
 Globals.default__class_init__(CopyContainer)
 
 
 
-class CopySource:
+class CopySource(ExtensionClass.Base):
     """Interface for objects which allow themselves to be copied."""
-    
+
+    # declare a dummy permission for Copy or Move here that we check
+    # in cb_isCopyable.
+    __ac_permissions__=(
+        ('Copy or Move', (), ('Anonymous', 'Manager',)),
+        )
+
     def _canCopy(self, op=0):
         """Called to make sure this object is copyable. The op var
         is 0 for a copy, 1 for a move."""
@@ -391,6 +404,21 @@ class CopySource:
         pass
 
     def _getCopy(self, container):
+        # Commit a subtransaction to:
+        # 1) Make sure the data about to be exported is current
+        # 2) Ensure self._p_jar and container._p_jar are set even if
+        #    either one is a new object
+        get_transaction().commit(1)
+
+        if self._p_jar is None:
+            raise CopyError, (
+                'Object "%s" needs to be in the database to be copied' %
+                `self`)
+        if container._p_jar is None:
+            raise CopyError, (
+                'Container "%s" needs to be in the database' %
+                `container`)
+
         # Ask an object for a new copy of itself.
         f=tempfile.TemporaryFile()
         self._p_jar.exportFile(self._p_oid,f)
@@ -403,7 +431,7 @@ class CopySource:
         # Called after the copy is finished to accomodate special cases.
         # The op var is 0 for a copy, 1 for a move.
         pass
-    
+
     def _setId(self, id):
         # Called to set the new id of a copied object.
         self.id=id
@@ -412,7 +440,7 @@ class CopySource:
         # Is object copyable? Returns 0 or 1
         if not (hasattr(self, '_canCopy') and self._canCopy(0)):
             return 0
-        if hasattr(self, '_p_jar') and self._p_jar is None:
+        if not self.cb_userHasCopyOrMovePermission():
             return 0
         return 1
 
@@ -426,8 +454,15 @@ class CopySource:
         except: n=()
         if absattr(self.id) in n:
             return 0
+        if not self.cb_userHasCopyOrMovePermission():
+            return 0
         return 1
 
+    def cb_userHasCopyOrMovePermission(self):
+        if getSecurityManager().checkPermission('Copy or Move', self):
+            return 1
+
+Globals.default__class_init__(CopySource)
 
 
 def sanity_check(c, ob):
@@ -463,11 +498,11 @@ def cookie_path(request):
 fMessageDialog=Globals.HTML("""
 <HTML>
 <HEAD>
-<TITLE><dtml-var title></TITLE>
+<TITLE>&dtml-title;</TITLE>
 </HEAD>
 <BODY BGCOLOR="#FFFFFF">
-<FORM ACTION="<dtml-var action>" METHOD="GET" <dtml-if
- target>TARGET="<dtml-var target>"</dtml-if>>
+<FORM ACTION="&dtml-action;" METHOD="GET" <dtml-if
+ target>TARGET="&dtml-target;"</dtml-if>>
 <TABLE BORDER="0" WIDTH="100%%" CELLPADDING="10">
 <TR>
   <TD VALIGN="TOP">
@@ -516,7 +551,15 @@ eNotFound=MessageDialog(
 
 eNotSupported=fMessageDialog(
               title='Not Supported',
-              message='The item <EM>%s</EM> does not support this operation.',
+              message=(
+              'The action against the <em>%s</em> object could not be carried '
+              'out. '
+              'One of the following constraints caused the problem: <br><br>'
+              'The object does not support this operation.'
+              '<br><br>-- OR --<br><br>'
+              'The currently logged-in user does not have the <b>Copy or '
+              'Move</b> permission respective to the object.'
+              ),
               action ='manage_main',)
 
 eNoItemsSpecified=MessageDialog(
