@@ -1,22 +1,21 @@
 ##############################################################################
 #
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
-# 
+#
 ##############################################################################
 """DTML Method objects."""
 
-__version__='$Revision: 1.73 $'[11:-2]
+__version__='$Revision: 1.79.6.4 $'[11:-2]
 
 import History
 from Globals import HTML, DTMLFile, MessageDialog
-from string import join,split,strip,rfind,atoi,lower
 from SimpleItem import Item_w__name__, pretty_tb
 from OFS.content_types import guess_content_type
 from PropertyManager import PropertyManager
@@ -72,6 +71,7 @@ class DTMLMethod(RestrictedDTML, HTML, Acquisition.Implicit, RoleManager,
         +Cacheable.manage_options
         )
 
+    # Careful in changes--used by DTMLDocument!
     __ac_permissions__=(
     ('View management screens',
      ('document_src', 'PrincipiaSearchSource')),
@@ -102,7 +102,7 @@ class DTMLMethod(RestrictedDTML, HTML, Acquisition.Implicit, RoleManager,
             if data is not _marker:
                 # Return cached results.
                 return data
-        
+
         kw['document_id']   =self.getId()
         kw['document_title']=self.title
 
@@ -114,7 +114,7 @@ class DTMLMethod(RestrictedDTML, HTML, Acquisition.Implicit, RoleManager,
             self.__dict__['validate'] = security.DTMLValidate
             first_time_through = 1
         try:
-        
+
             if client is None:
                 # Called as subtemplate, so don't need error propagation!
                 r=apply(HTML.__call__, (self, client, REQUEST), kw)
@@ -180,7 +180,7 @@ class DTMLMethod(RestrictedDTML, HTML, Acquisition.Implicit, RoleManager,
         Returns the cacheNamespaceKeys.
         '''
         return self._cache_namespace_keys
-        
+
     def setCacheNamespaceKeys(self, keys, REQUEST=None):
         '''
         Sets the list of names that should be looked up in the
@@ -188,7 +188,7 @@ class DTMLMethod(RestrictedDTML, HTML, Acquisition.Implicit, RoleManager,
         '''
         ks = []
         for key in keys:
-            key = strip(str(key))
+            key = str(key).strip()
             if key:
                 ks.append(key)
         self._cache_namespace_keys = tuple(ks)
@@ -221,18 +221,21 @@ class DTMLMethod(RestrictedDTML, HTML, Acquisition.Implicit, RoleManager,
 
     def _er(self,data,title,SUBMIT,dtpref_cols,dtpref_rows,REQUEST):
         dr,dc = self._size_changes[SUBMIT]
-        
-        rows=max(1,atoi(dtpref_rows)+dr)
-        cols=max(40,atoi(dtpref_cols)+dc)
-        e=(DateTime('GMT') + 365).rfc822()
-        resp=REQUEST['RESPONSE']
-        resp.setCookie('dtpref_rows',str(rows),path='/',expires=e)
-        resp.setCookie('dtpref_cols',str(cols),path='/',expires=e)
-        return self.manage_main(
-            self,REQUEST,title=title,__str__=self.quotedHTML(data),
-            dtpref_cols=cols,dtpref_rows=rows)
+        rows = str(max(1, int(dtpref_rows) + dr))
+        cols = str(dtpref_cols)
+        if cols.endswith('%'):
+           cols = str(min(100, max(25, int(cols[:-1]) + dc))) + '%'
+        else:
+           cols = str(max(35, int(cols) + dc))
+        e = (DateTime("GMT") + 365).rfc822()
+        setCookie = REQUEST["RESPONSE"].setCookie
+        setCookie("dtpref_rows", rows, path='/', expires=e)
+        setCookie("dtpref_cols", cols, path='/', expires=e)
+        REQUEST.other.update({"dtpref_cols":cols, "dtpref_rows":rows})
+        return self.manage_main(self, REQUEST, title=title,
+                                __str__=self.quotedHTML(data))
 
-    def manage_edit(self,data,title,SUBMIT='Change',dtpref_cols='50',
+    def manage_edit(self,data,title,SUBMIT='Change',dtpref_cols='100%',
                     dtpref_rows='20',REQUEST=None):
         """
         Replaces a Documents contents with Data, Title with Title.
@@ -263,7 +266,11 @@ class DTMLMethod(RestrictedDTML, HTML, Acquisition.Implicit, RoleManager,
         if self.wl_isLocked():
             raise ResourceLockedError, 'This DTML Method is locked via WebDAV'
 
-        if type(file) is not type(''): file=file.read()
+        if type(file) is not type(''):
+            if REQUEST and not file:
+                raise ValueError, 'No file specified'
+            file=file.read()
+
         self.munge(file)
         self.ZCacheable_invalidate()
         if REQUEST:
@@ -289,7 +296,7 @@ class DTMLMethod(RestrictedDTML, HTML, Acquisition.Implicit, RoleManager,
         raise 'Forbidden', (
             'You are not authorized to change <em>%s</em> because you '
             'do not have proxy roles.\n<!--%s, %s-->' % (self.__name__, u, roles))
-            
+
 
     def manage_proxy(self, roles=(), REQUEST=None):
         "Change Proxy Roles"
@@ -338,37 +345,48 @@ class DTMLMethod(RestrictedDTML, HTML, Acquisition.Implicit, RoleManager,
                 ))
 
 import re
-from string import find, strip
 token = "[a-zA-Z0-9!#$%&'*+\-.\\\\^_`|~]+"
 hdr_start = re.compile(r'(%s):(.*)' % token).match
 
+
 def decapitate(html, RESPONSE=None):
     headers = []
-    spos = 0
+    spos  = 0
+    eolen = 1
     while 1:
         m = hdr_start(html, spos)
         if not m:
+            if html[spos:spos+2] == '\r\n':
+                eolen = 2
+                break
             if html[spos:spos+1] == '\n':
+                eolen = 1
                 break
             return html
         header = list(m.groups())
         headers.append(header)
         spos = m.end() + 1
         while spos < len(html) and html[spos] in ' \t':
-            eol = find(html, '\n', spos)
-            if eol < 0: return html
-            header.append(strip(html[spos:eol]))
-            spos = eol + 1
+            eol = html.find('\r\n', spos)
+            if eol <> -1:
+                eolen = 2
+            else:
+                eol = html.find('\n', spos)
+                if eol < 0: return html
+                eolen = 1
+            header.append(html[spos:eol].strip())
+            spos = eol + eolen
     if RESPONSE is not None:
         for header in headers:
             hkey = header.pop(0)
             RESPONSE.setHeader(hkey, ' '.join(header).strip())
-    return html[spos + 1:]
+    return html[spos + eolen:]
+
 
 default_dm_html="""<dtml-var standard_html_header>
 <h2><dtml-var title_or_id> <dtml-var document_title></h2>
 <p>
-This is the <dtml-var document_id> Document 
+This is the <dtml-var document_id> Document
 in the <dtml-var title_and_id> Folder.
 </p>
 <dtml-var standard_html_footer>"""
@@ -392,4 +410,3 @@ def addDTMLMethod(self, id, title='', file='', REQUEST=None, submit=None):
         if submit==" Add and Edit ": u="%s/%s" % (u,quote(id))
         REQUEST.RESPONSE.redirect(u+'/manage_main')
     return ''
-
