@@ -1,23 +1,23 @@
 ##############################################################################
 #
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
-# 
+#
 ##############################################################################
 
 """Folder object
 
 Folders are the basic container objects and are analogous to directories.
 
-$Id: Folder.py,v 1.97.16.1 2002/04/01 19:53:20 andreasjung Exp $"""
+$Id: Folder.py,v 1.101 2002/08/14 21:42:56 mj Exp $"""
 
-__version__='$Revision: 1.97.16.1 $'[11:-2]
+__version__='$Revision: 1.101 $'[11:-2]
 
 import Globals, SimpleItem, ObjectManager, PropertyManager
 import AccessControl.Role, webdav.Collection, FindSupport
@@ -46,7 +46,7 @@ def manage_addFolder(self, id, title='',
     self._setObject(id, ob)
     ob=self._getOb(id)
 
-    checkPermission=getSecurityManager().checkPermission    
+    checkPermission=getSecurityManager().checkPermission
 
     if createUserF:
         if not checkPermission('Add User Folders', ob):
@@ -84,12 +84,12 @@ class Folder(
     __implements__ = (WriteLockInterface,)
     meta_type='Folder'
 
-    _properties=({'id':'title', 'type': 'string', 'mode':'w'},)
+    _properties=({'id':'title', 'type': 'string','mode':'w'},)
 
     manage_options=(
-        (ObjectManager.ObjectManager.manage_options[0],)+
+        ObjectManager.ObjectManager.manage_options+
         (
-        {'label':'View', 'action':'index_html',
+        {'label':'View', 'action':'',
          'help':('OFSP','Folder_View.stx')},
         )+
         PropertyManager.PropertyManager.manage_options+
