@@ -1,20 +1,20 @@
 ##############################################################################
 #
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
-# 
+#
 ##############################################################################
 """Image object"""
 
-__version__='$Revision: 1.134.12.1 $'[11:-2]
+__version__='$Revision: 1.141.4.7 $'[11:-2]
 
-import Globals, string, struct
+import Globals, struct
 from OFS.content_types import guess_content_type
 from Globals import DTMLFile
 from PropertyManager import PropertyManager
@@ -30,6 +30,8 @@ from DateTime import DateTime
 from Cache import Cacheable
 from mimetools import choose_boundary
 from ZPublisher import HTTPRangeSupport
+from ZPublisher.HTTPRequest import FileUpload
+from cgi import escape
 
 StringType=type('')
 manage_addFileForm=DTMLFile('dtml/imageAdd', globals(),Kind='File',kind='file')
@@ -43,7 +45,7 @@ def manage_addFile(self,id,file='',title='',precondition='', content_type='',
     title=str(title)
     content_type=str(content_type)
     precondition=str(precondition)
-    
+
     id, title = cookId(id, title, file)
 
     self=self.this()
@@ -53,7 +55,8 @@ def manage_addFile(self,id,file='',title='',precondition='', content_type='',
 
     # Now we "upload" the data.  By doing this in two steps, we
     # can use a database trick to make the upload more efficient.
-    self._getOb(id).manage_upload(file)
+    if file:
+        self._getOb(id).manage_upload(file)
     if content_type:
         self._getOb(id).content_type=content_type
 
@@ -64,11 +67,11 @@ def manage_addFile(self,id,file='',title='',precondition='', content_type='',
 class File(Persistent, Implicit, PropertyManager,
            RoleManager, Item_w__name__, Cacheable):
     """A File object is a content object for arbitrary files."""
-    
+
     __implements__ = (WriteLockInterface, HTTPRangeSupport.HTTPRangeInterface)
     meta_type='File'
 
-    
+
     precondition=''
     size=None
 
@@ -77,7 +80,7 @@ class File(Persistent, Implicit, PropertyManager,
     manage_editForm._setName('manage_editForm')
     manage=manage_main=manage_editForm
     manage_uploadForm=manage_editForm
-    
+
     manage_options=(
         (
         {'label':'Edit', 'action':'manage_main',
@@ -105,7 +108,7 @@ class File(Persistent, Implicit, PropertyManager,
         ('Delete objects',
          ('DELETE',)),
         )
-   
+
 
     _properties=({'id':'title', 'type': 'string'},
                  {'id':'content_type', 'type':'string'},
@@ -115,7 +118,7 @@ class File(Persistent, Implicit, PropertyManager,
         self.__name__=id
         self.title=title
         self.precondition=precondition
-       
+
         data, size = self._read_data(file)
         content_type=self._get_content_type(file, data, id, content_type)
         self.update_data(data, content_type, size)
@@ -130,10 +133,11 @@ class File(Persistent, Implicit, PropertyManager,
         Returns the contents of the file or image.  Also, sets the
         Content-Type HTTP header to the objects content type.
         """
+
         # HTTP If-Modified-Since header handling.
         header=REQUEST.get_header('If-Modified-Since', None)
         if header is not None:
-            header=string.split(header, ';')[0]
+            header=header.split( ';')[0]
             # Some proxies seem to send invalid date strings for this
             # header. If the date string is not valid, we ignore it
             # rather than raise an error to be generally consistent
@@ -156,12 +160,13 @@ class File(Persistent, Implicit, PropertyManager,
                     RESPONSE.setHeader('Content-Type', self.content_type)
                     RESPONSE.setHeader('Content-Length', self.size)
                     RESPONSE.setHeader('Accept-Ranges', 'bytes')
+                    self.ZCacheable_set(None)
                     RESPONSE.setStatus(304)
                     return ''
 
         if self.precondition and hasattr(self,self.precondition):
-            # Grab whatever precondition was defined and then 
-            # execute it.  The precondition will raise an exception 
+            # Grab whatever precondition was defined and then
+            # execute it.  The precondition will raise an exception
             # if something violates its terms.
             c=getattr(self,self.precondition)
             if hasattr(c,'isDocTemp') and c.isDocTemp:
@@ -192,7 +197,7 @@ class File(Persistent, Implicit, PropertyManager,
                         ranges = None
                 else:
                     # Date
-                    date = string.split(if_range, ';')[0]
+                    date = if_range.split( ';')[0]
                     try: mod_since=long(DateTime(date).timeTime())
                     except: mod_since=None
                     if mod_since is not None:
@@ -215,7 +220,7 @@ class File(Persistent, Implicit, PropertyManager,
                         break
 
                 if not satisfiable:
-                    RESPONSE.setHeader('Content-Range', 
+                    RESPONSE.setHeader('Content-Range',
                         'bytes */%d' % self.size)
                     RESPONSE.setHeader('Accept-Ranges', 'bytes')
                     RESPONSE.setHeader('Last-Modified',
@@ -225,20 +230,19 @@ class File(Persistent, Implicit, PropertyManager,
                     RESPONSE.setStatus(416)
                     return ''
 
-                # Can we optimize?
-                ranges = HTTPRangeSupport.optimizeRanges(ranges, self.size)
+                ranges = HTTPRangeSupport.expandRanges(ranges, self.size)
                                 
                 if len(ranges) == 1:
                     # Easy case, set extra header and return partial set.
                     start, end = ranges[0]
                     size = end - start
-                    
+
                     RESPONSE.setHeader('Last-Modified',
                         rfc1123_date(self._p_mtime))
                     RESPONSE.setHeader('Content-Type', self.content_type)
                     RESPONSE.setHeader('Content-Length', size)
                     RESPONSE.setHeader('Accept-Ranges', 'bytes')
-                    RESPONSE.setHeader('Content-Range', 
+                    RESPONSE.setHeader('Content-Range',
                         'bytes %d-%d/%d' % (start, end - 1, self.size))
                     RESPONSE.setStatus(206) # Partial content
 
@@ -256,11 +260,11 @@ class File(Persistent, Implicit, PropertyManager,
                             lstart = l - (pos - start)
 
                             if lstart < 0: lstart = 0
-                            
+
                             # find the endpoint
                             if end <= pos:
                                 lend = l - (pos - end)
-                                
+
                                 # Send and end transmission
                                 RESPONSE.write(data[lstart:lend])
                                 break
@@ -269,26 +273,23 @@ class File(Persistent, Implicit, PropertyManager,
                             RESPONSE.write(data[lstart:])
 
                         data = data.next
-                    
+
                     return ''
-                    
+
                 else:
-                    # When we get here, ranges have been optimized, so they are
-                    # in order, non-overlapping, and start and end values are
-                    # positive integers.
                     boundary = choose_boundary()
-                    
+
                     # Calculate the content length
                     size = (8 + len(boundary) + # End marker length
                         len(ranges) * (         # Constant lenght per set
-                            49 + len(boundary) + len(self.content_type) + 
+                            49 + len(boundary) + len(self.content_type) +
                             len('%d' % self.size)))
                     for start, end in ranges:
                         # Variable length per set
-                        size = (size + len('%d%d' % (start, end - 1)) + 
+                        size = (size + len('%d%d' % (start, end - 1)) +
                             end - start)
-                            
-                    
+
+
                     # Some clients implement an earlier draft of the spec, they
                     # will only accept x-byteranges.
                     draftprefix = (request_range is not None) and 'x-' or ''
@@ -302,8 +303,11 @@ class File(Persistent, Implicit, PropertyManager,
                             draftprefix, boundary))
                     RESPONSE.setStatus(206) # Partial content
 
-                    pos = 0
                     data = self.data
+                    # The Pdata map allows us to jump into the Pdata chain
+                    # arbitrarily during out-of-order range searching.
+                    pdata_map = {}
+                    pdata_map[0] = data
 
                     for start, end in ranges:
                         RESPONSE.write('\r\n--%s\r\n' % boundary)
@@ -311,13 +315,26 @@ class File(Persistent, Implicit, PropertyManager,
                             self.content_type)
                         RESPONSE.write(
                             'Content-Range: bytes %d-%d/%d\r\n\r\n' % (
-                                start, end - 1, self.size)) 
+                                start, end - 1, self.size))
 
                         if type(data) is StringType:
                             RESPONSE.write(data[start:end])
 
                         else:
-                            # Yippee. Linked Pdata objects.
+                            # Yippee. Linked Pdata objects. The following
+                            # calculations allow us to fast-forward through the
+                            # Pdata chain without a lot of dereferencing if we
+                            # did the work already.
+                            first_size = len(pdata_map[0].data)
+                            if start < first_size:
+                                closest_pos = 0
+                            else:
+                                closest_pos = (
+                                    ((start - first_size) >> 16 << 16) +
+                                    first_size)
+                            pos = min(closest_pos, max(pdata_map.keys()))
+                            data = pdata_map[pos]
+
                             while data is not None:
                                 l = len(data.data)
                                 pos = pos + l
@@ -326,23 +343,25 @@ class File(Persistent, Implicit, PropertyManager,
                                     lstart = l - (pos - start)
 
                                     if lstart < 0: lstart = 0
-                                    
+
                                     # find the endpoint
                                     if end <= pos:
                                         lend = l - (pos - end)
-                                        
+
                                         # Send and loop to next range
                                         RESPONSE.write(data[lstart:lend])
-                                        # Back up the position marker, it will
-                                        # be incremented again for the next
-                                        # part.
-                                        pos = pos - l
                                         break
 
                                     # Not yet at the end, transmit what we have.
                                     RESPONSE.write(data[lstart:])
 
                                 data = data.next
+                                # Store a reference to a Pdata chain link so we
+                                # don't have to deref during this request again.
+                                pdata_map[pos] = data
+
+                    # Do not keep the link references around.
+                    del pdata_map
 
                     RESPONSE.write('\r\n--%s--\r\n' % boundary)
                     return ''
@@ -357,7 +376,9 @@ class File(Persistent, Implicit, PropertyManager,
         self.ZCacheable_set(None)
 
         data=self.data
-        if type(data) is type(''): return data
+        if type(data) is type(''): 
+            RESPONSE.setBase(None)
+            return data
 
         while data is not None:
             RESPONSE.write(data.data)
@@ -418,7 +439,7 @@ class File(Persistent, Implicit, PropertyManager,
         if REQUEST:
             message="Saved changes."
             return self.manage_main(self,REQUEST,manage_tabs_message=message)
-        
+
     def _get_content_type(self, file, body, id, content_type=None):
         headers=getattr(file, 'headers', None)
         if headers and headers.has_key('content-type'):
@@ -430,13 +451,15 @@ class File(Persistent, Implicit, PropertyManager,
         return content_type
 
     def _read_data(self, file):
-        
+
         n=1 << 16
-        
+
         if type(file) is StringType:
             size=len(file)
             if size < n: return file, size
             return Pdata(file), size
+        elif isinstance(file, FileUpload) and not file:
+            raise ValueError, 'File not specified'
 
         if hasattr(file, '__class__') and file.__class__ is Pdata:
             size=len(file)
@@ -444,7 +467,7 @@ class File(Persistent, Implicit, PropertyManager,
 
         seek=file.seek
         read=file.read
-        
+
         seek(0,2)
         size=end=file.tell()
 
@@ -456,9 +479,9 @@ class File(Persistent, Implicit, PropertyManager,
         # Make sure we have an _p_jar, even if we are a new object, by
         # doing a sub-transaction commit.
         get_transaction().commit(1)
-        
+
         jar=self._p_jar
-        
+
         if jar is None:
             # Ugh
             seek(0)
@@ -474,7 +497,7 @@ class File(Persistent, Implicit, PropertyManager,
             if pos < n: pos=0 # we always want at least n bytes
             seek(pos)
             data=Pdata(read(end-pos))
-            
+
             # Woooop Woooop Woooop! This is a trick.
             # We stuff the data directly into our jar to reduce the
             # number of updates necessary.
@@ -483,17 +506,17 @@ class File(Persistent, Implicit, PropertyManager,
             # This is needed and has side benefit of getting
             # the thing registered:
             data.next=next
-            
+
             # Now make it get saved in a sub-transaction!
             get_transaction().commit(1)
 
             # Now make it a ghost to free the memory.  We
             # don't need it anymore!
             data._p_changed=None
-            
+
             next=data
             end=pos
-        
+
         return next, size
 
     def PUT(self, REQUEST, RESPONSE):
@@ -503,7 +526,7 @@ class File(Persistent, Implicit, PropertyManager,
         type=REQUEST.get_header('content-type', None)
 
         file=REQUEST['BODYFILE']
-        
+
         data, size = self._read_data(file)
         content_type=self._get_content_type(file, data, self.__name__,
                                             type or self.content_type)
@@ -511,7 +534,7 @@ class File(Persistent, Implicit, PropertyManager,
 
         RESPONSE.setStatus(204)
         return RESPONSE
-    
+
     def get_size(self):
         """Get the size of a file or image.
 
@@ -534,8 +557,10 @@ class File(Persistent, Implicit, PropertyManager,
 
     def __str__(self): return str(self.data)
     def __len__(self): return 1
-    
-    manage_FTPget=index_html
+
+    def manage_FTPget(self):
+        """Return body for ftp."""
+        return str(self.data)
 
 
 manage_addImageForm=DTMLFile('dtml/imageAdd',globals(),
@@ -559,13 +584,14 @@ def manage_addImage(self, id, file, title='', precondition='', content_type='',
 
     # First, we create the image without data:
     self._setObject(id, Image(id,title,'',content_type, precondition))
-        
+
     # Now we "upload" the data.  By doing this in two steps, we
     # can use a database trick to make the upload more efficient.
-    self._getOb(id).manage_upload(file)
+    if file:
+        self._getOb(id).manage_upload(file)
     if content_type:
         self._getOb(id).content_type=content_type
-    
+
     if REQUEST is not None:
         try:    url=self.DestinationURL()
         except: url=REQUEST['URL1']
@@ -580,7 +606,7 @@ def getImageInfo(data):
     width = -1
     content_type = ''
 
-    # handle GIFs   
+    # handle GIFs
     if (size >= 10) and data[:6] in ('GIF87a', 'GIF89a'):
         # Check to see if content_type is correct
         content_type = 'image/gif'
@@ -597,7 +623,7 @@ def getImageInfo(data):
         w, h = struct.unpack(">LL", data[16:24])
         width = int(w)
         height = int(h)
-            
+
     # Maybe this is for an older PNG version.
     elif (size >= 16) and (data[:8] == '\211PNG\r\n\032\n'):
         # Check to see if we have the right content type
@@ -638,7 +664,7 @@ class Image(File):
     __implements__ = (WriteLockInterface,)
     meta_type='Image'
 
-    
+
     height=''
     width=''
 
@@ -658,7 +684,7 @@ class Image(File):
 
     _properties=({'id':'title', 'type': 'string'},
                  {'id':'content_type', 'type':'string','mode':'w'},
-                 {'id':'height', 'type':'string' },
+                 {'id':'height', 'type':'string'},
                  {'id':'width', 'type':'string'},
                  )
 
@@ -679,7 +705,7 @@ class Image(File):
     manage_editForm._setName('manage_editForm')
     manage=manage_main=manage_editForm
     manage_uploadForm=manage_editForm
-    
+
     # private
     update_data__roles__=()
     def update_data(self, data, content_type=None, size=None):
@@ -705,7 +731,7 @@ class Image(File):
         return self.tag()
 
     def tag(self, height=None, width=None, alt=None,
-            scale=0, xscale=0, yscale=0, css_class=None, **args):
+            scale=0, xscale=0, yscale=0, css_class=None, title=None, **args):
         """
         Generate an HTML IMG tag for this image, with customization.
         Arguments to self.tag() can be any valid attributes of an IMG tag.
@@ -737,7 +763,11 @@ class Image(File):
 
         if alt is None:
             alt=getattr(self, 'title', '')
-        result = '%s alt="%s"' % (result, alt)
+        result = '%s alt="%s"' % (result, escape(alt, 1))
+
+        if title is None:
+            title=getattr(self, 'title', '')
+        result = '%s title="%s"' % (result, escape(title, 1))
 
         if height:
             result = '%s height="%s"' % (result, height)
@@ -745,7 +775,7 @@ class Image(File):
         if width:
             result = '%s width="%s"' % (result, width)
 
-        if not 'border' in map(string.lower, args.keys()):
+        if not 'border' in [ x.lower() for x in  args.keys()]:
             result = '%s border="0"' % result
 
         if css_class is not None:
@@ -762,17 +792,17 @@ def cookId(id, title, file):
     if not id and hasattr(file,'filename'):
         filename=file.filename
         title=title or filename
-        id=filename[max(string.rfind(filename, '/'),
-                        string.rfind(filename, '\\'),
-                        string.rfind(filename, ':'),
-                        )+1:]                  
+        id=filename[max(filename.rfind('/'),
+                        filename.rfind('\\'),
+                        filename.rfind(':'),
+                        )+1:]
     return id, title
 
 class Pdata(Persistent, Implicit):
     # Wrapper for possibly large data
 
     next=None
-    
+
     def __init__(self, data):
         self.data=data
 
@@ -792,14 +822,5 @@ class Pdata(Persistent, Implicit):
             self=next
             r.append(self.data)
             next=self.next
-        
-        return string.join(r,'')
-
-
-
-
-
-
-
-
 
+        return ''.join(r)
