@@ -1,37 +1,42 @@
 ##############################################################################
 #
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
-# 
+#
 ##############################################################################
 __doc__="""Object Manager
 
-$Id: ObjectManager.py,v 1.145.16.3 2002/04/15 22:37:49 matt Exp $"""
+$Id: ObjectManager.py,v 1.158.6.1 2003/11/02 18:14:44 efge Exp $"""
 
-__version__='$Revision: 1.145.16.3 $'[11:-2]
+__version__='$Revision: 1.158.6.1 $'[11:-2]
 
 import App.Management, Acquisition, Globals, CopySupport, Products
 import os, App.FactoryDispatcher, re, Products
 from OFS.Traversable import Traversable
+from OFS import SimpleItem
 from Globals import DTMLFile, Persistent
 from Globals import MessageDialog, default__class_init__
 from Globals import REPLACEABLE, NOT_REPLACEABLE, UNIQUE
 from webdav.NullResource import NullResource
 from webdav.Collection import Collection
 from Acquisition import aq_base
+from AccessControl.SecurityInfo import ClassSecurityInfo
+from ZODB.POSException import ConflictError
 from urllib import quote
 from cStringIO import StringIO
 import marshal
 import App.Common
 from AccessControl import getSecurityManager
 from zLOG import LOG, ERROR
-import sys,string,fnmatch,copy
+import sys,fnmatch,copy
+from cgi import escape
+from types import StringType, UnicodeType
 
 import XMLExportImport
 customImporters={
@@ -49,11 +54,12 @@ def checkValidId(self, id, allow_dup=0):
     # check_valid_id() will be called again later with allow_dup
     # set to false before the object is added.
 
-    if not id or (type(id) != type('')):
-        raise BadRequestException, 'Empty or invalid id specified.'
+    if not id or not isinstance(id, StringType):
+        if isinstance(id, UnicodeType): id = escape(id)
+        raise BadRequestException, ('Empty or invalid id specified', id)
     if bad_id(id) is not None:
         raise BadRequestException, (
-            'The id "%s" contains characters illegal in URLs.' % id)
+            'The id "%s" contains characters illegal in URLs.' % escape(id))
     if id[0]=='_': raise BadRequestException, (
         'The id "%s" is invalid - it begins with an underscore.'  % id)
     if id[:3]=='aq_': raise BadRequestException, (
@@ -82,7 +88,6 @@ def checkValidId(self, id, allow_dup=0):
             'The id "%s" contains characters illegal in URLs.' % id
             )
 
-
 class BeforeDeleteException( Exception ): pass # raise to veto deletion
 class BreakoutException ( Exception ): pass  # raised to break out of loops
 
@@ -98,7 +103,7 @@ class ObjectManager(
     ):
     """Generic object manager
 
-    This class provides core behavior for collections of heterogeneous objects. 
+    This class provides core behavior for collections of heterogeneous objects.
     """
 
     __ac_permissions__=(
@@ -119,7 +124,7 @@ class ObjectManager(
     meta_type  ='Object Manager'
 
     meta_types=() # Sub-object types that are specific to this object
-    
+
     _objects   =()
 
     manage_main=DTMLFile('dtml/main', globals())
@@ -128,8 +133,6 @@ class ObjectManager(
     manage_options=(
         {'label':'Contents', 'action':'manage_main',
          'help':('OFSP','ObjectManager_Contents.stx')},
-#        {'label':'Import/Export', 'action':'manage_importExportForm',
-#         'help':('OFSP','ObjectManager_Import-Export.stx')},         
         )
 
     isAnObjectManager=1
@@ -146,60 +149,66 @@ class ObjectManager(
             except: pass
         mt.sort()
         self.meta_types=tuple(mt)
-        
+
         default__class_init__(self)
 
     def all_meta_types(self, interfaces=None):
+        # A list of products registered elsewhere
+        external_candidates = []
+
+        # Look at _product_meta_types, if there is one
         _pmt=()
         if hasattr(self, '_product_meta_types'): _pmt=self._product_meta_types
         elif hasattr(self, 'aq_acquire'):
             try: _pmt=self.aq_acquire('_product_meta_types')
             except:  pass
+        external_candidates.extend(list(_pmt))
 
-        if interfaces is None:  pmt = list(_pmt)
+        # Look at all globally visible meta types.
+        for entry in Products.meta_types:
+            if ( (interfaces is not None) or (entry.get("visibility", None)=="Global") ):
+                external_candidates.append(entry)
+
+        # Filter the list of external candidates based on the
+        # specified interface constraint
+        if interfaces is None:
+            interface_constrained_meta_types = external_candidates
         else:
-            pmt = []
-
-            for entry in pmt:
+            interface_constrained_meta_types = []
+            for entry in external_candidates:
                 try:
                     eil = entry.get('interfaces',None)
-
                     if eil is not None:
                         for ei in eil:
-                            for i in interfaces: 
+                            for i in interfaces:
                                 if ei is i or ei.extends(i):
-                                    pmt.append(entry) 
+                                    interface_constrained_meta_types.append(entry)
                                     raise BreakoutException # only append 1ce
-                except BreakoutException:   
+                except BreakoutException:
                     pass
-            
-        gmt = []
 
-        for entry in Products.meta_types:
+        # Meta types specified by this instance are not checked against the
+        # interface constraint. This is as it always has been, but Im not
+        # sure it is correct.
+        interface_constrained_meta_types.extend(list(self.meta_types))
 
-            if interfaces is None:
-                if entry.get("visibility", None) == "Global":
-                    gmt.append(entry)
+        # Filter the list based on each meta-types's container_filter
+        meta_types = []
+        for entry in interface_constrained_meta_types:
+            container_filter = entry.get('container_filter',None)
+            if container_filter is None:
+                meta_types.append(entry)
             else:
-                try:
-                    eil = entry.get("interfaces", None)
-                    if eil is not None:
-                        for ei in eil:
-                            for i in interfaces: 
-                                if ei is i or ei.extends(i):
-                                    gmt.append(entry) 
-                                    raise BreakoutException # only append 1ce
-                except BreakoutException:   
-                    pass
+                if container_filter(self):
+                    meta_types.append(entry)
 
-        return list(self.meta_types)+gmt+pmt
+        return meta_types
 
     def _subobject_permissions(self):
         return (Products.__ac_permissions__+
                 self.aq_acquire('_getProductRegistryData')('ac_permissions')
                 )
 
-
     def filtered_meta_types(self, user=None):
         # Return a list of the types for which the user has
         # adequate permission to add that type of object.
@@ -245,9 +254,6 @@ class ObjectManager(
                 break
 
         self._objects=self._objects+({'id':id,'meta_type':t},)
-        # Prepare the _p_jar attribute immediately. _getCopy() may need it.
-        if hasattr(object, '_p_jar') and object._p_jar is None:
-            object._p_jar = self._p_jar
         self._setOb(id,object)
         object=self._getOb(id)
 
@@ -292,6 +298,8 @@ class ObjectManager(
                     object.manage_beforeDelete(item, container)
             except BeforeDeleteException, ob:
                 raise
+            except ConflictError:
+                raise
             except:
                 LOG('Zope',ERROR,'manage_beforeDelete() threw',
                     error=sys.exc_info())
@@ -304,6 +312,8 @@ class ObjectManager(
             object.manage_beforeDelete(object, self)
         except BeforeDeleteException, ob:
             raise
+        except ConflictError:
+            raise
         except:
             LOG('Zope',ERROR,'manage_beforeDelete() threw',
                 error=sys.exc_info())
@@ -311,7 +321,7 @@ class ObjectManager(
         self._objects=tuple(filter(lambda i,n=id: i['id']!=n, self._objects))
         self._delOb(id)
 
-        # Indicate to the object that it has been deleted. This is 
+        # Indicate to the object that it has been deleted. This is
         # necessary for object DB mount points. Note that we have to
         # tolerate failure here because the object being deleted could
         # be a Broken object, and it is not possible to set attributes
@@ -405,7 +415,7 @@ class ObjectManager(
                             vals.append(get(id))
                             seen[physicalPath]=1
                     except: pass
-                    
+
             if hasattr(obj,'aq_parent'):
                 obj=obj.aq_parent
                 relativePhysicalPath = ('..',) + relativePhysicalPath
@@ -419,7 +429,7 @@ class ObjectManager(
 
     def manage_delObjects(self, ids=[], REQUEST=None):
         """Delete a subordinate object
-        
+
         The objects specified in 'ids' get deleted.
         """
         if type(ids) is type(''): ids=[ids]
@@ -432,17 +442,17 @@ class ObjectManager(
         for n in ids:
             if n in p:
                 return MessageDialog(title='Not Deletable',
-                       message='<EM>%s</EM> cannot be deleted.' % n,
+                       message='<EM>%s</EM> cannot be deleted.' % escape(n),
                        action ='./manage_main',)
         while ids:
             id=ids[-1]
             v=self._getOb(id, self)
             if v is self:
-                raise 'BadRequest', '%s does not exist' % ids[-1]
+                raise 'BadRequest', '%s does not exist' % escape(ids[-1])
             self._delObject(id)
             del ids[-1]
         if REQUEST is not None:
-                return self.manage_main(self, REQUEST, update_menu=1)
+            return self.manage_main(self, REQUEST, update_menu=1)
 
 
     def tpValues(self):
@@ -470,7 +480,7 @@ class ObjectManager(
 
     def manage_exportObject(self, id='', download=None, toxml=None,
                             RESPONSE=None,REQUEST=None):
-        """Exports an object to a file and returns that file."""        
+        """Exports an object to a file and returns that file."""
         if not id:
             # can't use getId() here (breaks on "old" exported objects)
             id=self.id
@@ -479,7 +489,7 @@ class ObjectManager(
         else: ob=self._getOb(id)
 
         suffix=toxml and 'xml' or 'zexp'
-        
+
         if download:
             f=StringIO()
             if toxml: XMLExportImport.exportXML(ob._p_jar, ob._p_oid, f)
@@ -497,7 +507,7 @@ class ObjectManager(
             ob._p_jar.exportFile(ob._p_oid, f)
 
         if REQUEST is not None:
-            return self.manage_main(self, REQUEST, 
+            return self.manage_main(self, REQUEST,
                 manage_tabs_message=
                 '<em>%s</em> sucessfully exported to <em>%s</em>' % (id,f),
                 title = 'Object exported')
@@ -509,24 +519,23 @@ class ObjectManager(
         """Import an object from a file"""
         dirname, file=os.path.split(file)
         if dirname:
-            raise BadRequestException, 'Invalid file name %s' % file
+            raise BadRequestException, 'Invalid file name %s' % escape(file)
 
         instance_home = INSTANCE_HOME
-        software_home = os.path.join(SOFTWARE_HOME, '..%s..' % os.sep)
-        software_home = os.path.normpath(software_home)
-        
-        for impath in (instance_home, software_home):
+        zope_home = ZOPE_HOME
+
+        for impath in (instance_home, zope_home):
             filepath = os.path.join(impath, 'import', file)
             if os.path.exists(filepath):
                 break
         else:
-            raise BadRequestException, 'File does not exist: %s' % file
+            raise BadRequestException, 'File does not exist: %s' % escape(file)
 
         self._importObjectFromFile(filepath, verify=not not REQUEST,
                                    set_owner=set_owner)
-        
+
         if REQUEST is not None:
-            return self.manage_main(self, REQUEST, 
+            return self.manage_main(self, REQUEST,
                 manage_tabs_message='<em>%s</em> sucessfully imported' % id,
                 title = 'Object imported',
                 update_menu=1)
@@ -552,7 +561,7 @@ class ObjectManager(
         ob.manage_changeOwnershipType(explicit=0)
 
     # FTP support methods
-    
+
     def manage_FTPlist(self, REQUEST):
         "Directory listing for FTP"
         out=()
@@ -565,7 +574,7 @@ class ObjectManager(
             if not hasattr(ob,'aq_parent'):
                 break
             ob=ob.aq_parent
-        
+
         files=self.objectItems()
 
         # recursive ride through all subfolders (ls -R) (ajung)
@@ -578,7 +587,7 @@ class ObjectManager(
                     all_files.extend(findChilds(f[1]))
                 else:
                     all_files.append(f)
-            
+
             files = all_files
 
         try:
@@ -588,7 +597,7 @@ class ObjectManager(
             files.sort()
 
         # Perform globbing on list of files (ajung)
-           
+
         globbing = REQUEST.environ.get('GLOBBING','')
         if globbing :
             files = filter(lambda x,g=globbing: fnmatch.fnmatch(x[0],g) , files)
@@ -598,7 +607,7 @@ class ObjectManager(
         except AttributeError:
             files=list(files)
             files.sort()
-            
+
         if not (hasattr(self,'isTopLevelPrincipiaApplicationObject') and
                 self.isTopLevelPrincipiaApplicationObject):
             files.insert(0,('..',self.aq_parent))
@@ -610,7 +619,7 @@ class ObjectManager(
             except: stat=None
             if stat is not None:
                 out=out+((k,stat),)
-        return marshal.dumps(out)   
+        return marshal.dumps(out)
 
     def manage_FTPstat(self,REQUEST):
         "Psuedo stat used for FTP listings"
@@ -647,7 +656,6 @@ class ObjectManager(
                 return NullResource(self, key, request).__of__(self)
         raise KeyError, key
 
-
 def findChilds(obj,dirname=''):
     """ recursive walk through the object hierarchy to
     find all childs of an object (ajung)
