@@ -1,5 +1,5 @@
 ##############################################################################
-# 
+#
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -8,21 +8,21 @@
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
-# 
+#
 ##############################################################################
 
 """Property management"""
-__version__='$Revision: 1.42 $'[11:-2]
+__version__='$Revision: 1.46.6.8 $'[11:-2]
 
 import ExtensionClass, Globals
 import ZDOM
 from PropertySheets import DefaultPropertySheets, vps
 from ZPublisher.Converters import type_converters
 from Globals import DTMLFile, MessageDialog
-from string import find,join,lower,split
 from Acquisition import Implicit, aq_base
 from Globals import Persistent
-
+from cgi import escape
+from types import ListType
 
 
 class PropertyManager(ExtensionClass.Base, ZDOM.ElementWithAttributes):
@@ -90,9 +90,9 @@ class PropertyManager(ExtensionClass.Base, ZDOM.ElementWithAttributes):
 
     manage_options=(
         {'label':'Properties', 'action':'manage_propertiesForm',
-         'help':('OFSP','Properties.stx')},         
+         'help':('OFSP','Properties.stx')},
         )
-    
+
     manage_propertiesForm=DTMLFile('dtml/properties', globals(),
                                    property_extensible_schema__=1)
     manage_propertyTypeForm=DTMLFile('dtml/propertyType', globals())
@@ -122,7 +122,7 @@ class PropertyManager(ExtensionClass.Base, ZDOM.ElementWithAttributes):
 
     def valid_property_id(self, id):
         if not id or id[:1]=='_' or (id[:3]=='aq_') \
-           or (' ' in id) or hasattr(aq_base(self), id):
+           or (' ' in id) or hasattr(aq_base(self), id) or escape(id) != id:
             return 0
         return 1
 
@@ -134,7 +134,7 @@ class PropertyManager(ExtensionClass.Base, ZDOM.ElementWithAttributes):
         return 0
 
     def getProperty(self, id, d=None):
-        """Get the property 'id', returning the optional second 
+        """Get the property 'id', returning the optional second
            argument or None if no such property is found."""
         if self.hasProperty(id):
             return getattr(self, id)
@@ -156,6 +156,8 @@ class PropertyManager(ExtensionClass.Base, ZDOM.ElementWithAttributes):
 
     def _setPropValue(self, id, value):
         self._wrapperCheck(value)
+        if type(value) == ListType:
+            value = tuple(value)
         setattr(self,id,value)
 
     def _delPropValue(self, id):
@@ -165,7 +167,7 @@ class PropertyManager(ExtensionClass.Base, ZDOM.ElementWithAttributes):
         # for selection and multiple selection properties
         # the value argument indicates the select variable
         # of the property
-        
+
         self._wrapperCheck(value)
         if not self.valid_property_id(id):
             raise 'Bad Request', 'Invalid or duplicate property id'
@@ -189,7 +191,7 @@ class PropertyManager(ExtensionClass.Base, ZDOM.ElementWithAttributes):
         # the value to the type of the existing property.
         self._wrapperCheck(value)
         if not self.hasProperty(id):
-            raise 'Bad Request', 'The property %s does not exist' % id
+            raise 'Bad Request', 'The property %s does not exist' % escape(id)
         if type(value)==type(''):
             proptype=self.getPropertyType(id) or 'string'
             if type_converters.has_key(proptype):
@@ -198,8 +200,8 @@ class PropertyManager(ExtensionClass.Base, ZDOM.ElementWithAttributes):
 
     def _delProperty(self, id):
         if not self.hasProperty(id):
-            raise ValueError, 'The property %s does not exist' % id
-        delattr(self,id)
+            raise ValueError, 'The property %s does not exist' % escape(id)
+        self._delPropValue(id)
         self._properties=tuple(filter(lambda i, n=id: i['id'] != n,
                                       self._properties))
 
@@ -213,7 +215,7 @@ class PropertyManager(ExtensionClass.Base, ZDOM.ElementWithAttributes):
 
     def propertyItems(self):
         """Return a list of (id,property) tuples """
-        return map(lambda i,s=self: (i['id'],getattr(s,i['id'])), 
+        return map(lambda i,s=self: (i['id'],getattr(s,i['id'])),
                                     self._properties)
     def _propertyMap(self):
         """Return a tuple of mappings, giving meta-data for properties """
@@ -259,7 +261,10 @@ class PropertyManager(ExtensionClass.Base, ZDOM.ElementWithAttributes):
         for prop in self._propertyMap():
             name=prop['id']
             if 'w' in prop.get('mode', 'wd'):
-                value=REQUEST.get(name, '')
+                if prop['type'] == 'multiple selection':
+                    value=REQUEST.get(name, [])
+                else:
+                    value=REQUEST.get(name, '')
                 self._updateProperty(name, value)
         if REQUEST:
             message="Saved changes."
@@ -282,14 +287,15 @@ class PropertyManager(ExtensionClass.Base, ZDOM.ElementWithAttributes):
         for name, value in props.items():
             if self.hasProperty(name):
                 if not 'w' in propdict[name].get('mode', 'wd'):
-                    raise 'BadRequest', '%s cannot be changed' % name
+                    raise 'BadRequest', '%s cannot be changed' % escape(name)
                 self._updateProperty(name, value)
         if REQUEST:
             message="Saved changes."
-            return self.manage_propertiesForm(self,REQUEST,manage_tabs_message=message)
+            return self.manage_propertiesForm(self,REQUEST,
+                                              manage_tabs_message=message)
 
     # Note - this is experimental, pending some community input.
-    
+
     def manage_changePropertyTypes(self, old_ids, props, REQUEST=None):
         """Replace one set of properties with another
 
@@ -307,10 +313,14 @@ class PropertyManager(ExtensionClass.Base, ZDOM.ElementWithAttributes):
             self._setProperty(prop.new_id, prop.new_value, prop.new_type)
         if REQUEST is not None:
             return self.manage_propertiesForm(self, REQUEST)
-            
+
 
     def manage_delProperties(self, ids=None, REQUEST=None):
         """Delete one or more properties specified by 'ids'."""
+        if REQUEST:
+            # Bugfix for property named "ids" (Casey)
+            if ids == self.getProperty('ids', None): ids = None
+            ids = REQUEST.get('_ids', ids)
         if ids is None:
             return MessageDialog(
                    title='No property specified',
@@ -321,11 +331,11 @@ class PropertyManager(ExtensionClass.Base, ZDOM.ElementWithAttributes):
         for id in ids:
             if not hasattr(aq_base(self), id):
                 raise 'BadRequest', (
-                      'The property <em>%s</em> does not exist' % id)
+                      'The property <em>%s</em> does not exist' % escape(id))
             if (not 'd' in propdict[id].get('mode', 'wd')) or (id in nd):
                 return MessageDialog(
                 title  ='Cannot delete %s' % id,
-                message='The property <em>%s</em> cannot be deleted.' % id,
+                message='The property <em>%s</em> cannot be deleted.' % escape(id),
                 action ='manage_propertiesForm')
             self._delProperty(id)
 
