@@ -1,24 +1,24 @@
 ##############################################################################
 #
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
-# 
+#
 ##############################################################################
 
 """Property sheets"""
-__version__='$Revision: 1.81.4.1 $'[11:-2]
 
-import time, string, App.Management, Globals
+__version__='$Revision: 1.87.6.6 $'[11:-2]
+
+import time,  App.Management, Globals, sys
 from webdav.WriteLockInterface import WriteLockInterface
 from ZPublisher.Converters import type_converters
 from Globals import DTMLFile, MessageDialog
-from string import find,join,lower,split,rfind
 from Acquisition import Implicit, Explicit
 from App.Common import rfc1123_date, iso8601_date
 from webdav.common import urlbase
@@ -27,6 +27,13 @@ from Globals import Persistent
 from Traversable import Traversable
 from Acquisition import aq_base
 from AccessControl import getSecurityManager
+from webdav.common import isDavCollection
+from cgi import escape
+from types import ListType
+
+# DM: we would like to import this from somewhere
+BadRequestException= 'Bad Request'
+
 
 class View(App.Management.Tabs, Base):
     """A view of an object, typically used for management purposes
@@ -41,7 +48,7 @@ class View(App.Management.Tabs, Base):
         RESPONSE.redirect(URL1+'/manage')
 
     def tpURL(self): return self.getId()
-        
+
     def manage_options(self):
         """Return a manage option data structure for me instance
         """
@@ -52,11 +59,11 @@ class View(App.Management.Tabs, Base):
         else:
             pre=r['URL']
             for i in (1,2,3):
-                l=rfind(pre,'/')
+                l=pre.rfind('/')
                 if l >= 0:
                     pre=pre[:l]
             pre=pre+'/'
-            
+
         r=[]
         for d in self.aq_parent.aq_parent.manage_options:
             path=d['action']
@@ -70,10 +77,10 @@ class View(App.Management.Tabs, Base):
         return r
 
     def tabs_path_info(self, script, path):
-        l=rfind(path,'/')
+        l=path.rfind('/')
         if l >= 0:
             path=path[:l]
-            l=rfind(path,'/')
+            l=path.rfind('/')
             if l >= 0: path=path[:l]
         return View.inheritedAttribute('tabs_path_info')(
             self, script, path)
@@ -107,7 +114,7 @@ class PropertySheet(Traversable, Persistent, Implicit):
          ),
         )
 
-    __reserved_ids= ('values','items') 
+    __reserved_ids= ('values','items')
 
     def property_extensible_schema__(self):
         """Return a flag indicating whether new properties may be
@@ -120,8 +127,10 @@ class PropertySheet(Traversable, Persistent, Implicit):
         # space identifier.
 
         if id in self.__reserved_ids:
-            raise ValueError, "'%s' is a reserved Id (forbidden Ids are: %s)" % \
-                    (id, self.__reserved_ids)
+            raise ValueError(
+                  "'%s' is a reserved Id (forbidden Ids are: %s) " % (
+                  id, self.__reserved_ids
+                  ))
 
         self.id=id
         self._md=md or {}
@@ -142,7 +151,7 @@ class PropertySheet(Traversable, Persistent, Implicit):
 
     def valid_property_id(self, id):
         if not id or id[:1]=='_' or (id[:3]=='aq_') \
-           or (' ' in id):
+           or (' ' in id) or escape(id) != id:
             return 0
         return 1
 
@@ -181,7 +190,7 @@ class PropertySheet(Traversable, Persistent, Implicit):
         # systems.
         self._wrapperCheck(value)
         if not self.valid_property_id(id):
-            raise 'Bad Request', 'Invalid property id, %s.' % id
+            raise 'Bad Request', 'Invalid property id, %s.' % escape(id)
 
         if not self.property_extensible_schema__():
             raise 'Bad Request', (
@@ -191,7 +200,8 @@ class PropertySheet(Traversable, Persistent, Implicit):
         if hasattr(aq_base(self),id):
             if not (id=='title' and not self.__dict__.has_key(id)):
                 raise 'Bad Request', (
-                    'Invalid property id, <em>%s</em>. It is in use.' % id)
+                    'Invalid property id, <em>%s</em>. It is in use.' %
+                        escape(id))
         if meta is None: meta={}
         prop={'id':id, 'type':type, 'meta':meta}
         pself._properties=pself._properties+(prop,)
@@ -203,6 +213,10 @@ class PropertySheet(Traversable, Persistent, Implicit):
             prop['select_variable']=value
             if type=='selection': value=None
             else: value=[]
+
+        # bleah - can't change kw name in api, so use ugly workaround.
+        if sys.modules['__builtin__'].type(value) == ListType:
+            value = tuple(value)
         setattr(self, id, value)
 
     def _updateProperty(self, id, value, meta=None):
@@ -212,10 +226,10 @@ class PropertySheet(Traversable, Persistent, Implicit):
         # it will used to _replace_ the properties meta data.
         self._wrapperCheck(value)
         if not self.hasProperty(id):
-            raise 'Bad Request', 'The property %s does not exist.' % id
+            raise 'Bad Request', 'The property %s does not exist.' % escape(id)
         propinfo=self.propertyInfo(id)
         if not 'w' in propinfo.get('mode', 'wd'):
-            raise 'Bad Request', '%s cannot be changed.' % id
+            raise 'Bad Request', '%s cannot be changed.' % escape(id)
         if type(value)==type(''):
             proptype=propinfo.get('type', 'string')
             if type_converters.has_key(proptype):
@@ -227,19 +241,23 @@ class PropertySheet(Traversable, Persistent, Implicit):
                 if prop['id']==id: prop['meta']=meta
                 props.append(prop)
             pself._properties=tuple(props)
+
+        # bleah - can't change kw name in api, so use ugly workaround.
+        if type(value) == ListType:
+            value = tuple(value)
         setattr(self.v_self(), id, value)
 
     def _delProperty(self, id):
         # Delete the property with the given id. If a property with the
         # given id does not exist, a ValueError is raised.
         if not self.hasProperty(id):
-            raise 'Bad Request', 'The property %s does not exist.' % id
+            raise 'Bad Request', 'The property %s does not exist.' % escape(id)
         vself=self.v_self()
         if hasattr(vself, '_reserved_names'):
             nd=vself._reserved_names
         else: nd=()
         if (not 'd' in self.propertyInfo(id).get('mode', 'wd')) or (id in nd):
-            raise 'Bad Request', '%s cannot be deleted.' % id
+            raise 'Bad Request', '%s cannot be deleted.' % escape(id)
         delattr(vself, id)
         pself=self.p_self()
         pself._properties=tuple(filter(lambda i, n=id: i['id'] != n,
@@ -256,14 +274,14 @@ class PropertySheet(Traversable, Persistent, Implicit):
 
     def propertyItems(self):
         # Return a list of (id, property) tuples.
-        return map(lambda i, s=self: (i['id'], s.getProperty(i['id'])), 
+        return map(lambda i, s=self: (i['id'], s.getProperty(i['id'])),
                    self._propertyMap())
 
     def propertyInfo(self, id):
         # Return a mapping containing property meta-data
         for p in self._propertyMap():
             if p['id']==id: return p
-        raise ValueError, 'The property %s does not exist.' % id
+        raise ValueError, 'The property %s does not exist.' % escape(id)
 
     def _propertyMap(self):
         # Return a tuple of mappings, giving meta-data for properties.
@@ -292,7 +310,7 @@ class PropertySheet(Traversable, Persistent, Implicit):
              '  %s\n' \
              '  </d:responsedescription>\n'
 
-    def dav__allprop(self, propstat=propstat, join=string.join):
+    def dav__allprop(self, propstat=propstat ):
         # DAV helper method - return one or more propstat elements
         # indicating property names and values for all properties.
         result=[]
@@ -301,17 +319,17 @@ class PropertySheet(Traversable, Persistent, Implicit):
             value=self.getProperty(name)
 
             if type=='tokens':
-                value=join(str(value), ' ')
+                value=' '.join(map(str, value))
             elif type=='lines':
-                value=join(str(value), '\n')
+                value='\n'.join(map(str, value))
             # check for xml property
             attrs=item.get('meta', {}).get('__xml_attrs__', None)
             if attrs is not None:
                 attrs=map(lambda n: ' %s="%s"' % n, attrs.items())
-                attrs=join(attrs, '')
+                attrs=''.join(attrs)
             else:
                 # Quote non-xml items here?
-                attrs='' 
+                attrs=''
 
             if hasattr(self,"dav__"+name):
                 prop='  <n:%s%s>%s</n:%s>' % (name, attrs, value, name)
@@ -320,24 +338,23 @@ class PropertySheet(Traversable, Persistent, Implicit):
 
             result.append(prop)
         if not result: return ''
-        result=join(result, '\n')
-        
+        result='\n'.join(result)
+
         return propstat % (self.xml_namespace(), result, '200 OK', '')
 
-    def dav__propnames(self, propstat=propstat, join=string.join):
+    def dav__propnames(self, propstat=propstat):
         # DAV helper method - return a propstat element indicating
         # property names for all properties in this PropertySheet.
         result=[]
         for name in self.propertyIds():
             result.append('  <n:%s/>' % name)
         if not result: return ''
-        result=join(result, '\n')
+        result='\n'.join(result)
         return propstat % (self.xml_namespace(), result, '200 OK', '')
 
 
     def dav__propstat(self, name, result,
-                      propstat=propstat, propdesc=propdesc,
-                      join=string.join):
+                      propstat=propstat, propdesc=propdesc):
         # DAV helper method - return a propstat element indicating
         # property name and value for the requested property.
         xml_id=self.xml_namespace()
@@ -354,14 +371,14 @@ class PropertySheet(Traversable, Persistent, Implicit):
             name, type=item['id'], item.get('type','string')
             value=self.getProperty(name)
             if type=='tokens':
-                value=join(str(value), ' ')
+                value=' '.join(map(str, value))
             elif type=='lines':
-                value=join(str(value), '\n')
+                value='\n'.join(map(str, value))
             # allow for xml properties
             attrs=item.get('meta', {}).get('__xml_attrs__', None)
             if attrs is not None:
                 attrs=map(lambda n: ' %s="%s"' % n, attrs.items())
-                attrs=join(attrs, '')
+                attrs=''.join(attrs)
             else:
                 # quote non-xml items here?
                 attrs=''
@@ -371,7 +388,7 @@ class PropertySheet(Traversable, Persistent, Implicit):
             if not result.has_key(code):
                 result[code]=[prop]
             else: result[code].append(prop)
-            return            
+            return
 
 
     del propstat
@@ -379,7 +396,7 @@ class PropertySheet(Traversable, Persistent, Implicit):
 
 
     # Web interface
-    
+
     manage=DTMLFile('dtml/properties', globals())
     def manage_propertiesForm(self, URL1):
         " "
@@ -420,7 +437,7 @@ class PropertySheet(Traversable, Persistent, Implicit):
         for name, value in props.items():
             if self.hasProperty(name):
                 if not 'w' in propdict[name].get('mode', 'wd'):
-                    raise 'BadRequest', '%s cannot be changed' % name
+                    raise 'BadRequest', '%s cannot be changed' % escape(name)
                 self._updateProperty(name, value)
         if REQUEST is not None:
             return MessageDialog(
@@ -430,6 +447,10 @@ class PropertySheet(Traversable, Persistent, Implicit):
 
     def manage_delProperties(self, ids=None, REQUEST=None):
         """Delete one or more properties specified by 'ids'."""
+        if REQUEST:
+            # Bugfix for properties named "ids" (casey)
+            if ids == self.getProperty('ids', None): ids = None
+            ids = REQUEST.get('_ids', ids)
         if ids is None:
             return MessageDialog(
                    title='No property specified',
@@ -448,7 +469,7 @@ class Virtual:
 
     def __init__(self):
         pass
-    
+
     def v_self(self):
         return self.aq_parent.aq_parent
 
@@ -477,7 +498,7 @@ class DAVProperties(Virtual, PropertySheet, View):
         {'id':'supportedlock',    'mode':'r'},
         {'id':'lockdiscovery',    'mode':'r'},
         )
-     
+
     def getProperty(self, id, default=None):
         method='dav__%s' % id
         if not hasattr(self, method):
@@ -485,23 +506,23 @@ class DAVProperties(Virtual, PropertySheet, View):
         return getattr(self, method)()
 
     def _setProperty(self, id, value, type='string', meta=None):
-        raise ValueError, '%s cannot be set.' % id
+        raise ValueError, '%s cannot be set.' % escape(id)
 
     def _updateProperty(self, id, value):
-        raise ValueError, '%s cannot be updated.' % id
+        raise ValueError, '%s cannot be updated.' % escape(id)
 
     def _delProperty(self, id):
-        raise ValueError, '%s cannot be deleted.' % id
+        raise ValueError, '%s cannot be deleted.' % escape(id)
 
     def _propertyMap(self):
         # Only use getlastmodified if returns a value
         if hasattr(self.v_self(), '_p_mtime'):
             return self.pm + ({'id':'getlastmodified',  'mode':'r'},)
         return self.pm
-    
+
     def propertyMap(self):
         return map(lambda dict: dict.copy(), self._propertyMap())
-    
+
     def dav__creationdate(self):
         return iso8601_date(43200.0)
 
@@ -510,7 +531,7 @@ class DAVProperties(Virtual, PropertySheet, View):
 
     def dav__resourcetype(self):
         vself=self.v_self()
-        if (hasattr(vself, '__dav_collection__') or
+        if (isDavCollection(vself) or
             getattr(aq_base(vself), 'isAnObjectManager', None)):
             return '<n:collection/>'
         return ''
@@ -550,8 +571,8 @@ class DAVProperties(Virtual, PropertySheet, View):
 
     def dav__lockdiscovery(self):
         security = getSecurityManager()
-        user = security.getUser().getUserName()
-        
+        user = security.getUser().getId()
+
 
         vself = self.v_self()
         out = '\n'
@@ -562,9 +583,9 @@ class DAVProperties(Virtual, PropertySheet, View):
                 creator = lock.getCreator()[-1]
                 if creator == user: fake=0
                 else:               fake=1
-                    
+
                 out = '%s\n%s' % (out, lock.asLockDiscoveryProperty('n',fake=fake))
-                
+
             out = '%s\n' % out
 
         return out
@@ -576,7 +597,7 @@ Globals.default__class_init__(DAVProperties)
 class PropertySheets(Traversable, Implicit, App.Management.Tabs):
     """A tricky container to keep property sets from polluting
        an object's direct attribute namespace."""
-    
+
     id='propertysheets'
 
     __ac_permissions__=(
@@ -592,10 +613,13 @@ class PropertySheets(Traversable, Implicit, App.Management.Tabs):
         )
 
 
+    # optionally to be overridden by derived classes
+    PropertySheetClass= PropertySheet
+
     webdav =DAVProperties()
     def _get_defaults(self):
         return (self.webdav,)
-    
+
     def __propsets__(self):
         propsets=self.aq_parent.__propsets__
         __traceback_info__= propsets, type(propsets)
@@ -623,7 +647,7 @@ class PropertySheets(Traversable, Implicit, App.Management.Tabs):
             r.append((id, n.__of__(self)))
 
         return r
-        
+
     def get(self, name, default=None):
         for propset in self.__propsets__():
             if propset.id==name or (hasattr(propset, 'xml_namespace') and \
@@ -631,12 +655,14 @@ class PropertySheets(Traversable, Implicit, App.Management.Tabs):
                 return propset.__of__(self)
         return default
 
-    def manage_addPropertySheet(self, id, ns):
+    def manage_addPropertySheet(self, id, ns, REQUEST=None):
         """ """
         md={'xmlns':ns}
-        ps=PropertySheet(id, md)
+        ps= self.PropertySheetClass(id, md)
         self.addPropertySheet(ps)
-        return 'OK'
+        if REQUEST is None: return ps
+        ps= self.get(id)
+        REQUEST.RESPONSE.redirect('%s/manage' % ps.absolute_url())
 
     def addPropertySheet(self, propset):
         propsets=self.aq_parent.__propsets__
@@ -648,8 +674,27 @@ class PropertySheets(Traversable, Implicit, App.Management.Tabs):
         for propset in self.aq_parent.__propsets__:
             if propset.getId() != name and  propset.xml_namespace() != name:
                 result.append(propset)
-        self.parent.__propsets__=tuple(result)
-        
+        self.aq_parent.__propsets__=tuple(result)
+
+    ## DM: deletion support
+    def isDeletable(self,name):
+        '''currently, we say that *name* is deletable when it is not a
+        default sheet. Later, we may further restrict deletability
+        based on an instance attribute.'''
+        ps= self.get(name)
+        if ps is None: return 0
+        if ps in self._get_defaults(): return 0
+        return 1
+
+    def manage_delPropertySheets(self, ids=(), REQUEST=None):
+        '''delete all sheets identified by *ids*.'''
+        for id in ids:
+            if not self.isDeletable(id):
+                raise BadRequestException, 'attempt to delete undeletable property sheet: ' + id
+            self.delPropertySheet(id)
+        if REQUEST is not None: 
+            REQUEST.RESPONSE.redirect('%s/manage' % self.absolute_url())
+
     def __len__(self):
         return len(self.__propsets__())
 
@@ -669,20 +714,20 @@ class PropertySheets(Traversable, Implicit, App.Management.Tabs):
         if r is None:
             pre='../'
         else:
-            pre=r['URL']
+            pre=r['URLPATH0']
             for i in (1,2):
-                l=rfind(pre,'/')
+                l=pre.rfind('/')
                 if l >= 0:
                     pre=pre[:l]
             pre=pre+'/'
-            
+
         r=[]
         for d in self.aq_parent.manage_options:
             r.append({'label': d['label'], 'action': pre+d['action']})
         return r
 
     def tabs_path_info(self, script, path):
-        l=rfind(path,'/')
+        l=path.rfind('/')
         if l >= 0: path=path[:l]
         return PropertySheets.inheritedAttribute('tabs_path_info')(
             self, script, path)
@@ -726,7 +771,7 @@ class FixedSchema(PropertySheet):
             if 'd' in mode:
                 d['mode']=filter(lambda c: c != 'd', mode)
             r.append(d)
-            
+
         return tuple(r)
 
     def propertyMap(self):
@@ -737,7 +782,7 @@ class FixedSchema(PropertySheet):
         return self._base._extensible
 
 Globals.default__class_init__(FixedSchema)
-    
+
 
 
 class vps(Base):
@@ -749,7 +794,7 @@ class vps(Base):
     """
     def __init__(self, c=PropertySheets):
         self.c=c
-        
+
     def __of__(self, parent):
         return self.c().__of__(parent)
 
@@ -763,7 +808,7 @@ def xml_escape(v):
     """ convert any content from ISO-8859-1 to UTF-8
     The main use is to escape non-US object property values
     (e.g. containing accented characters). Also we convert "<" and ">"
-    to entities to keep the properties XML compliant. 
+    to entities to keep the properties XML compliant.
     """
     v = str(v)
     v = v.replace('&', '&amp;')
