@@ -1,37 +1,39 @@
 ##############################################################################
 #
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
-# 
+#
 ##############################################################################
 '''This module implements a simple item mix-in for objects that have a
 very simple (e.g. one-screen) management interface, like documents,
 Aqueduct database adapters, etc.
 
 This module can also be used as a simple template for implementing new
-item types. 
+item types.
 
-$Id: SimpleItem.py,v 1.92 2001/11/28 15:50:57 matt Exp $'''
-__version__='$Revision: 1.92 $'[11:-2]
+$Id: SimpleItem.py,v 1.105 2002/08/30 20:21:29 caseman Exp $'''
+__version__='$Revision: 1.105 $'[11:-2]
 
 import re, sys, Globals, App.Management, Acquisition, App.Undo
 import AccessControl.Role, AccessControl.Owned, App.Common
 from webdav.Resource import Resource
 from ExtensionClass import Base
 from CopySupport import CopySource
-from string import join, lower, find, split
 from types import InstanceType, StringType
 from ComputedAttribute import ComputedAttribute
 from AccessControl import getSecurityManager
 from Traversable import Traversable
-from Acquisition import aq_base
+from Acquisition import aq_base, aq_parent, aq_inner, aq_acquire
+from DocumentTemplate.ustr import ustr
+from zExceptions.ExceptionFormatter import format_exception
 import time
+from zLOG import LOG, BLATHER
 
 import marshal
 import ZDOM
@@ -47,7 +49,7 @@ class Item(Base, Resource, CopySource, App.Management.Tabs, Traversable,
     """A common base class for simple, non-container objects."""
     isPrincipiaFolderish=0
     isTopLevelPrincipiaApplicationObject=0
-    
+
     def manage_afterAdd(self, item, container): pass
     def manage_beforeDelete(self, item, container): pass
     def manage_afterClone(self, item): pass
@@ -79,17 +81,17 @@ class Item(Base, Resource, CopySource, App.Management.Tabs, Traversable,
     # Meta type used for selecting all objects of a given type.
     meta_type='simple item'
 
-    # Default title.  
+    # Default title.
     title=''
 
     # Default propertysheet info:
     __propsets__=()
- 
+
     manage_options=(
         App.Undo.UndoSupport.manage_options
         +AccessControl.Owned.Owned.manage_options
         )
-    
+
     # Attributes that must be acquired
     REQUEST=Acquisition.Acquired
 
@@ -119,7 +121,7 @@ class Item(Base, Resource, CopySource, App.Management.Tabs, Traversable,
             title=title()
         id = self.getId()
         return title and ("%s (%s)" % (title,id)) or id
-    
+
     def this(self):
         # Handy way to talk to ourselves in document templates.
         return self
@@ -140,42 +142,43 @@ class Item(Base, Resource, CopySource, App.Management.Tabs, Traversable,
         self, client=None, REQUEST={},
         error_type=None, error_value=None, tb=None,
         error_tb=None, error_message='',
-        tagSearch=re.compile(r'[a-zA-Z]>').search):
+        tagSearch=re.compile(r'[a-zA-Z]>').search,
+        error_log_url=''):
 
         try:
             if error_type  is None: error_type =sys.exc_info()[0]
             if error_value is None: error_value=sys.exc_info()[1]
-            
-            # turn error_type into a string            
-            if hasattr(error_type, '__name__'):
-                error_type=error_type.__name__
 
             # allow for a few different traceback options
             if tb is None and error_tb is None:
                 tb=sys.exc_info()[2]
             if type(tb) is not type('') and (error_tb is None):
-                error_tb=pretty_tb(error_type, error_value, tb)
+                error_tb = pretty_tb(error_type, error_value, tb)
             elif type(tb) is type('') and not error_tb:
-                error_tb=tb
+                error_tb = tb
+
+            # turn error_type into a string
+            if hasattr(error_type, '__name__'):
+                error_type=error_type.__name__
 
             if hasattr(self, '_v_eek'):
+                # Stop if there is recursion.
                 raise error_type, error_value, tb
             self._v_eek=1
-   
-            if lower(str(error_type)) in ('redirect',):
+
+            if str(error_type).lower() in ('redirect',):
                 raise error_type, error_value, tb
 
             if not error_message:
-                if type(error_value) is InstanceType:
-                    try:
-                        s=str(error_value)
-                    except:
-                        pass
-                    else:
-                        if tagSearch(s) is not None:
-                            error_message=error_value
-                elif (type(error_value) is StringType
-                      and tagSearch(error_value) is not None):
+                try:
+                    s = ustr(error_value)
+                except:
+                    s = error_value
+                try:
+                    match = tagSearch(s)
+                except TypeError:
+                    match = None
+                if match is not None:
                     error_message=error_value
 
             if client is None: client=self
@@ -191,14 +194,27 @@ class Item(Base, Resource, CopySource, App.Management.Tabs, Traversable,
                           'error_value': error_value,
                           'error_tb': error_tb,
                           'error_traceback': error_tb,
-                          'error_message': error_message}
+                          'error_message': error_message,
+                          'error_log_url': error_log_url}
+
                 if isinstance(s, HTML):
                     v = s(client, REQUEST, **kwargs)
                 elif callable(s):
                     v = s(**kwargs)
                 else:
                     v = HTML.__call__(s, client, REQUEST, **kwargs)
-            except: v = error_value or "Sorry, an error occurred"
+            except:
+                LOG('OFS', BLATHER,
+                    'Exception while rendering an error message',
+                    error=sys.exc_info())
+                try:
+                    strv = str(error_value)
+                except:
+                    strv = ('<unprintable %s object>' % 
+                            str(type(error_value).__name__))
+                v = strv + (
+                    " (Also, an error occurred while attempting "
+                    "to render the standard error message.)")
             raise error_type, v, tb
         finally:
             if hasattr(self, '_v_eek'): del self._v_eek
@@ -206,7 +222,7 @@ class Item(Base, Resource, CopySource, App.Management.Tabs, Traversable,
 
     def manage(self, URL1):
         " "
-        raise 'Redirect', "%s/manage_main" % URL1 
+        raise 'Redirect', "%s/manage_main" % URL1
 
     # This keeps simple items from acquiring their parents
     # objectValues, etc., when used in simple tree tags.
@@ -215,14 +231,14 @@ class Item(Base, Resource, CopySource, App.Management.Tabs, Traversable,
     objectIds=objectItems=objectValues
 
     # FTP support methods
-    
+
     def manage_FTPstat(self,REQUEST):
         "psuedo stat, used by FTP for directory listings"
         from AccessControl.User import nobody
         mode=0100000
-        
+
         # check read permissions
-        if (hasattr(aq_base(self),'manage_FTPget') and 
+        if (hasattr(aq_base(self),'manage_FTPget') and
             hasattr(self.manage_FTPget, '__roles__')):
             try:
                 if getSecurityManager().validateValue(self.manage_FTPget):
@@ -231,17 +247,17 @@ class Item(Base, Resource, CopySource, App.Management.Tabs, Traversable,
             if nobody.allowed(self.manage_FTPget,
                               self.manage_FTPget.__roles__):
                 mode=mode | 0004
-                
+
         # check write permissions
         if hasattr(aq_base(self),'PUT') and hasattr(self.PUT, '__roles__'):
             try:
                 if getSecurityManager().validateValue(self.PUT):
                     mode=mode | 0220
             except: pass
-            
+
             if nobody.allowed(self.PUT, self.PUT.__roles__):
                 mode=mode | 0002
-                
+
         # get size
         if hasattr(aq_base(self), 'get_size'):
             size=self.get_size()
@@ -274,7 +290,7 @@ class Item(Base, Resource, CopySource, App.Management.Tabs, Traversable,
             if not hasattr(ob,'aq_parent'):
                 break
             ob=ob.aq_parent
-            
+
         stat=marshal.loads(self.manage_FTPstat(REQUEST))
         id = self.getId()
         return marshal.dumps((id,stat))
@@ -310,46 +326,20 @@ class Item_w__name__(Item):
         and getPhysicalPath() are designed to operate together.
         '''
         path = (self.__name__,)
-        
-        p = getattr(self,'aq_inner', None)
-        if p is not None: 
-            path = p.aq_parent.getPhysicalPath() + path
-            
+
+        p = aq_parent(aq_inner(self))
+        if p is not None:
+            path = p.getPhysicalPath() + path
+
         return path
 
 
-def format_exception(etype,value,tb,limit=None):
-    import traceback
-    result=['Traceback (innermost last):']
-    if limit is None:
-        if hasattr(sys, 'tracebacklimit'):
-            limit = sys.tracebacklimit
-    n = 0
-    while tb is not None and (limit is None or n < limit):
-        f = tb.tb_frame
-        lineno = tb.tb_lineno
-        co = f.f_code
-        filename = co.co_filename
-        name = co.co_name
-        locals=f.f_locals
-        result.append('  File %s, line %d, in %s'
-                      % (filename,lineno,name))
-        try: result.append('    (Object: %s)' %
-                           locals[co.co_varnames[0]].__name__)
-        except: pass
-        try: result.append('    (Info: %s)' %
-                           str(locals['__traceback_info__']))
-        except: pass
-        tb = tb.tb_next
-        n = n+1
-    result.append(join(traceback.format_exception_only(etype, value),' '))
-    return result
-
-def pretty_tb(t,v,tb):
-    tb=format_exception(t,v,tb,200)
-    tb=join(tb,'\n')
+def pretty_tb(t, v, tb, as_html=1):
+    tb = format_exception(t, v, tb, as_html=as_html)
+    tb = '\n'.join(tb)
     return tb
 
+
 class SimpleItem(Item, Globals.Persistent,
                  Acquisition.Implicit,
                  AccessControl.Role.RoleManager,
@@ -363,5 +353,30 @@ class SimpleItem(Item, Globals.Persistent,
          'action':'manage_access',
          'help':('OFSP', 'Security.stx')},
         )
- 
+
     __ac_permissions__=(('View', ()),)
+
+    def __repr__(self):
+        """Show the physical path of the object and its context if available.
+        """
+        try:
+            path = '/'.join(self.getPhysicalPath())
+        except:
+            path = None
+        context_path = None
+        context = aq_parent(self)
+        container = aq_parent(aq_inner(self))
+        if aq_base(context) is not aq_base(container):
+            try:
+                context_path = '/'.join(context.getPhysicalPath())
+            except:
+                context_path = None
+        res = '<%s' % self.__class__.__name__
+        if path:
+            res += ' at %s' % path
+        else:
+            res += ' at 0x%x' % id(self)
+        if context_path:
+            res += ' used for %s' % context_path
+        res += '>'
+        return res
