@@ -1,23 +1,22 @@
 ##############################################################################
 #
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
-# 
+#
 ##############################################################################
-__rcs_id__='$Id: SendMailTag.py,v 1.15 2002/01/15 14:58:37 jens Exp $'
-__version__='$Revision: 1.15 $'[11:-2]
+__rcs_id__='$Id: SendMailTag.py,v 1.17.6.1 2003/01/14 19:15:44 regebro Exp $'
+__version__='$Revision: 1.17.6.1 $'[11:-2]
 
-from MailHost import MailBase
+from MailHost import MailBase, MailHostError
 from DocumentTemplate.DT_Util import parse_params,render_blocks
 from DocumentTemplate.DT_String import String
 from socket import gethostname
-import string
 
 class SendMailTag:
     '''the send mail tag, used like thus:
@@ -65,16 +64,17 @@ class SendMailTag:
                           subject=None, smtphost=None, port='25',
                           encode=None)
 
-        for key in ('mailto', 'mailfrom', 'subject', 'smtphost', 'port'):
-            if not args.has_key(key):args[key]=''
         smtphost=None
-
+        
         has_key=args.has_key
         if has_key('mailhost'): mailhost=args['mailhost']
         elif has_key('smtphost'): mailhost=smtphost=args['smtphost']
         elif has_key(''): mailhost=args['mailhost']=args['']
         else: raise MailHostError, 'No mailhost was specified in tag'
 
+        for key in ('mailto', 'mailfrom', 'subject', 'port'):
+            if not args.has_key(key):args[key]=''
+
         if has_key('encode') and args['encode'] not in \
         ('base64', 'quoted-printable', 'uuencode', 'x-uuencode',
          'uue', 'x-uue'):
@@ -93,7 +93,7 @@ class SendMailTag:
         self.mailfrom=args['mailfrom']
         self.subject=None or args['subject']
         if args['port'] and type(args['port']) is type('s'):
-            self.port=args['port']=string.atoi(args['port'])
+            self.port=args['port']=int(args['port'])
         elif args['port']=='':
             self.port=args['port']=25
         else:
