@@ -1,20 +1,19 @@
 ##############################################################################
 #
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
-# 
+#
 ##############################################################################
 import Globals, AccessControl.User
 from Globals import Persistent
 from Acquisition import Implicit
 from OFS import SimpleItem
-from string import rfind
 
 manage_addPrincipiaDraftForm=Globals.HTMLFile('dtml/draftAdd',globals())
 def manage_addPrincipiaDraft(self, id, baseid, PATH_INFO, REQUEST=None):
@@ -39,12 +38,12 @@ class Draft(Persistent, Implicit, SimpleItem.Item):
         self.id=id
         self._refid=baseid
         version=PATH_INFO
-        l=rfind(version,'/')
+        l=version.rfind('/')
         if l >= 0: version=version[:l]
         self._version="%s/%s" % (version, id)
         self.users__draft__=uf=AccessControl.User.UserFolder()
         self.__allow_groups__=uf
-    
+
     def icon(self):
         try: return getattr(self.aq_parent.aq_base,self._refid).icon
         except: return 'p_/broken'
@@ -76,7 +75,7 @@ class Draft(Persistent, Implicit, SimpleItem.Item):
     def __bobo_traverse__(self, REQUEST, name):
         if name[-9:]=='__draft__': return getattr(self, name)
 
-        
+
         try: db=self._p_jar.db()
         except:
             # BoboPOS 2
@@ -87,15 +86,15 @@ class Draft(Persistent, Implicit, SimpleItem.Item):
             cleanup=Cleanup()
             cleanup.__del__=jar.close
             REQUEST[Cleanup]=cleanup
-            
-            
+
+
         dself=getdraft(self, jar)
-        
+
         ref=getattr(dself.aq_parent.aq_base,dself._refid).aq_base.__of__(dself)
         if hasattr(ref, name): return dself, ref, getattr(ref, name)
         return getattr(self, name)
-    
-    def nonempty(self):        
+
+    def nonempty(self):
         try: db=self._p_jar.db()
         except:
             # BoboPOS 2
@@ -118,7 +117,7 @@ class Draft(Persistent, Implicit, SimpleItem.Item):
             d=self._p_jar.getVersion()
             if d==s: d=''
             db.commitVersion(s, d)
-            
+
         if REQUEST:
             REQUEST['RESPONSE'].redirect(REQUEST['URL2']+'/manage_main')
 
@@ -142,7 +141,7 @@ class Draft(Persistent, Implicit, SimpleItem.Item):
         if not self._version:
             self._version=self.absolute_url(1)
 
-    def manage_beforeDelete(self, item, container):        
+    def manage_beforeDelete(self, item, container):
         if self.nonempty():
             raise 'Version Error', (
                 'Attempt to %sdelete a non-empty version.<p>'
