@@ -1,14 +1,14 @@
 ##############################################################################
 #
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
-# 
+#
 ##############################################################################
 
 """Python Scripts Product
@@ -17,10 +17,10 @@ This product provides support for Script objects containing restricted
 Python code.
 """
 
-__version__='$Revision: 1.38.10.2 $'[11:-2]
+__version__='$Revision: 1.43.6.7 $'[11:-2]
 
-import sys, os, traceback, re, marshal
-from Globals import DTMLFile, MessageDialog, package_home
+import sys, os, traceback, re, marshal, new
+from Globals import DTMLFile, MessageDialog, package_home, InitializeClass
 import AccessControl, OFS, RestrictedPython
 from OFS.SimpleItem import SimpleItem
 from DateTime.DateTime import DateTime
@@ -31,8 +31,7 @@ from Shared.DC.Scripts.Script import Script, BindingsUI, defaultBindings
 from AccessControl import getSecurityManager
 from OFS.History import Historical, html_diff
 from OFS.Cache import Cacheable
-from AccessControl import full_write_guard, safe_builtins
-from AccessControl.ZopeGuards import guarded_getattr, guarded_getitem
+from AccessControl.ZopeGuards import get_safe_globals, guarded_getattr
 from zLOG import LOG, ERROR, INFO, PROBLEM
 
 # Track the Python bytecode version
@@ -153,7 +152,11 @@ class PythonScript(Script, Historical, Cacheable):
         """Replace the body of the script with the text in file."""
         if self.wl_isLocked():
             raise ResourceLockedError, "The script is locked via WebDAV."
-        if type(file) is not type(''): file = file.read()
+
+        if type(file) is not type(''):
+            if not file: raise ValueError, 'File not specified'
+            file = file.read()
+
         self.write(file)
         message = 'Saved changes.'
         return self.ZPythonScriptHTML_editForm(self, REQUEST,
@@ -179,7 +182,7 @@ class PythonScript(Script, Historical, Cacheable):
         """Parameters to test the script with."""
         param_names = []
         for name in self._params.split(','):
-           
+
             name = name.strip()
             if name and name[0] != '*' and re.match('\w',name):
                 param_names.append(name.split('=', 1)[0])
@@ -197,7 +200,7 @@ class PythonScript(Script, Historical, Cacheable):
             getattr(self, 'Script_magic', None) != Script_magic):
             global _log_complaint
             if _log_complaint:
-                LOG(self.meta_type, INFO, _log_complaint)    
+                LOG(self.meta_type, INFO, _log_complaint)
                 _log_complaint = 0
             # Changes here won't get saved, unless this Script is edited.
             body = self._body.rstrip()
@@ -206,12 +209,13 @@ class PythonScript(Script, Historical, Cacheable):
             self._compile()
             self._v_change = 1
         elif self._code is None:
-            self._v_f = None
+            self._v_ft = self._v_f = None
         else:
             self._newfun(marshal.loads(self._code))
 
     def _compiler(self, *args):
         return RestrictedPython.compile_restricted_function(*args)
+
     def _compile(self):
         r = self._compiler(self._params, self._body or 'pass',
                            self.id, self.meta_type)
@@ -220,7 +224,7 @@ class PythonScript(Script, Historical, Cacheable):
         self.warnings = tuple(r[2])
         if errors:
             self._code = None
-            self._v_f = None
+            self._v_ft = self._v_f = None
             self._setFuncSignature((), (), 0)
             # Fix up syntax errors.
             filestring = '  File "<string>",'
@@ -242,16 +246,14 @@ class PythonScript(Script, Historical, Cacheable):
         self._v_change = 0
 
     def _newfun(self, code):
-        g = {'__debug__': __debug__,
-             '__builtins__': safe_builtins,
-             '_getattr_': guarded_getattr,
-             '_getitem_': guarded_getitem,
-             '_write_': full_write_guard,
-             '_print_': RestrictedPython.PrintCollector
-             }
+        g = get_safe_globals()
+        g['_getattr_'] = guarded_getattr
+        g['__debug__'] = __debug__
+        g['__name__'] = None
         l = {}
         exec code in g, l
         self._v_f = f = l.values()[0]
+        self._v_ft = (f.func_code, g, f.func_defaults or ())
         return f
 
     def _makeFunction(self, dummy=0): # CMFCore.FSPythonScript uses dummy arg.
@@ -259,7 +261,7 @@ class PythonScript(Script, Historical, Cacheable):
         self._compile()
 
     def _editedBindings(self):
-        if getattr(self, '_v_f', None) is not None:
+        if getattr(self, '_v_ft', None) is not None:
             self._makeFunction()
 
     def _exec(self, bound_names, args, kw):
@@ -286,26 +288,27 @@ class PythonScript(Script, Historical, Cacheable):
                 # Got a cached value.
                 return result
 
-        __traceback_info__ = bound_names, args, kw, self.func_defaults
+        #__traceback_info__ = bound_names, args, kw, self.func_defaults
 
-        f = self._v_f
-        if f is None:
+        ft = self._v_ft
+        if ft is None:
+            __traceback_supplement__ = (
+                PythonScriptTracebackSupplement, self)
             raise RuntimeError, '%s %s has errors.' % (self.meta_type, self.id)
+
+        fcode, g, fadefs = ft
+        g = g.copy()
         if bound_names is not None:
-            # Updating func_globals directly *should* be thread-safe.
-            f.func_globals.update(bound_names)
-    
-        # Execute the function in a new security context.
-        security=getSecurityManager()
-        security.addContext(self)
-        try:
-            result = apply(f, args, kw)
-            if keyset is not None:
-                # Store the result in the cache.
-                self.ZCacheable_set(result, keywords=keyset)
-            return result
-        finally:
-            security.removeContext(self)
+            g.update(bound_names)
+        g['__traceback_supplement__'] = (
+            PythonScriptTracebackSupplement, self, -1)
+        f = new.function(fcode, g, None, fadefs)
+
+        result = apply(f, args, kw)
+        if keyset is not None:
+            # Store the result in the cache.
+            self.ZCacheable_set(result, keywords=keyset)
+        return result
 
     def manage_haveProxy(self,r): return r in self._proxy_roles
 
@@ -316,7 +319,7 @@ class PythonScript(Script, Historical, Cacheable):
         if user is not None and user.allowed(self, roles):
             return
         raise 'Forbidden', ('You are not authorized to change <em>%s</em> '
-            'because you do not have proxy roles.\n<!--%s, %s-->' 
+            'because you do not have proxy roles.\n<!--%s, %s-->'
             % (self.id, user, roles))
 
     security.declareProtected('Change proxy roles',
@@ -345,7 +348,7 @@ class PythonScript(Script, Historical, Cacheable):
         self.dav__simpleifhandler(REQUEST, RESPONSE, refresh=1)
         self.write(REQUEST.get('BODY', ''))
         RESPONSE.setStatus(204)
-        return RESPONSE        
+        return RESPONSE
 
     manage_FTPput = PUT
 
@@ -400,9 +403,9 @@ class PythonScript(Script, Historical, Cacheable):
             if body != self._body:
                 self._body = body
             if bup:
-                self._setupBindings(bindmap)
-
-            self._makeFunction()
+                self.ZBindings_edit(bindmap)
+            else:
+                self._makeFunction()
         except:
             LOG(self.meta_type, ERROR, 'write failed', error=sys.exc_info())
             raise
@@ -436,7 +439,7 @@ class PythonScript(Script, Historical, Cacheable):
         hlines = ['%s %s "%s"' % (prefix, self.meta_type, self.id)]
         mm = self._metadata_map().items()
         mm.sort()
-        for kv in mm: 
+        for kv in mm:
             hlines.append('%s=%s' % kv)
         if self.errors:
             hlines.append('')
@@ -467,6 +470,17 @@ class PythonScript(Script, Historical, Cacheable):
             RESPONSE.setHeader('Content-Type', 'text/plain')
         return self.read()
 
+
+InitializeClass(PythonScript)
+
+class PythonScriptTracebackSupplement:
+    """Implementation of ITracebackSupplement"""
+    def __init__(self, script, line=0):
+        self.object = script
+        # If line is set to -1, it means to use tb_lineno.
+        self.line = line
+
+
 _first_indent = re.compile('(?m)^ *(?! |$)')
 _nonempty_line = re.compile('(?m)^(.*\S.*)$')
 
