@@ -1,17 +1,17 @@
 ##############################################################################
 #
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
-# 
+#
 ##############################################################################
-__doc__='''$Id: Lazy.py,v 1.6 2001/11/28 15:51:09 matt Exp $'''
-__version__='$Revision: 1.6 $'[11:-2]
+__doc__='''$Id: Lazy.py,v 1.7.6.2 2002/12/11 19:12:14 caseman Exp $'''
+__version__='$Revision: 1.7.6.2 $'[11:-2]
 
 
 class Lazy:
@@ -20,9 +20,9 @@ class Lazy:
     __allow_access_to_unprotected_subobjects__=1
 
     def __repr__(self): return `list(self)`
-    
-    def __len__(self):
 
+    def __len__(self):
+        # This is a worst-case len, subclasses should try to do better
         try: return self._len
         except AttributeError: pass
 
@@ -34,35 +34,14 @@ class Lazy:
             except:
                 self._len=l
                 return l
+                
 
     def __add__(self, other):
-        try:
-            for base in other.__class__.__bases__:
-                if base.__name__ == 'Lazy':
-                    break
-            else:
-                raise TypeError
-        except:
-            raise TypeError, "Can not concatenate objects. Both must be lazy sequences."
+        if not isinstance(other, Lazy):
+            raise TypeError(
+                "Can not concatenate objects. Both must be lazy sequences.")
+        return LazyCat([self, other])
 
-        if self.__class__.__name__ == 'LazyCat':
-            if hasattr(self, '_seq'):
-                seq = self._seq
-            else:
-                seq = [self._data]
-        else:
-            seq = [self]
-
-        if other.__class__.__name__ == 'LazyCat':
-            if hasattr(other, '_seq'):
-                seq = seq + other._seq
-            else:
-                seq.append(other._data)
-        else:
-            seq.append(other)
-
-        return LazyCat(seq)
-    
     def __getslice__(self,i1,i2):
         r=[]
         for i in xrange(i1,i2):
@@ -75,8 +54,21 @@ class Lazy:
 class LazyCat(Lazy):
     # Lazy concatenation of one or more sequences.  Should be handy
     # for accessing small parts of big searches.
-    
+
     def __init__(self, sequences, length=None):
+        if len(sequences) < 100:
+            # Optimize structure of LazyCats to avoid nesting
+            # We don't do this for large numbers of input sequences
+            # to make instantiation faster instead
+            flattened_seq = []
+            for s in sequences:
+                if isinstance(s, LazyCat):
+                    # If one of the sequences passed is itself a LazyCat, add
+                    # its base sequences rather than nest LazyCats
+                    flattened_seq.extend(s._seq)
+                else:
+                    flattened_seq.append(s)
+            sequences = flattened_seq
         self._seq=sequences
         self._data=[]
         self._sindex=0
@@ -118,6 +110,18 @@ class LazyCat(Lazy):
                 self._eindex=eindex=-1
         self._eindex=eindex
         return data[i]
+        
+    def __len__(self):
+        # Make len of LazyCat only as expensive as the lens 
+        # of its underlying sequences
+        try:
+            return self._len
+        except:
+            l = 0
+            for s in self._seq:
+               l += len(s)
+            self._len = l
+            return l
 
 class LazyMap(Lazy):
     # Act like a sequence, but get data from a filtering process.
@@ -238,3 +242,21 @@ class LazyMop(Lazy):
                 raise IndexError, index
         self._eindex=e
         return data[i]
+
+class LazyValues(Lazy):
+    """Given a sequence of two tuples typically (key, value) act as
+    though we are just a list of the values lazily"""
+    
+    def __init__(self, seq):
+        self._seq = seq
+    
+    def __len__(self):
+        return len(self._seq)
+    
+    def __getitem__(self, index):
+        return self._seq[index][1]
+    
+    def __getslice__(self, start, end):
+        return self.__class__(self._seq[start:end])
+        
+    slice = __getslice__
