@@ -1,22 +1,24 @@
 ##############################################################################
 #
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
-# 
+#
 ##############################################################################
-"""ZCatalog product"""
+""" ZCatalog product
+
+$Id: ZCatalog.py,v 1.121.2.4 2003/12/07 16:16:37 yuppie Exp $
+"""
 
 from Globals import DTMLFile, MessageDialog
 import Globals
 
 from OFS.Folder import Folder
-from OFS.FindSupport import FindSupport
 from OFS.ObjectManager import ObjectManager
 from DateTime import DateTime
 from Acquisition import Implicit
@@ -25,30 +27,27 @@ from DocumentTemplate.DT_Util import InstanceDict, TemplateDict
 from DocumentTemplate.DT_Util import Eval
 from AccessControl.Permission import name_trans
 from Catalog import Catalog, CatalogError
-from AccessControl import getSecurityManager
 from AccessControl.DTML import RestrictedDTML
-from zLOG import LOG, ERROR
+from AccessControl.Permissions import \
+    manage_zcatalog_entries, manage_zcatalog_indexes, search_zcatalog
 from ZCatalogIndexes import ZCatalogIndexes
-from Products.PluginIndexes.common.PluggableIndex import PluggableIndexInterface
-from Products.PluginIndexes.TextIndex.Vocabulary import Vocabulary
+from Products.PluginIndexes.common.PluggableIndex \
+     import PluggableIndexInterface
 from Products.PluginIndexes.TextIndex import Splitter
-import string,  urllib, os, sys, time, types
-
+import urllib, time, types
+import string
+from IZCatalog import IZCatalog
 
 
 manage_addZCatalogForm=DTMLFile('dtml/addZCatalog',globals())
 
 def manage_addZCatalog(self, id, title,
-                       vocab_id='create_default_catalog_',
+                       vocab_id=None, # Deprecated
                        REQUEST=None):
     """Add a ZCatalog object
     """
     id=str(id)
     title=str(title)
-    vocab_id=str(vocab_id)
-    if vocab_id == 'create_default_catalog_':
-        vocab_id = None
-        
     c=ZCatalog(id, title, vocab_id, self)
     self._setObject(id, c)
     if REQUEST is not None:
@@ -56,16 +55,18 @@ def manage_addZCatalog(self, id, title,
 
 
 class ZCatalog(Folder, Persistent, Implicit):
+    __implements__ = IZCatalog
+
     """ZCatalog object
 
     A ZCatalog contains arbirary index like references to Zope
-    objects.  ZCatalog's can index either 'Field' values of object, or 
+    objects.  ZCatalog's can index either 'Field' values of object, or
     'Text' values.
 
     ZCatalog does not store references to the objects themselves, but
     rather to a unique identifier that defines how to get to the
     object.  In Zope, this unique idenfier is the object's relative
-    path to the ZCatalog (since two Zope object's cannot have the same 
+    path to the ZCatalog (since two Zope object's cannot have the same
     URL, this is an excellent unique qualifier in Zope).
 
     Most of the dirty work is done in the _catalog object, which is an
@@ -84,25 +85,21 @@ class ZCatalog(Folder, Persistent, Implicit):
          'help': ('OFSP','ObjectManager_Contents.stx')},
         {'label': 'Catalog',            # TAB: Cataloged Objects
          'action': 'manage_catalogView',
-         'target': 'manage_main',
          'help':('ZCatalog','ZCatalog_Cataloged-Objects.stx')},
         {'label': 'Properties',         # TAB: Properties
          'action': 'manage_propertiesForm',
          'help': ('OFSP','Properties.stx')},
         {'label': 'Indexes',            # TAB: Indexes
-         'action': 'Indexes/manage_workspace',
+         'action': 'manage_catalogIndexes',
          'help': ('ZCatalog','ZCatalog_Indexes.stx')},
         {'label': 'Metadata',           # TAB: Metadata
-         'action': 'manage_catalogSchema', 
-         'target':'manage_main',
+         'action': 'manage_catalogSchema',
          'help':('ZCatalog','ZCatalog_MetaData-Table.stx')},
         {'label': 'Find Objects',       # TAB: Find Objects
-         'action': 'manage_catalogFind', 
-         'target':'manage_main',
+         'action': 'manage_catalogFind',
          'help':('ZCatalog','ZCatalog_Find-Items-to-ZCatalog.stx')},
         {'label': 'Advanced',           # TAB: Advanced
-         'action': 'manage_catalogAdvanced', 
-         'target':'manage_main',
+         'action': 'manage_catalogAdvanced',
          'help':('ZCatalog','ZCatalog_Advanced.stx')},
         {'label': 'Undo',               # TAB: Undo
          'action': 'manage_UndoForm',
@@ -117,30 +114,34 @@ class ZCatalog(Folder, Persistent, Implicit):
 
     __ac_permissions__=(
 
-        ('Manage ZCatalog Entries',
+        (manage_zcatalog_entries,
          ['manage_catalogObject', 'manage_uncatalogObject',
           'catalog_object', 'uncatalog_object', 'refreshCatalog',
-          
+
           'manage_catalogView', 'manage_catalogFind',
           'manage_catalogSchema', 'manage_catalogIndexes',
           'manage_catalogAdvanced', 'manage_objectInformation',
-          
+
           'manage_catalogReindex', 'manage_catalogFoundItems',
           'manage_catalogClear', 'manage_addColumn', 'manage_delColumn',
           'manage_addIndex', 'manage_delIndex', 'manage_clearIndex',
           'manage_reindexIndex', 'manage_main', 'availableSplitters',
-          
+
           # these two are deprecated:
           'manage_delColumns', 'manage_deleteIndex'
-          ], 
+          ],
          ['Manager']),
 
-        ('Search ZCatalog',
+        (search_zcatalog,
          ['searchResults', '__call__', 'uniqueValuesFor',
-          'getpath', 'schema', 'indexes', 'index_objects',
+          'getpath', 'schema', 'indexes', 'index_objects', 
           'all_meta_types', 'valid_roles', 'resolve_url',
           'getobject'],
-         ['Anonymous', 'Manager']), 
+         ['Anonymous', 'Manager']),
+         
+        (manage_zcatalog_indexes, 
+         ['getIndexObjects'], 
+         ['Manager']),
         )
 
 
@@ -158,63 +159,44 @@ class ZCatalog(Folder, Persistent, Implicit):
     threshold=10000
     _v_total=0
     _v_transaction = None
-    
+
     def __init__(self, id, title='', vocab_id=None, container=None):
-        if vocab_id is not None and container is None:
-            raise CatalogError, ("You cannot specify a vocab_id without "
-                                     "also specifying a container.")
+        # ZCatalog no longer cares about vocabularies
+        # so the vocab_id argument is ignored (Casey)
+
         if container is not None:
             self=self.__of__(container)
         self.id=id
         self.title=title
 
+        # vocabulary and vocab_id are left for backwards
+        # compatibility only, they are not used anymore
         self.vocabulary = None
-        
+        self.vocab_id = ''
+
         self.threshold = 10000
         self._v_total = 0
 
-        if vocab_id is None:
-            v = Vocabulary('Vocabulary', 'Vocabulary', globbing=1)
-            self._setObject('Vocabulary', v)
-            self.vocabulary = v
-            self.vocab_id = 'Vocabulary'
-        else:
-            self.vocab_id = vocab_id
+        self._catalog = Catalog()
+
+    def __len__(self):
+        return len(self._catalog)
 
 
-        self._catalog = Catalog(vocabulary=self.vocab_id)
-
-        self.addColumn('id')
-        self.addIndex('id', 'FieldIndex')
-
-        self.addColumn('title')
-        self.addIndex('title', 'TextIndex')
-
-        self.addColumn('meta_type')
-        self.addIndex('meta_type', 'FieldIndex')
-
-        self.addColumn('bobobase_modification_time')
-        self.addIndex('bobobase_modification_time', 'FieldIndex')
-
-        self.addColumn('summary')
-        self.addIndex('PrincipiaSearchSource', 'TextIndex')
-
-        self.addIndex('path','PathIndex')
-
-    def __len__(self): return len(self._catalog)
-
-    def getVocabulary(self):
-        """ more ack! """
-        return getattr(self, self.vocab_id)
+    # getVocabulary method is no longer supported
+    # def getVocabulary(self):
+    #   """ more ack! """
+    #   return getattr(self, self.vocab_id)
 
 
     def manage_edit(self, RESPONSE, URL1, threshold=1000, REQUEST=None):
         """ edit the catalog """
         if type(threshold) is not type(1):
-            threshold=string.atoi(threshold)
+            threshold=int(threshold)
         self.threshold = threshold
 
-        RESPONSE.redirect(URL1 + '/manage_main?manage_tabs_message=Catalog%20Changed')
+        RESPONSE.redirect(
+            URL1 + '/manage_main?manage_tabs_message=Catalog%20Changed')
 
 
     def manage_subbingToggle(self, REQUEST, RESPONSE, URL1):
@@ -223,23 +205,27 @@ class ZCatalog(Folder, Persistent, Implicit):
             self.threshold = None
         else:
             self.threshold = 10000
-        
-        RESPONSE.redirect(URL1 + '/manage_catalogAdvanced?manage_tabs_message=Catalog%20Changed')      
+
+        RESPONSE.redirect(
+            URL1 +
+            '/manage_catalogAdvanced?manage_tabs_message=Catalog%20Changed')
 
     def manage_catalogObject(self, REQUEST, RESPONSE, URL1, urls=None):
         """ index Zope object(s) that 'urls' point to """
         if urls:
             if isinstance(urls, types.StringType):
                 urls=(urls,)
-                
+
             for url in urls:
                 obj = self.resolve_path(url)
                 if not obj:
                     obj = self.resolve_url(url, REQUEST)
-                if obj is not None: 
+                if obj is not None:
                     self.catalog_object(obj, url)
 
-        RESPONSE.redirect(URL1 + '/manage_catalogView?manage_tabs_message=Object%20Cataloged')
+        RESPONSE.redirect(
+            URL1 +
+            '/manage_catalogView?manage_tabs_message=Object%20Cataloged')
 
 
     def manage_uncatalogObject(self, REQUEST, RESPONSE, URL1, urls=None):
@@ -252,7 +238,9 @@ class ZCatalog(Folder, Persistent, Implicit):
             for url in urls:
                 self.uncatalog_object(url)
 
-        RESPONSE.redirect(URL1 + '/manage_catalogView?manage_tabs_message=Object%20Uncataloged')
+        RESPONSE.redirect(
+            URL1 +
+            '/manage_catalogView?manage_tabs_message=Object%20Uncataloged')
 
 
     def manage_catalogReindex(self, REQUEST, RESPONSE, URL1):
@@ -265,13 +253,14 @@ class ZCatalog(Folder, Persistent, Implicit):
 
         elapse = time.time() - elapse
         c_elapse = time.clock() - c_elapse
-        
-        RESPONSE.redirect(URL1 +
-                          '/manage_catalogAdvanced?manage_tabs_message=' +
-                          urllib.quote('Catalog Updated<br>'
-                                       'Total time: %s<br>'
-                                       'Total CPU time: %s' % (`elapse`, `c_elapse`)))
-        
+
+        RESPONSE.redirect(
+            URL1 +
+            '/manage_catalogAdvanced?manage_tabs_message=' +
+            urllib.quote('Catalog Updated \n'
+                         'Total time: %s\n'
+                         'Total CPU time: %s' % (`elapse`, `c_elapse`)))
+
 
     def refreshCatalog(self, clear=0):
         """ re-index everything we can find """
@@ -294,7 +283,9 @@ class ZCatalog(Folder, Persistent, Implicit):
         self._catalog.clear()
 
         if REQUEST and RESPONSE:
-            RESPONSE.redirect(URL1 + '/manage_catalogAdvanced?manage_tabs_message=Catalog%20Cleared')
+            RESPONSE.redirect(
+              URL1 +
+              '/manage_catalogAdvanced?manage_tabs_message=Catalog%20Cleared')
 
 
     def manage_catalogFoundItems(self, REQUEST, RESPONSE, URL2, URL1,
@@ -303,18 +294,18 @@ class ZCatalog(Folder, Persistent, Implicit):
                                  obj_expr=None, obj_mtime=None,
                                  obj_mspec=None, obj_roles=None,
                                  obj_permission=None):
-    
+
         """ Find object according to search criteria and Catalog them
         """
 
         elapse = time.time()
         c_elapse = time.clock()
-        
+
         words = 0
         obj = REQUEST.PARENTS[1]
-        path = string.join(obj.getPhysicalPath(), '/')
+        path = '/'.join(obj.getPhysicalPath())
+
 
-        
         results = self.ZopeFindAndApply(obj,
                                         obj_metatypes=obj_metatypes,
                                         obj_ids=obj_ids,
@@ -331,9 +322,15 @@ class ZCatalog(Folder, Persistent, Implicit):
 
         elapse = time.time() - elapse
         c_elapse = time.clock() - c_elapse
-        
-        RESPONSE.redirect(URL1 + '/manage_catalogView?manage_tabs_message=' +
-                          urllib.quote('Catalog Updated<br>Total time: %s<br>Total CPU time: %s' % (`elapse`, `c_elapse`)))
+
+        RESPONSE.redirect(
+            URL1 +
+            '/manage_catalogView?manage_tabs_message=' +
+            urllib.quote('Catalog Updated\n'
+                         'Total time: %s\n'
+                         'Total CPU time: %s'
+                         % (`elapse`, `c_elapse`))
+            )
 
 
     def manage_addColumn(self, name, REQUEST=None, RESPONSE=None, URL1=None):
@@ -341,56 +338,71 @@ class ZCatalog(Folder, Persistent, Implicit):
         self.addColumn(name)
 
         if REQUEST and RESPONSE:
-            RESPONSE.redirect(URL1 + '/manage_catalogSchema?manage_tabs_message=Column%20Added')
+            RESPONSE.redirect(
+                URL1 +
+                '/manage_catalogSchema?manage_tabs_message=Column%20Added')
 
 
     def manage_delColumns(self, names, REQUEST=None, RESPONSE=None, URL1=None):
         """ Deprecated method. Use manage_delColumn instead. """
         # log a deprecation warning
         import warnings
-        warnings.warn("The manage_delColumns method of ZCatalog is deprecated"\
-        "since Zope 2.4.2.\n"\
-        "This method is only kept for backwards compatibility for a while\n"\
-        "and will go away in a future release.\n"\
-        "\n"\
-        "Please use instead the manage_delColumn method.\n"\
-        ,DeprecationWarning)
-        self.manage_delColumn(names, REQUEST=REQUEST, RESPONSE=RESPONSE, URL1=URL1)
+        warnings.warn(
+            "The manage_delColumns method of ZCatalog is deprecated"
+            "since Zope 2.4.2.\n"
+            "This method is only kept for backwards compatibility "
+            "for a while\n"
+            "and will go away in a future release.\n"
+            "\n"
+            "Please use instead the manage_delColumn method.\n"
+            ,DeprecationWarning)
+
+        self.manage_delColumn(names, REQUEST=REQUEST, RESPONSE=RESPONSE,
+                              URL1=URL1)
 
 
     def manage_delColumn(self, names, REQUEST=None, RESPONSE=None, URL1=None):
         """ delete a column or some columns """
         if isinstance(names, types.StringType):
             names = (names,)
-            
+
         for name in names:
             self.delColumn(name)
 
         if REQUEST and RESPONSE:
-            RESPONSE.redirect(URL1 + '/manage_catalogSchema?manage_tabs_message=Column%20Deleted')
+            RESPONSE.redirect(
+                URL1 +
+                '/manage_catalogSchema?manage_tabs_message=Column%20Deleted')
 
 
-    def manage_addIndex(self, name, type, extra=None,REQUEST=None, RESPONSE=None, URL1=None):
-        """ add an index """
+    def manage_addIndex(self, name, type, extra=None,
+                        REQUEST=None, RESPONSE=None, URL1=None):
+        """add an index """
         self.addIndex(name, type,extra)
 
         if REQUEST and RESPONSE:
-            RESPONSE.redirect(URL1 + '/manage_main?manage_tabs_message=Index%20Added')
-        
+            RESPONSE.redirect(
+                URL1 +
+                '/manage_catalogIndexes?manage_tabs_message=Index%20Added')
+
 
     def manage_deleteIndex(self, ids=None, REQUEST=None, RESPONSE=None,
         URL1=None):
         """ Deprecated method. Use manage_delIndex instead. """
         # log a deprecation warning
         import warnings
-        warnings.warn("The manage_deleteIndex method of ZCatalog is deprecated"\
-        "since Zope 2.4.2.\n"\
-        "This method is only kept for backwards compatibility for a while\n"\
-        "and will go away in a future release.\n"\
-        "\n"\
-        "Please use instead the manage_delIndex method.\n"\
-        ,DeprecationWarning)
-        self.manage_delIndex(ids=ids, REQUEST=REQUEST, RESPONSE=RESPONSE, URL1=URL1)
+        warnings.warn(
+            "The manage_deleteIndex method of ZCatalog is deprecated"
+            "since Zope 2.4.2.\n"
+            "This method is only kept for backwards compatibility for a "
+            "while\n"
+            "and will go away in a future release.\n"
+            "\n"
+            "Please use instead the manage_delIndex method.\n"
+            ,DeprecationWarning)
+
+        self.manage_delIndex(ids=ids, REQUEST=REQUEST, RESPONSE=RESPONSE,
+                             URL1=URL1)
 
 
     def manage_delIndex(self, ids=None, REQUEST=None, RESPONSE=None,
@@ -399,16 +411,18 @@ class ZCatalog(Folder, Persistent, Implicit):
         if not ids:
             return MessageDialog(title='No items specified',
                 message='No items were specified!',
-                action = "./manage_main",)
+                action = "./manage_catalogIndexes",)
 
         if isinstance(ids, types.StringType):
             ids = (ids,)
 
         for name in ids:
             self.delIndex(name)
-        
+
         if REQUEST and RESPONSE:
-            RESPONSE.redirect(URL1 + '/manage_main?manage_tabs_message=Index%20Deleted')
+            RESPONSE.redirect(
+                URL1 +
+                '/manage_catalogIndexes?manage_tabs_message=Index%20Deleted')
 
 
     def manage_clearIndex(self, ids=None, REQUEST=None, RESPONSE=None,
@@ -417,37 +431,52 @@ class ZCatalog(Folder, Persistent, Implicit):
         if not ids:
             return MessageDialog(title='No items specified',
                 message='No items were specified!',
-                action = "./manage_main",)
+                action = "./manage_catalogIndexes",)
 
         if isinstance(ids, types.StringType):
             ids = (ids,)
 
         for name in ids:
             self.clearIndex(name)
-        
+
         if REQUEST and RESPONSE:
-            RESPONSE.redirect(URL1 + '/manage_main?manage_tabs_message=Index%20Cleared')
+            RESPONSE.redirect(
+                URL1 +
+                '/manage_catalogIndexes?manage_tabs_message=Index%20Cleared')
 
 
-    def reindexIndex(self,name,REQUEST):
-        
-        paths = tuple(self._catalog.paths.values())
+    def reindexIndex(self, name, REQUEST):
+        paths = self._catalog.uids.keys()
 
         for p in paths:
             obj = self.resolve_path(p)
             if not obj:
                 obj = self.resolve_url(p, REQUEST)
             if obj is not None:
-                self.catalog_object(obj, p, idxs=[name])             
+                # don't update metadata when only reindexing a single
+                # index via the UI
+                try:
+                    self.catalog_object(obj, p, idxs=[name],
+                                        update_metadata=0)
+                except TypeError:
+                    # Fall back to Zope 2.6.2 interface. This is necessary for
+                    # products like CMF 1.4.2 and earlier that subclass from
+                    # ZCatalog and don't support the update_metadata argument.
+                    # May be removed some day.
+                    from warnings import warn
+                    warn('catalog_object interface of %s not up to date'
+                         % self.__class__.__name__,
+                         DeprecationWarning)
+                    self.catalog_object(obj, p, idxs=[name])
 
-
-    def manage_reindexIndex(self, ids=None, REQUEST=None, RESPONSE=None, URL1=None):
-        """ Reindex indexe(s) from a ZCatalog"""
+    def manage_reindexIndex(self, ids=None, REQUEST=None, RESPONSE=None,
+                            URL1=None):
+        """Reindex indexe(s) from a ZCatalog"""
         if not ids:
             return MessageDialog(title='No items specified',
                 message='No items were specified!',
-                action = "./manage_main",)
-                
+                action = "./manage_catalogIndexes",)
+
         if isinstance(ids, types.StringType):
             ids = (ids,)
 
@@ -455,7 +484,10 @@ class ZCatalog(Folder, Persistent, Implicit):
             self.reindexIndex(name, REQUEST)
 
         if REQUEST and RESPONSE:
-            RESPONSE.redirect(URL1 + '/manage_main?manage_tabs_message=Reindexing%20Performed')
+            RESPONSE.redirect(
+                URL1 +
+                '/manage_catalogIndexes'
+                '?manage_tabs_message=Reindexing%20Performed')
 
 
     def availableSplitters(self):
@@ -463,7 +495,7 @@ class ZCatalog(Folder, Persistent, Implicit):
         return Splitter.availableSplitters
 
 
-    def catalog_object(self, obj, uid=None, idxs=[]):
+    def catalog_object(self, obj, uid=None, idxs=None, update_metadata=1):
         """ wrapper around catalog """
 
         if uid is None:
@@ -473,17 +505,18 @@ class ZCatalog(Folder, Persistent, Implicit):
                     "A cataloged object must support the 'getPhysicalPath' "
                     "method if no unique id is provided when cataloging"
                     )
-            else: uid=string.join(uid(), '/')
+            else: uid='/'.join(uid())
         elif not isinstance(uid,types.StringType):
             raise CatalogError('The object unique id must be a string.')
 
-        self._catalog.catalogObject(obj, uid, None,idxs)
+        self._catalog.catalogObject(obj, uid, None, idxs,
+                                    update_metadata=update_metadata)
         # None passed in to catalogObject as third argument indicates
         # that we shouldn't try to commit subtransactions within any
         # indexing code.  We throw away the result of the call to
         # catalogObject (which is a word count), because it's
         # worthless to us here.
-        
+
         if self.threshold is not None:
             # figure out whether or not to commit a subtransaction.
             t = id(get_transaction())
@@ -502,26 +535,29 @@ class ZCatalog(Folder, Persistent, Implicit):
             # exceeded within the boundaries of the current transaction.
             if self._v_total > self.threshold:
                 get_transaction().commit(1)
-                self._p_jar.cacheFullSweep(3)
+                self._p_jar.cacheGC()
                 self._v_total = 0
 
     def uncatalog_object(self, uid):
-        """ wrapper around catalog """
+        """Wrapper around catalog """
         self._catalog.uncatalogObject(uid)
 
     def uniqueValuesFor(self, name):
-        """ returns the unique values for a given FieldIndex """
+        """Return the unique values for a given FieldIndex """
         return self._catalog.uniqueValuesFor(name)
 
     def getpath(self, rid):
-        """
-        Return the path to a cataloged object given a 'data_record_id_'
+        """Return the path to a cataloged object given a 'data_record_id_'
         """
         return self._catalog.paths[rid]
 
-    def getobject(self, rid, REQUEST=None):
+    def getrid(self, path, default=None):
+        """Return 'data_record_id_' the to a cataloged object given a 'path'
         """
-        Return a cataloged object given a 'data_record_id_'
+        return self._catalog.uids.get(path, default)
+
+    def getobject(self, rid, REQUEST=None):
+        """Return a cataloged object given a 'data_record_id_'
         """
         obj = self.aq_parent.unrestrictedTraverse(self.getpath(rid))
         if not obj:
@@ -530,6 +566,16 @@ class ZCatalog(Folder, Persistent, Implicit):
             obj = self.resolve_url(self.getpath(rid), REQUEST)
         return obj
 
+    def getMetadataForUID(self, uid):
+        """return the correct metadata given the uid, usually the path"""
+        rid = self._catalog.uids[uid]
+        return self._catalog.getMetadataForRID(rid)
+
+    def getIndexDataForUID(self, uid):
+        """return the current index contents given the uid, usually the path"""
+        rid = self._catalog.uids[uid]
+        return self._catalog.getIndexDataForRID(rid)
+
     def getMetadataForRID(self, rid):
         """return the correct metadata for the cataloged record id"""
         return self._catalog.getMetadataForRID(int(rid))
@@ -537,7 +583,7 @@ class ZCatalog(Folder, Persistent, Implicit):
     def getIndexDataForRID(self, rid):
         """return the current index contents for the specific rid"""
         return self._catalog.getIndexDataForRID(rid)
-    
+
     def schema(self):
         return self._catalog.schema.keys()
 
@@ -545,8 +591,15 @@ class ZCatalog(Folder, Persistent, Implicit):
         return self._catalog.indexes.keys()
 
     def index_objects(self):
+        # This method returns unwrapped indexes!
+        # You should probably use getIndexObjects instead
         return self._catalog.indexes.values()
 
+    def getIndexObjects(self):
+        # Return a list of wrapped(!) indexes
+        getIndex = self._catalog.getIndex
+        return [getIndex(name) for name in self.indexes()]
+
     def _searchable_arguments(self):
         r = {}
         n={'optional':1}
@@ -570,20 +623,20 @@ class ZCatalog(Folder, Persistent, Implicit):
         return r
 
     def searchResults(self, REQUEST=None, used=None, **kw):
-        """
-        Search the catalog according to the ZTables search interface.
+        """Search the catalog according to the ZTables search interface.
+
         Search terms can be passed in the REQUEST or as keyword
-        arguments. 
+        arguments.
         """
 
-        return apply(self._catalog.searchResults, (REQUEST,used), kw)
+        return self._catalog.searchResults(REQUEST, used, **kw)
 
     __call__=searchResults
 
 ## this stuff is so the find machinery works
 
     meta_types=() # Sub-object types that are specific to this object
-    
+
     # Dont need this anymore -- we inherit from object manager
     #def all_meta_types(self):
     #    pmt=()
@@ -624,7 +677,7 @@ class ZCatalog(Folder, Persistent, Implicit):
 
         This is a *great* hack.  Zope find just doesn't do what we
         need here; the ability to apply a method to all the objects
-        *as they're found* and the need to pass the object's path into 
+        *as they're found* and the need to pass the object's path into
         that method.
 
         """
@@ -634,7 +687,7 @@ class ZCatalog(Folder, Persistent, Implicit):
 
             if obj_metatypes and 'all' in obj_metatypes:
                 obj_metatypes=None
-                
+
             if obj_mtime and type(obj_mtime)==type('s'):
                 obj_mtime=DateTime(obj_mtime).timeTime()
 
@@ -643,7 +696,7 @@ class ZCatalog(Folder, Persistent, Implicit):
 
             if obj_roles and type(obj_roles) is type('s'):
                 obj_roles=[obj_roles]
-                
+
             if obj_expr:
                 # Setup expr machinations
                 md=td()
@@ -665,7 +718,7 @@ class ZCatalog(Folder, Persistent, Implicit):
         for id, ob in items:
             if pre: p="%s/%s" % (pre, id)
             else:   p=id
-            
+
             dflag=0
             if hasattr(ob, '_p_changed') and (ob._p_changed == None):
                 dflag=1
@@ -682,7 +735,7 @@ class ZCatalog(Folder, Persistent, Implicit):
                 and
                 (not obj_searchterm or
                  (hasattr(ob, 'PrincipiaSearchSource') and
-                  string.find(ob.PrincipiaSearchSource(), obj_searchterm) >= 0
+                  ob.PrincipiaSearchSource().find(obj_searchterm) >= 0
                   ))
                 and
                 (not obj_expr or expr_match(ob, obj_expr))
@@ -698,7 +751,7 @@ class ZCatalog(Folder, Persistent, Implicit):
                 else:
                     add_result((p, ob))
                     dflag=0
-                    
+
             if search_sub and hasattr(bs, 'objectItems'):
                 self.ZopeFindAndApply(ob, obj_ids, obj_metatypes,
                                       obj_searchterm, obj_expr,
@@ -712,20 +765,20 @@ class ZCatalog(Folder, Persistent, Implicit):
         return result
 
     def resolve_url(self, path, REQUEST):
-        """ 
+        """
         Attempt to resolve a url into an object in the Zope
         namespace. The url may be absolute or a catalog path
         style url. If no object is found, None is returned.
         No exceptions are raised.
         """
         script=REQUEST.script
-        if string.find(path, script) != 0:
-            path='%s/%s' % (script, path) 
+        if path.find(script) != 0:
+            path='%s/%s' % (script, path)
         try: return REQUEST.resolve_url(path)
         except: pass
 
     def resolve_path(self, path):
-        """ 
+        """
         Attempt to resolve a url into an object in the Zope
         namespace. The url may be absolute or a catalog path
         style url. If no object is found, None is returned.
@@ -755,7 +808,7 @@ class ZCatalog(Folder, Persistent, Implicit):
                 if ob is None:
                     removed.append(path)
                     continue
-            ppath = string.join(ob.getPhysicalPath(), '/')
+            ppath = '/'.join(ob.getPhysicalPath())
             if path != ppath:
                 fixed.append((path, ppath))
             else:
@@ -784,13 +837,49 @@ class ZCatalog(Folder, Persistent, Implicit):
         ct=time.clock()-ct
         return 'Finished conversion in %s seconds (%s cpu)' % (tt, ct)
 
+
+    def manage_convertIndex(self, ids, REQUEST=None, RESPONSE=None, URL1=None):
+        """convert old-style indexes to new-style indexes"""
+
+        from Products.PluginIndexes.KeywordIndex import KeywordIndex
+        from Products.PluginIndexes.FieldIndex import FieldIndex
+        from Products.PluginIndexes.TextIndex import TextIndex
+
+        converted = []
+        for id in ids:
+            idx = self.Indexes[id]
+
+            iface = getattr(idx,'__implements__',None)
+            if iface is None:
+
+                mt = idx.meta_type
+
+                converted.append(id)
+                self.delIndex(id)
+
+                if mt in ('Field Index','Keyword Index'):
+                    self.addIndex(id,mt.replace(' ',''))
+                elif mt == 'Text Index':
+                    # TODO: Lexicon handling to be added
+                    self.addIndex(id,'TextIndex')
+
+        if converted:
+            RESPONSE.redirect(
+                URL1 +
+                '/manage_main?manage_tabs_message=Indexes%20converted')
+        else:
+            RESPONSE.redirect(
+                URL1 +
+                '/manage_main?'
+                'manage_tabs_message='
+                'No%20indexes%20found%20to%20be%20converted')
+
+
     #
-    # Indexing methods 
+    # Indexing methods
     #
 
     def addIndex(self, name, type,extra=None):
-
-
         # Convert the type by finding an appropriate product which supports
         # this interface by that name.  Bleah
 
@@ -800,7 +889,7 @@ class ZCatalog(Folder, Persistent, Implicit):
         p = None
 
         for prod in products:
-            if prod['name'] == type: 
+            if prod['name'] == type:
                 p = prod
                 break
 
@@ -812,27 +901,26 @@ class ZCatalog(Folder, Persistent, Implicit):
         if base is None:
             raise ValueError, "Index type %s does not support addIndex" % type
 
-        # This code is somewhat lame but every index type has its own function
-        # signature *sigh* and there is no common way to pass additional parameters
-        # to the constructor. The suggested way for new index types is to use
-        # an "extra" record.
+        # This code is *really* lame but every index type has its own
+        # function signature *sigh* and there is no common way to pass
+        # additional parameters to the constructor. The suggested way
+        # for new index types is to use an "extra" record.
 
         if 'extra' in base.__init__.func_code.co_varnames:
-            index = apply(base,(name,), {"extra":extra,"caller":self})
+            index = base(name, extra=extra, caller=self)
+        elif 'caller' in base.__init__.func_code.co_varnames:
+            index = base(name, caller=self)
         else:
-            index = base(name,self)
-        
+            index = base(name)
 
         self._catalog.addIndex(name,index)
 
 
     def delIndex(self, name ):
-
         self._catalog.delIndex(name)
 
     def clearIndex(self, name):
-
-        self._catalog.indexes[name].clear()
+        self._catalog.getIndex(name).clear()
 
 
     def addColumn(self, name, default_value=None):
@@ -841,7 +929,7 @@ class ZCatalog(Folder, Persistent, Implicit):
     def delColumn(self, name):
         return self._catalog.delColumn(name)
 
-    
+
 Globals.default__class_init__(ZCatalog)
 
 
@@ -866,13 +954,13 @@ def expr_match(ob, ed, c=InstanceDict, r=0):
 
 def mtime_match(ob, t, q, fn=hasattr):
     if not fn(ob, '_p_mtime'):
-        return 0    
+        return 0
     return q=='<' and (ob._p_mtime < t) or (ob._p_mtime > t)
 
 def role_match(ob, permission, roles, lt=type([]), tt=type(())):
     pr=[]
     fn=pr.append
-    
+
     while 1:
         if hasattr(ob, permission):
             p=getattr(ob, permission)
