@@ -2,9 +2,9 @@
 
 Usage:
     cd lib/python
-    python Products/ZCatalog/tests/loadmail.py command args
+    python Products/ZCatalog/regressiontests/loadmail.py command args
 
-where each command has it's own command-line arguments that it expects.
+where each command has its own command-line arguments that it expects.
 
 Note that all of the commands operate on the Zope database,
 typically var/Data.fs.
@@ -27,11 +27,11 @@ where the numbers are:
 
 Commands:
 
-    base mbox max
+    base mbox [max]
 
       Build a base database by:
 
-        - Deleting ../../Data.fs
+        - Deleting ../../var/Data.fs
 
         - Starting Zope
 
@@ -41,15 +41,16 @@ Commands:
           and adding them as documents to the mail folder.
 
 
-    index threshold
+    index [threshold]
 
        Index all of the DTML documents in the database, committing
-       sub-transactions after each threshold objects.
+       sub-transactions after each threshold objects.  threshold defaults to
+       1000.
 
        If the threshold is less than the number of messages, then the
        size of the temporary sub-transaction commit file is output.
 
-    inc mbox start end [threads wait]
+    inc mbox start end [threads [wait]]
 
        Incrementally index messages start to end in unix mailbox mbox.
 
@@ -73,9 +74,9 @@ Commands:
 
           - The cpu time
 
-          - The number of ZODB trabsaction conflicts detected when reading
+          - The number of ZODB transaction conflicts detected when reading
 
-          - The number of ZODB trabsaction conflicts detected when committing
+          - The number of ZODB transaction conflicts detected when committing
 
     edit edits deletes inserts threads wait
 
@@ -88,18 +89,19 @@ Commands:
            3.41 (0, 1)
 
        as described above.
-       
+
     pdebug command args
 
        Run one of the other commands in the Python debugger.
 
 sample suite of tests::
-    
+
     cd lib/python
-    python Products/ZCatalog/tests/loadmail.py base ~/zope.mbox 1000
-    python Products/ZCatalog/tests/loadmail.py index 100
-    python Products/ZCatalog/tests/loadmail.py inc ~/python-dev.mbox 0 10 2
-    python Products/ZCatalog/tests/loadmail.py edit 10 10 10 2
+    python Products/ZCatalog/regressiontests/loadmail.py base ~/zope.mbox 1000
+    python Products/ZCatalog/regressiontests/loadmail.py index 100
+    python Products/ZCatalog/regressiontests/loadmail.py \
+                                                  inc ~/python-dev.mbox 0 10 2
+    python Products/ZCatalog/regressiontests/loadmail.py edit 10 10 10 2
 
 """
 
@@ -115,17 +117,22 @@ from urllib import quote
 
 def do(db, f, args, returnf=None):
     """Do something and measure it's impact"""
-    size=db.getSize()
-    mem=VmSize()
-    t=time.time()
-    c=time.clock()
-    r=apply(f, args)
-    t=time.time() - t
-    c=time.clock() - c
-    size=db.getSize()-size
-    mem=VmSize()-mem
-    if returnf is not None: returnf(t, c, size, mem, r)
-    else: return t, c, size, mem, r
+    t = c = size = mem = r = None
+    try:
+        size=db.getSize()
+        mem=VmSize()
+        t=time.time()
+        c=time.clock()
+        r=apply(f, args)
+        t=time.time() - t
+        c=time.clock() - c
+        size=db.getSize()-size
+        mem=VmSize()-mem
+    finally:
+        if returnf is not None:
+            returnf(t, c, size, mem, r)
+        else:
+            return t, c, size, mem, r
 
 def loadmessage(dest, message, i, body=None, headers=None):
     if body is None: body=message.fp.read()
@@ -152,7 +159,7 @@ def loadmessage(dest, message, i, body=None, headers=None):
             try: doc.manage_addProperty(name, v, type)
             except: pass
 
-def loadmail(dest, name, mbox, printstat=0, max=99999999):
+def loadmail(dest, name, mbox, printstat=0, max=-1):
 
     try:
         import Products.BTreeFolder.BTreeFolder
@@ -160,20 +167,23 @@ def loadmail(dest, name, mbox, printstat=0, max=99999999):
         dest.manage_addFolder(name)
     else:
         Products.BTreeFolder.BTreeFolder.manage_addBTreeFolder(dest, name)
-    
+
     dest=getattr(dest, name)
     f=open(mbox)
     mb=mailbox.UnixMailbox(f)
     i=0
     message=mb.next()
     while message:
-        if i > max: break
+        if max >= 0 and i > max:
+            break
         if i%100 == 0 and printstat:
-            sys.stdout.write("\t%s\t%s\t\r" % (i, f.tell()))
+            fmt = "\t%s\t%s\t\r"
+            if os.environ.get('TERM') in ('dumb', 'emacs'):
+                fmt = "\t%s\t%s\t\n"
+            sys.stdout.write(fmt % (i, f.tell()))
             sys.stdout.flush()
         if i and (i%5000 == 0):
             get_transaction().commit()
-            dest._p_jar._cache.invalidate(None)
             dest._p_jar._cache.minimize()
 
         loadmessage(dest, message, i)
@@ -181,9 +191,10 @@ def loadmail(dest, name, mbox, printstat=0, max=99999999):
         message=mb.next()
 
     dest.number_of_messages=i
+    print
     get_transaction().commit()
 
-def loadinc(name, mb, printstat=0, max=99999999, wait=1):
+def loadinc(name, mb, f, printstat=0, max=99999999, wait=1):
     from ZODB.POSException import ConflictError
     from time import sleep
     from whrandom import uniform
@@ -227,27 +238,21 @@ def loadinc(name, mb, printstat=0, max=99999999, wait=1):
 
         doc=app=mdest=0
         jar.close()
-            
-            
+
+
     if printstat: sys.stdout.write("\t%s\t%s\t\n" % (i, f.tell()))
     sys.stdout.flush()
     return rconflicts, wconflicts
 
-def buildbase():
-    try: os.unlink('../../var/Data.fs')
-    except: pass
-    import Zope, Products.ZCatalog.ZCatalog
-    app=Zope.app()
-    Products.ZCatalog.ZCatalog.manage_addZCatalog(app, 'cat', '')
-    get_transaction().commit()
-    return app
-
 def base():
     try: os.unlink('../../var/Data.fs')
     except: pass
     import Zope
     app=Zope.app()
-    max=atoi(sys.argv[3])
+    if len(sys.argv) > 3:
+        max = atoi(sys.argv[3])
+    else:
+        max = -1
     print do(Zope.DB, loadmail, (app, 'mail', sys.argv[2], 1, max))
     Zope.DB.close()
 
@@ -259,7 +264,7 @@ def indexf(app):
     r.PARENTS=[0, app.mail]
     app.cat.manage_catalogFoundItems(r,r,'','',['DTML Document'])
     get_transaction().commit()
-    
+
 def index():
     os.environ['STUPID_LOG_FILE']=''
     os.environ['STUPID_LOG_SEVERITY']='-111'
@@ -267,15 +272,34 @@ def index():
     import AccessControl.SecurityManagement, AccessControl.SpecialUsers
     app=Zope.app()
     Products.ZCatalog.ZCatalog.manage_addZCatalog(app, 'cat', '')
-    app.cat.threshold=atoi(sys.argv[2])
-    app.cat._catalog.delIndex('bobobase_modification_time')
+    try:
+        app.cat.threshold = atoi(sys.argv[2])
+    except IndexError:
+        app.cat.threashold = 1000
+
+    from Products.ZCTextIndex.ZCTextIndex \
+         import PLexicon
+    from Products.ZCTextIndex.Lexicon \
+         import Splitter, CaseNormalizer
+
+    app.cat._setObject('lex',
+                       PLexicon('lex', '', Splitter(), CaseNormalizer())
+                       )
+
+    class extra:
+        doc_attr = 'PrincipiaSearchSource'
+        lexicon_id = 'lex'
+        index_type = 'Okapi BM25 Rank'
+
+    app.cat.addIndex('PrincipiaSearchSource', 'ZCTextIndex', extra)
+
     get_transaction().commit()
     system = AccessControl.SpecialUsers.system
     AccessControl.SecurityManagement.newSecurityManager(None, system)
     r=RE()
     r.PARENTS=[app.cat, app]
     print do(Zope.DB, indexf, (app,))
-    hist(sys.argv[2])
+    #hist(sys.argv[2])
     Zope.DB.close()
 
 def initmaili(n):
@@ -332,7 +356,8 @@ def inc():
         amin=min+i*count
         dest='maili%s' % amin
         initmaili(dest)
-        mb=mailbox.UnixMailbox(open(mbox))
+        f = open(mbox)
+        mb=mailbox.UnixMailbox(f)
         j=0
         while j < amin:
             mb.next()
@@ -342,7 +367,7 @@ def inc():
         def returnf(t, c, size, mem, r, lock=lock):
             print c, r
             lock.release()
-        argss.append((lock, (dest, mb, 0, count, wait), returnf))
+        argss.append((lock, (dest, mb, f, 1, count, wait), returnf))
 
     for lock, args, returnf in argss:
         thread.start_new_thread(do, (Zope.DB, loadinc, args, returnf))
@@ -357,8 +382,8 @@ def inc():
 
     print t, c, size, mem
 
-    hist("%s-%s-%s" % (omin, count, threads))
-    
+    #hist("%s-%s-%s" % (omin, count, threads))
+
     Zope.DB.close()
 
 words=['banishment', 'indirectly', 'imprecise', 'peeks',
@@ -589,7 +614,7 @@ def incedit(edits, wait, ndel=20, nins=20):
 
         doc=app=0
         jar.close()
-        
+
     return rconflicts, wconflicts
 
 def edit():
@@ -652,7 +677,7 @@ def edit():
     print t, c, size, mem
 
     #hist("e%s" % (threads))
-    
+
     Zope.DB.close()
 
 def VmSize():
