@@ -406,7 +406,7 @@ static char Splitter_module_documentation[] =
 "\n"
 "for use in an inverted index\n"
 "\n"
-"$Id: Splitter.c,v 1.18 2001/11/28 15:51:11 matt Exp $\n"
+"$Id: Splitter.c,v 1.19 2002/03/21 15:48:55 htrd Exp $\n"
 ;
 
 
@@ -414,7 +414,6 @@ void
 initSplitter(void)
 {
   PyObject *m, *d;
-  char *rev="$Revision: 1.18 $";
   
   /* Create the module and add the functions */
   m = Py_InitModule4("Splitter", Splitter_module_methods,
@@ -423,8 +422,6 @@ initSplitter(void)
   
   /* Add some symbolic constants to the module */
   d = PyModule_GetDict(m);
-  PyDict_SetItemString(d, "__version__",
-		       PyString_FromStringAndSize(rev+11,strlen(rev+11)-2));
 
   if (PyErr_Occurred()) Py_FatalError("can't initialize module Splitter");
 }
