@@ -1,19 +1,19 @@
 ##############################################################################
 #
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
-# 
+#
 ##############################################################################
 __doc__='''Generic Database Connection Support
 
-$Id: Connection.py,v 1.34 2001/11/28 15:51:13 matt Exp $'''
-__version__='$Revision: 1.34 $'[11:-2]
+$Id: Connection.py,v 1.35.6.3 2004/01/08 21:13:12 tseaver Exp $'''
+__version__='$Revision: 1.35.6.3 $'[11:-2]
 
 import Globals, OFS.SimpleItem, AccessControl.Role, Acquisition, sys
 from DateTime import DateTime
@@ -23,6 +23,9 @@ from string import find, join, split
 from Aqueduct import custom_default_report
 from cStringIO import StringIO
 from Results import Results
+from sys import exc_info
+from zLOG import LOG, ERROR
+from cgi import escape
 import DocumentTemplate, RDB
 
 class Connection(
@@ -30,7 +33,7 @@ class Connection(
     AccessControl.Role.RoleManager,
     OFS.SimpleItem.Item,
     Acquisition.Implicit,
-    ):    
+    ):
 
     # Specify definitions for tabs:
     manage_options=(
@@ -42,7 +45,7 @@ class Connection(
         +AccessControl.Role.RoleManager.manage_options
         +OFS.SimpleItem.Item.manage_options
         )
- 
+
     # Specify how individual operations add up to "permissions":
     __ac_permissions__=(
         ('View management screens', ('manage_main',)),
@@ -63,13 +66,17 @@ class Connection(
         Globals.Persistent.__setstate__(self, state)
         if self.connection_string:
             try: self.connect(self.connection_string)
-            except: pass
+            except:
+                LOG('Shared.DC.ZRDB.Connection',
+                    ERROR,
+                    'Error connecting to relational database.',
+                    error=exc_info())
 
     def title_and_id(self):
         s=Connection.inheritedAttribute('title_and_id')(self)
         if hasattr(self, '_v_connected') and self._v_connected:
             s="%s, which is connected" % s
-        else: 
+        else:
             s="%s, which is <font color=red> not connected</font>" % s
         return s
 
@@ -77,7 +84,7 @@ class Connection(
         s=Connection.inheritedAttribute('title_or_id')(self)
         if hasattr(self, '_v_connected') and self._v_connected:
             s="%s (connected)" % s
-        else: 
+        else:
             s="%s (<font color=red> not connected</font>)" % s
         return s
 
@@ -87,7 +94,7 @@ class Connection(
         self.title=title
         self.connection_string=connection_string
         if check: self.connect(connection_string)
-    
+
     manage_properties=DTMLFile('dtml/connectionEdit', globals())
     def manage_edit(self, title, connection_string, check=None, REQUEST=None):
         """Change connection
@@ -96,7 +103,7 @@ class Connection(
         if REQUEST is not None:
             return MessageDialog(
                 title='Edited',
-                message='<strong>%s</strong> has been edited.' % self.id,
+                message='<strong>%s</strong> has been edited.' % escape(self.id),
                 action ='./manage_main',
                 )
 
@@ -116,7 +123,7 @@ class Connection(
 
         if REQUEST is None:
             return result       #return unadulterated result objects
-        
+
         if result._searchable_result_columns():
             r=custom_default_report(self.id, result)
         else:
@@ -132,16 +139,23 @@ class Connection(
         report=apply(report,(self,REQUEST),{self.id:result})
 
         return report
-                
+
 
     manage_main=DTMLFile('dtml/connectionStatus', globals())
 
-    def manage_close_connection(self, REQUEST):
+    def manage_close_connection(self, REQUEST=None):
         " "
-        try: self._v_database_connection.close()
-        except: pass
+        try: 
+            if hasattr(self,'_v_database_connection'):
+                self._v_database_connection.close()
+        except:
+            LOG('Shared.DC.ZRDB.Connection',
+                ERROR,
+                'Error closing relational database connection.',
+                error=exc_info())
         self._v_connected=''
-        return self.manage_main(self, REQUEST)
+        if REQUEST is not None:
+            return self.manage_main(self, REQUEST)
 
     def manage_open_connection(self, REQUEST=None):
         " "
@@ -159,9 +173,7 @@ class Connection(
                 '''The database connection is not connected''')
 
     def connect(self,s):
-        try: self._v_database_connection.close()
-        except: pass
-        self._v_connected=''
+        self.manage_close_connection()
         DB=self.factory()
         try:
             try:
