@@ -1,23 +1,24 @@
 ##############################################################################
 #
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
-# 
+#
 ##############################################################################
 __doc__='''Search Interface Wizard
 
-$Id: Search.py,v 1.18 2001/11/28 15:51:13 matt Exp $'''
-__version__='$Revision: 1.18 $'[11:-2]
+$Id: Search.py,v 1.20.6.1 2004/01/08 21:13:12 tseaver Exp $'''
+__version__='$Revision: 1.20.6.1 $'[11:-2]
 
 from Globals import DTMLFile
 from Aqueduct import custom_default_report, custom_default_zpt_report, nicify, Args
 from string import join
+from cgi import escape
 from AccessControl import getSecurityManager
 
 addForm=DTMLFile('dtml/searchAdd', globals())
@@ -39,14 +40,14 @@ def manage_addZSearch(self, report_id, report_title, report_style,
     arguments={}
     keys=[]
 
-    checkPermission=getSecurityManager().checkPermission    
+    checkPermission=getSecurityManager().checkPermission
 
     for q in qs:
         url=q.absolute_url()
         if input_id:
             for name, arg in q._searchable_arguments().items():
                 if len(qs) > 1: key="%s/%s" % (id,name)
-                else: key=name 
+                else: key=name
                 arguments[key]=arg
                 keys.append(key)
         if q._searchable_result_columns() is None:
@@ -57,7 +58,7 @@ def manage_addZSearch(self, report_id, report_title, report_style,
                 cannot be generated.  Before creating a report
                 from this query, you must try out the query.  To
                 try out the query, <a href="%s">click here</a>.
-                """ % (q.title_and_id(), url))
+                """ % (escape(q.title_and_id()), escape(url, 1)))
 
     if object_type == 'dtml_methods':
 
@@ -75,7 +76,7 @@ def manage_addZSearch(self, report_id, report_title, report_style,
         self.manage_addDocument(
             report_id,report_title,
             ('<dtml-var standard_html_header>\n%s\n'
-             '<dtml-var standard_html_footer>' % 
+             '<dtml-var standard_html_footer>' %
              join(map(lambda q, report_style=report_style:
                       custom_default_report(q.id, q, no_table=report_style), qs),
                   '\n<hr>\n')))
@@ -99,13 +100,13 @@ def manage_addZSearch(self, report_id, report_title, report_style,
         self.manage_addProduct['PageTemplates'].manage_addPageTemplate(
             report_id,report_title,
             ('<html><body>\n%s\n'
-             '</body></html>' % 
+             '</body></html>' %
              join(map(lambda q, report_style=report_style:
                       custom_default_zpt_report(q.id, q, no_table=report_style), qs),
                   '\n<hr>\n')))
 
         if REQUEST: return self.manage_main(self,REQUEST)
-        
+
 
 def ZQueryIds(self):
     # Note that report server configurations will expend on this
@@ -133,26 +134,26 @@ def ZQueryIds(self):
         # Now extend search to parent
         try: o=o.aq_parent
         except: return t
-  
+
         if n > 100: return t # Seat belt
         n=n+1
 
 def _getquery(self,id):
 
-        o=self
-        i=0
-        while 1:
-            __traceback_info__=o
-            q=getattr(o,id)
-            try:
-                if hasattr(q,'_searchable_arguments'):
-                    try: q=q.__of__(self.aq_parent)
-                    except: pass
-                    return q
-            except: pass
-            if i > 100: raise AttributeError, id
-            i=i+1
-            o=o.aq_parent
+    o=self
+    i=0
+    while 1:
+        __traceback_info__=o
+        q=getattr(o,id)
+        try:
+            if hasattr(q,'_searchable_arguments'):
+                try: q=q.__of__(self.aq_parent)
+                except: pass
+                return q
+        except: pass
+        if i > 100: raise AttributeError, id
+        i=i+1
+        o=o.aq_parent
 
 
 def default_input_form(arguments,action='query',
@@ -198,7 +199,7 @@ def default_input_form(arguments,action='query',
             '<h2><dtml-var document_title></h2>\n'
             'This query requires no input.<p>\n'
             '<input type="SUBMIT" name="SUBMIT" value="Submit Query">\n'
-            '</td></tr>\n</table>\n</form>\n'
+            '</form>\n'
             '<dtml-var standard_html_footer>\n'
             % (tabs, action)
             )
@@ -213,7 +214,7 @@ def default_input_zpt_form(arguments,action='query',
             "%s\n%s%s" % (
                 '<html><body>\n%s\n'
                 '<form action="%s" method="get">\n'
-                '<h2><dtml-var document_title></h2>\n'
+                '<h2 tal:content="template/title_or_id">Title</h2>\n'
                 'Enter query parameters:<br>'
                 '<table>\n'
                 % (tabs,action),
@@ -245,21 +246,10 @@ def default_input_zpt_form(arguments,action='query',
         return (
             '<html><body>\n%s\n'
             '<form action="%s" method="get">\n'
-            '<h2><dtml-var document_title></h2>\n'
-            'This query requires no input.<p>\n'
+            '<h2 tal:content="template/title_or_id">Title</h2>\n'
+            '<p>This query requires no input.</p>\n'
             '<input type="SUBMIT" name="SUBMIT" value="Submit Query">\n'
-            '</td></tr>\n</table>\n</form>\n'
+            '</form>\n'
             '</body></html>\n'
             % (tabs, action)
             )
-
-
-
-
-
-
-
-
-
-
-
