@@ -1,14 +1,14 @@
 ##############################################################################
 #
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
-# 
+#
 ##############################################################################
 """Provide support for linking an external transaction manager with Zope's
 """
@@ -26,7 +26,7 @@ class TM:
     """
 
     _registered=None
-    
+
     def _begin(self): pass
 
     def _register(self):
@@ -52,7 +52,7 @@ class TM:
 
     def tpc_finish(self, *ignored):
 
-        if self._finalize:	
+        if self._finalize:
             try: self._finish()
             finally: self._registered=0
 
@@ -62,6 +62,14 @@ class TM:
 
     tpc_abort = abort
 
+    def sortKey(self, *ignored):
+        """ The sortKey method is used for recent ZODB compatibility which
+            needs to have a known commit order for lock acquisition.  Most
+            DA's talking to RDBMS systems do not care about commit order, so
+            return the constant 1
+        """
+        return 1
+
 class Surrogate:
 
     def __init__(self, db):
