@@ -1,30 +1,31 @@
 ##############################################################################
 #
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
-# 
+#
 ##############################################################################
 """Rendering object hierarchies as Trees
 """
-__rcs_id__='$Id: TreeTag.py,v 1.49 2001/11/28 15:51:17 matt Exp $'
-__version__='$Revision: 1.49 $'[11:-2]
+__rcs_id__='$Id: TreeTag.py,v 1.53.6.1 2004/01/08 20:42:04 tseaver Exp $'
+__version__='$Revision: 1.53.6.1 $'[11:-2]
 
 from DocumentTemplate.DT_Util import *
 from DocumentTemplate.DT_String import String
 
-from string import join, split, rfind, find, translate, replace
+from cPickle import dumps
+from string import translate
 from urllib import quote, unquote
-from zlib import compress, decompress
+from zlib import compress, decompressobj
 from binascii import b2a_base64, a2b_base64
 import re
 
-tbl=join(map(chr, range(256)),'')
+tbl=''.join(map(chr, range(256)))
 tplus=tbl[:ord('+')]+'-'+tbl[ord('+')+1:]
 tminus=tbl[:ord('-')]+'+'+tbl[ord('-')+1:]
 
@@ -35,7 +36,7 @@ class Tree:
 
     def __init__(self, blocks):
         tname, args, section = blocks[0]
-        args=parse_params(args, name=None, expr=None, nowrap=1, 
+        args=parse_params(args, name=None, expr=None, nowrap=1,
                           expand=None, leaves=None,
                           header=None, footer=None,
                           branches=None, branches_expr=None,
@@ -71,11 +72,11 @@ class Tree:
         if prefix and not simple_name(prefix):
             raise ParseError, _tm(
                 'prefix is not a simple name', 'tree')
-        
+
         self.__name__ = name
         self.section=section.blocks
         self.args=args
-            
+
 
     def render(self,md):
         args=self.args
@@ -103,15 +104,15 @@ def tpRender(self, md, section, args,
       ['eagle'], # eagle is open
       ['eagle'], ['jeep', [1983, 1985]]  # eagle, jeep, 1983 jeep and 1985 jeep
 
-    where the items are object ids. The state will be converted to a
-    compressed and base64ed string that gets unencoded, uncompressed, 
-    and evaluated on the other side.
+    where the items are object ids. The state will be pickled to a
+    compressed and base64ed string that gets unencoded, uncompressed,
+    and unpickled on the other side.
 
     Note that ids used in state need not be connected to urls, since
     state manipulation is internal to rendering logic.
 
-    Note that to make eval safe, we do not allow the character '*' in
-    the state.
+    Note that to make unpickling safe, we use the MiniPickle module,
+    that only creates safe objects
     """
 
     data=[]
@@ -119,7 +120,7 @@ def tpRender(self, md, section, args,
     idattr=args['id']
     if hasattr(self, idattr):
         id=getattr(self, idattr)
-        if not simple_type(type(id)): id=id()            
+        if not simple_type(type(id)): id=id()
     elif hasattr(self, '_p_oid'): id=oid(self)
     else: id=pyid(self)
 
@@ -130,7 +131,7 @@ def tpRender(self, md, section, args,
         state=md['tree-state']
         diff=md['tree-diff']
         substate=md['-tree-substate-']
-        colspan=md['tree-colspan']      
+        colspan=md['tree-colspan']
         level=md['tree-level']
 
     except KeyError:
@@ -178,7 +179,7 @@ def tpRender(self, md, section, args,
 
         url=''
         root=md['URL']
-        l=rfind(root,'/')
+        l=root.rfind('/')
         if l >= 0: root=root[l+1:]
 
     treeData={'tree-root-url': root,
@@ -188,8 +189,8 @@ def tpRender(self, md, section, args,
     prefix = args.get('prefix')
     if prefix:
         for k, v in treeData.items():
-            treeData[prefix + replace(k[4:], '-', '_')] = v
-    
+            treeData[prefix + k[4:].replace('-', '_')] = v
+
     md._push(InstanceDict(self, md))
     md._push(treeData)
 
@@ -202,7 +203,7 @@ def tpRender(self, md, section, args,
         state=encode_seq(state)
         md['RESPONSE'].setCookie('tree-s',state)
 
-    return join(data,'')
+    return ''.join(data)
 
 def tpRenderTABLE(self, id, root_url, url, state, substate, diff, data,
                   colspan, section, md, treeData, level=0, args=None,
@@ -217,7 +218,7 @@ def tpRenderTABLE(self, id, root_url, url, state, substate, diff, data,
         urlattr=args['url']
         if urlattr and hasattr(self, urlattr):
             tpUrl=getattr(self, urlattr)
-            if not simple_type(type(tpUrl)): tpUrl=tpUrl()            
+            if not simple_type(type(tpUrl)): tpUrl=tpUrl()
             url = (url and ('%s/%s' % (url, tpUrl))) or tpUrl
             root_url = root_url or tpUrl
 
@@ -295,17 +296,17 @@ def tpRenderTABLE(self, id, root_url, url, state, substate, diff, data,
     diff.append(id)
 
 
-    _td_colspan='<TD COLSPAN="%s" NOWRAP></TD>'
-    _td_single ='<TD WIDTH="16" NOWRAP></TD>'
+    _td_colspan='<td colspan="%s" style="white-space: nowrap"></td>'
+    _td_single ='<td width="16" style="white-space: nowrap"></td>'
 
     sub=None
     if substate is state:
-        output('<TABLE CELLSPACING="0">\n')
+        output('<table cellspacing="0">\n')
         sub=substate[0]
         exp=items
     else:
         # Add prefix
-        output('<TR>\n')
+        output('<tr>\n')
 
         # Add +/- icon
         if items:
@@ -314,7 +315,7 @@ def tpRenderTABLE(self, id, root_url, url, state, substate, diff, data,
                 elif level > 1: output(_td_single * (level-1))
                 output(_td_single)
                 output('\n')
-            output('<TD WIDTH="16" VALIGN="TOP" NOWRAP>')
+            output('<td width="16" valign="top" style="white-space: nowrap">')
             for i in range(len(substate)):
                 sub=substate[i]
                 if sub[0]==id:
@@ -323,11 +324,11 @@ def tpRenderTABLE(self, id, root_url, url, state, substate, diff, data,
 
             ####################################
             # Mostly inline encode_seq for speed
-            s=compress(str(diff))
+            s=compress(dumps(diff,1))
             if len(s) > 57: s=encode_str(s)
             else:
                 s=b2a_base64(s)[:-1]
-                l=find(s,'=')
+                l=s.find('=')
                 if l >= 0: s=s[:l]
             s=translate(s, tplus)
             ####################################
@@ -343,30 +344,30 @@ def tpRenderTABLE(self, id, root_url, url, state, substate, diff, data,
 
             if exp:
                 ptreeData['tree-item-expanded']=1
-                output('<A NAME="%s" HREF="%s?%stree-c=%s#%s">'
-                       '<IMG SRC="%s/p_/mi" ALT="-" BORDER=0></A>' %
+                output('<a name="%s" href="%s?%stree-c=%s#%s">'
+                       '<img src="%s/p_/mi" alt="-" border="0" /></a>' %
                        (id, root_url, param, s, id, script))
             else:
-                output('<A NAME="%s" HREF="%s?%stree-e=%s#%s">'
-                       '<IMG SRC="%s/p_/pl" ALT="+" BORDER=0></A>' %
+                output('<a name="%s" href="%s?%stree-e=%s#%s">'
+                       '<img src="%s/p_/pl" alt="+" border="0" /></a>' %
                        (id, root_url, param, s, id, script))
-            output('</TD>\n')
+            output('</td>\n')
 
         else:
             if level > 2:   output(_td_colspan % level)
             elif level > 0: output(_td_single  * level)
             output(_td_single)
             output('\n')
-            
-    
+
+
         # add item text
         dataspan=colspan-level
-        output('<TD%s%s VALIGN="TOP" ALIGN="LEFT">' %
-               ((dataspan > 1 and (' COLSPAN="%s"' % dataspan) or ''),
-               (have_arg('nowrap') and args['nowrap'] and ' NOWRAP' or ''))
+        output('<td%s%s valign="top" align="left">' %
+               ((dataspan > 1 and (' colspan="%s"' % dataspan) or ''),
+               (have_arg('nowrap') and args['nowrap'] and ' style="white-space: nowrap"' or ''))
                )
         output(render_blocks(section, md))
-        output('</TD>\n</TR>\n')
+        output('</td>\n</tr>\n')
 
 
     if exp:
@@ -385,14 +386,14 @@ def tpRenderTABLE(self, id, root_url, url, state, substate, diff, data,
                 output(doc(
                     None, md,
                     standard_html_header=(
-                        '<TR>%s<TD WIDTH="16" NOWRAP></TD>'
-                        '<TD%s VALIGN="TOP">'
+                        '<tr>%s<td width="16" style="white-space: nowrap"></td>'
+                        '<td%s valign="top">'
                         % (h,
-                           (dataspan > 1 and (' COLSPAN="%s"' % dataspan)
+                           (dataspan > 1 and (' colspan="%s"' % dataspan)
                             or ''))),
-                    standard_html_footer='</TD></TR>',
+                    standard_html_footer='</td></tr>',
                     ))
-            
+
         if items==1:
             # leaves
             if have_arg('leaves'):
@@ -406,12 +407,12 @@ def tpRenderTABLE(self, id, root_url, url, state, substate, diff, data,
                     try: output(doc(
                         None,md,
                         standard_html_header=(
-                            '<TR>%s<TD WIDTH="16" NOWRAP></TD>'
-                            '<TD%s VALIGN="TOP">'
+                            '<tr>%s<td width="16" style="white-space: nowrap"></td>'
+                            '<td%s valign="top">'
                             % (h,
                                (dataspan > 1 and
-                                (' COLSPAN="%s"' % dataspan) or ''))),
-                        standard_html_footer='</TD></TR>',
+                                (' colspan="%s"' % dataspan) or ''))),
+                        standard_html_footer='</td></tr>',
                         ))
                     finally: md._pop(1)
         elif have_arg('expand'):
@@ -425,12 +426,12 @@ def tpRenderTABLE(self, id, root_url, url, state, substate, diff, data,
                 try: output(doc(
                     None,md,
                     standard_html_header=(
-                        '<TR>%s<TD WIDTH="16" NOWRAP></TD>'
-                        '<TD%s VALIGN="TOP">'
+                        '<tr>%s<td width="16" style="white-space: nowrap"></td>'
+                        '<td%s valign="top">'
                         % (h,
                            (dataspan > 1 and
-                            (' COLSPAN="%s"' % dataspan) or ''))),
-                    standard_html_footer='</TD></TR>',
+                            (' colspan="%s"' % dataspan) or ''))),
+                    standard_html_footer='</td></tr>',
                     ))
                 finally: md._pop(1)
         else:
@@ -464,16 +465,16 @@ def tpRenderTABLE(self, id, root_url, url, state, substate, diff, data,
                 output(doc(
                     None, md,
                     standard_html_header=(
-                        '<TR>%s<TD WIDTH="16" NOWRAP></TD>'
-                        '<TD%s VALIGN="TOP">'
+                        '<tr>%s<td width="16" style="white-space: nowrap"></td>'
+                        '<td%s valign="top">'
                         % (h,
-                           (dataspan > 1 and (' COLSPAN="%s"' % dataspan)
+                           (dataspan > 1 and (' colspan="%s"' % dataspan)
                             or ''))),
-                    standard_html_footer='</TD></TR>',
+                    standard_html_footer='</td></tr>',
                     ))
 
     del diff[-1]
-    if not diff: output('</TABLE>\n')
+    if not diff: output('</table>\n')
 
     return data
 
@@ -514,19 +515,19 @@ def apply_diff(state, diff, expand):
 
 def encode_seq(state):
     "Convert a sequence to an encoded string"
-    state=compress(str(state))
+    state=compress(dumps(state))
     l=len(state)
 
     if l > 57:
         states=[]
         for i in range(0,l,57):
             states.append(b2a_base64(state[i:i+57])[:-1])
-        state=join(states,'')
+        state=''.join(states)
     else: state=b2a_base64(state)[:-1]
 
-    l=find(state,'=')
+    l=state.find('=')
     if l >= 0: state=state[:l]
-    
+
     state=translate(state, tplus)
     return state
 
@@ -538,12 +539,12 @@ def encode_str(state):
         states=[]
         for i in range(0,l,57):
             states.append(b2a_base64(state[i:i+57])[:-1])
-        state=join(states,'')
+        state=''.join(states)
     else: state=b2a_base64(state)[:-1]
 
-    l=find(state,'=')
+    l=state.find('=')
     if l >= 0: state=state[:l]
-        
+
     state=translate(state, tplus)
     return state
 
@@ -566,7 +567,7 @@ def decode_seq(state):
             k=l%4
             if k: state=state+'='*(4-k)
             states.append(a2b_base64(state))
-        state=join(states,'')
+        state=''.join(states)
     else:
         l=len(state)
         k=l%4
@@ -574,10 +575,20 @@ def decode_seq(state):
         state=a2b_base64(state)
 
     state=decompress(state)
-    if find(state,'*') >= 0: raise 'Illegal State', state
-    try: return list(eval(state,{'__builtins__':{}}))
+    try: return list(MiniUnpickler(StringIO(state)).load())
     except: return []
-    
+
+def decompress(input,max_size=10240):
+    # This sillyness can go away in python 2.2
+    d = decompressobj()
+    output = ''
+    while input:
+        fragment_size = max(1,(max_size-len(output))/1000)
+        fragment,input = input[:fragment_size],input[fragment_size:]
+        output += d.decompress(fragment)
+        if len(output)>max_size:
+            raise ValueError('Compressed input too large')
+    return output+d.flush()
 
 def tpStateLevel(state, level=0):
     for sub in state:
@@ -604,7 +615,7 @@ def tpValuesIds(self, get_items, args,
 
                     if hasattr(item, idattr):
                         id=getattr(item, idattr)
-                        if not simple_type(type(id)): id=id()            
+                        if not simple_type(type(id)): id=id()
                     elif hasattr(item, '_p_oid'): id=oid(item)
                     else: id=pyid(item)
 
@@ -619,8 +630,81 @@ def tpValuesIds(self, get_items, args,
 
 def oid(self):
     return b2a_base64(str(self._p_oid))[:-1]
-    
+
 
 #icoSpace='<IMG SRC="Blank_icon" BORDER="0">'
 #icoPlus ='<IMG SRC="Plus_icon" BORDER="0">'
 #icoMinus='<IMG SRC="Minus_icon" BORDER="0">'
+
+
+
+
+
+###############################################################################
+## Everthing below here should go in a MiniPickle.py module, but keeping it
+## internal makes an easier patch
+
+
+import pickle
+from cStringIO import StringIO
+
+
+
+class MiniUnpickler(pickle.Unpickler):
+    """An unpickler that can only handle simple types.
+    """
+    def refuse_to_unpickle(self):
+        raise pickle.UnpicklingError, 'Refused'
+
+    dispatch = pickle.Unpickler.dispatch.copy()
+
+    for k,v in dispatch.items():
+        if k=='' or k in '().012FGIJKLMNTUVX]adeghjlpqrstu}':
+            # This key is necessary and safe, so leave it in the map
+            pass
+        else:
+            dispatch[k] = refuse_to_unpickle
+            # Anything unnecessary is banned, but here is some logic to explain why
+            if k in [pickle.GLOBAL, pickle.OBJ, pickle.INST, pickle.REDUCE, pickle.BUILD]:
+                # These are definite security holes
+                pass
+            elif k in [pickle.PERSID, pickle.BINPERSID]:
+                # These are just unnecessary
+                pass
+            elif k in [pickle.STRING]:
+                # This one is controversial: A string is harmlessm, but the
+                # implementation of pickle leaks memory (strings may be interned)
+                # The problem can be avoided by using binary pickles.
+                pass
+
+    del k
+    del v
+
+def _should_succeed(x,binary=1):
+    if x != MiniUnpickler(StringIO(pickle.dumps(x,binary))).load():
+        raise ValueError(x)
+
+def _should_fail(x,binary=1):
+    try:
+        MiniUnpickler(StringIO(pickle.dumps(x,binary))).load()
+        raise ValueError(x)
+    except pickle.UnpicklingError, e:
+        if e[0]!='Refused': raise ValueError(x)
+
+class _junk_class: pass
+
+def _test():
+    _should_fail('hello',0)
+    _should_succeed('hello')
+    _should_succeed(1)
+    _should_succeed(1L)
+    _should_succeed(1.0)
+    _should_succeed((1,2,3))
+    _should_succeed([1,2,3])
+    _should_succeed({1:2,3:4})
+    _should_fail(open)
+    _should_fail(_junk_class)
+    _should_fail(_junk_class())
+
+# Test MiniPickle on every import
+_test()
