@@ -1,21 +1,20 @@
 ##############################################################################
 #
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
-# 
+#
 ##############################################################################
 """Basic Item class and class manager
 """
 
 import Acquisition, ExtensionClass, Globals, OFS.PropertySheets, OFS.Folder
 from AccessControl.Permission import pname
-from string import strip
 import App.Dialogs, ZClasses, App.Factory, App.Product, App.ProductRegistry
 import ZClassOwner
 from AccessControl.PermissionMapping import aqwrap, PermissionMapper
@@ -43,7 +42,7 @@ class ZClassMethodsSheet(
     # Hijinks to let us create factories and classes within classes.
 
     #meta_types=App.Product.Product.meta_types
-    
+
     meta_types=(
         {'name': 'Z Class',
          'action':'manage_addZClassForm'},
@@ -54,7 +53,7 @@ class ZClassMethodsSheet(
          'action': 'manage_addPropertyInterfaceForm'
          },
         )
-    
+
     def manage_addPrincipiaFactory(
         self, id, title, object_type, initial, permission=None, REQUEST=None):
         ' '
@@ -68,7 +67,7 @@ class ZClassMethodsSheet(
 
     def _getProductRegistryMetaTypes(self):
         return self.getClassAttr('_zclass_method_meta_types',())
-    
+
     def _setProductRegistryMetaTypes(self, v):
         return self.setClassAttr('_zclass_method_meta_types', v)
 
@@ -121,26 +120,26 @@ class ZClassMethodsSheet(
         return id+' '
 
     def _setOb(self, id, object):
-        self.setClassAttr(strip(id), MWp(object))
+        self.setClassAttr(id.strip(), MWp(object))
 
     def _delOb(self, id):
-        self.delClassAttr(strip(id))
+        self.delClassAttr(id.strip())
 
     def _delObject(self, id, dp=1):
         # Ick!  This is necessary to deal with spaces. Waaa!
         object=self._getOb(id)
         object.manage_beforeDelete(object, self)
-        id=strip(id)
+        id=id.strip()
         self._objects=tuple(filter(lambda i,n=id:
-                                   strip(i['id']) != n,
+                                   i['id'].strip() != n,
                                    self._objects))
         self._delOb(id)
 
     def _getOb(self, id, default=_marker):
         if default is _marker:
-            r=self.getClassAttr(strip(id))
+            r=self.getClassAttr(id.strip())
         else:
-            try: r=self.getClassAttr(strip(id))
+            try: r=self.getClassAttr(id.strip())
             except: return default
 
         if hasattr(r, methodattr):
@@ -157,7 +156,7 @@ class ZClassMethodsSheet(
                 m._permissionMapper=wrapper
 
                 mw=MWp(m)
-                self.setClassAttr(strip(id), mw)
+                self.setClassAttr(id.strip(), mw)
 
             r=m
 
@@ -167,9 +166,9 @@ class ZClassMethodsSheet(
         if hasattr(self, 'aq_base'):
             b=self.aq_base
             if hasattr(b,name): return getattr(self, name)
-        
+
         try: return self[name]
-        except: return getattr(self, name) 
+        except: return getattr(self, name)
 
     def possible_permissions(self):
         return self.classDefinedAndInheritedPermissions()
@@ -214,7 +213,7 @@ methodattr='_ZClassMethodPermissionMapperMethod_'
 class MW(ExtensionClass.Base):
 
     def __init__(self, meth): self.__dict__[methodattr]=meth
-            
+
     def __of__(self, parent):
         m=getattr(self, methodattr)
         m=self.__dict__[methodattr]
@@ -231,7 +230,7 @@ class MWp(Globals.Persistent):
     def __getstate__(self):
         getattr(self, methodattr)
         return self.__dict__[methodattr]
-                
+
     def __of__(self, parent):
         m=getattr(self, methodattr)
         m=self.__dict__[methodattr]
@@ -240,13 +239,13 @@ class MWp(Globals.Persistent):
         if hasattr(m,'__of__'): return aqwrap(m, wrapper, parent)
         return m
 
-    
+
 
 # Backward compat. Waaaaa
 class W(Globals.Persistent, MW):
 
     _View_Permission='_View_Permission'
-        
+
     def __getattr__(self, name):
         # We want to make sure that any non-explicitly set methods are
         # private!
@@ -254,16 +253,16 @@ class W(Globals.Persistent, MW):
             # Oh this sucks
             return W.inheritedAttribute('__getattr__')(self, name)
         except: pass
-            
+
         if name[:1]=='_' and name[-11:]=="_Permission": return ''
         raise AttributeError, name
-            
+
     def __of__(self, parent):
         m=getattr(self, methodattr)
         m=self.__dict__[methodattr]
         if hasattr(m,'__of__'): return aqwrap(m, self, parent)
         return m
-    
+
 
 def findMethodIds(klass, methodTypes=(MWp, MW, W)):
     r=[]
