@@ -2,21 +2,22 @@
 #
 # Copyright (c) 2001, 2002 Zope Corporation and Contributors.
 # All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
-# 
+#
 ##############################################################################
 """Handy standard storage machinery
 """
 # Do this portably in the face of checking out with -kv
 import string
-__version__ = string.split('$Revision: 1.16.16.2 $')[-2:][0]
+__version__ = string.split('$Revision: 1.21.2.4 $')[-2:][0]
 
+import cPickle
 import ThreadLock, bpthread
 import time, UndoLogCompatible
 import POSException
@@ -30,7 +31,7 @@ class BaseStorage(UndoLogCompatible.UndoLogCompatible):
     _is_read_only = 0
 
     def __init__(self, name, base=None):
-        
+
         self.__name__=name
 
         # Allocate locks:
@@ -62,19 +63,29 @@ class BaseStorage(UndoLogCompatible.UndoLogCompatible):
     def close(self):
         pass
 
+    def sortKey(self):
+        """Return a string that can be used to sort storage instances.
+
+        The key must uniquely identify a storage and must be the same
+        across multiple instantiations of the same storage.
+        """
+        # name may not be sufficient, e.g. ZEO has a user-definable name.
+        return self.__name__
+
     def getName(self):
         return self.__name__
-    
+
     def getSize(self):
         return len(self)*300 # WAG!
-    
+
     def history(self, oid, version, length=1):
         pass
-                    
+
     def modifiedInVersion(self, oid):
         return ''
 
     def new_oid(self, last=None):
+        # 'last' is only for internal use, not part of the public API
         if self._is_read_only:
             raise POSException.ReadOnlyError()
         if last is None:
@@ -97,58 +108,64 @@ class BaseStorage(UndoLogCompatible.UndoLogCompatible):
 
     def isReadOnly(self):
         return self._is_read_only
-    
+
     def supportsUndo(self):
         return 0
-    
+
     def supportsVersions(self):
         return 0
-        
+
     def tpc_abort(self, transaction):
         self._lock_acquire()
         try:
-            if transaction is not self._transaction: return
+            if transaction is not self._transaction:
+                return
             self._abort()
             self._clear_temp()
-            self._transaction=None
+            self._transaction = None
             self._commit_lock_release()
-        finally: self._lock_release()
+        finally:
+            self._lock_release()
 
     def _abort(self):
         """Subclasses should redefine this to supply abort actions"""
         pass
 
     def tpc_begin(self, transaction, tid=None, status=' '):
+        if self._is_read_only:
+            raise POSException.ReadOnlyError()
         self._lock_acquire()
         try:
-            if self._transaction is transaction: return
+            if self._transaction is transaction:
+                return
             self._lock_release()
             self._commit_lock_acquire()
             self._lock_acquire()
-            self._transaction=transaction
+            self._transaction = transaction
             self._clear_temp()
 
-            user=transaction.user
-            desc=transaction.description
-            ext=transaction._extension
-            if ext: ext=dumps(ext,1)
-            else: ext=""
-            self._ude=user, desc, ext
+            user = transaction.user
+            desc = transaction.description
+            ext = transaction._extension
+            if ext:
+                ext = cPickle.dumps(ext, 1)
+            else:
+                ext = ""
+            self._ude = user, desc, ext
 
             if tid is None:
-                t=time.time()
-                t=apply(TimeStamp,(time.gmtime(t)[:5]+(t%60,)))
-                self._ts=t=t.laterThan(self._ts)
-                self._serial=`t`
+                now = time.time()
+                t = TimeStamp(*(time.gmtime(now)[:5] + (now % 60,)))
+                self._ts = t = t.laterThan(self._ts)
+                self._serial = `t`
             else:
-                self._ts=TimeStamp(tid)
-                self._serial=tid
-
-            self._tstatus=status
+                self._ts = TimeStamp(tid)
+                self._serial = tid
 
+            self._tstatus = status
             self._begin(self._serial, user, desc, ext)
-            
-        finally: self._lock_release()
+        finally:
+            self._lock_release()
 
     def _begin(self, tid, u, d, e):
         """Subclasses should redefine this to supply transaction start actions.
@@ -158,7 +175,8 @@ class BaseStorage(UndoLogCompatible.UndoLogCompatible):
     def tpc_vote(self, transaction):
         self._lock_acquire()
         try:
-            if transaction is not self._transaction: return
+            if transaction is not self._transaction:
+                return
             self._vote()
         finally:
             self._lock_release()
@@ -171,16 +189,17 @@ class BaseStorage(UndoLogCompatible.UndoLogCompatible):
     def tpc_finish(self, transaction, f=None):
         self._lock_acquire()
         try:
-            if transaction is not self._transaction: return
+            if transaction is not self._transaction:
+                return
             try:
-                if f is not None: f()
-
-                u,d,e=self._ude
+                if f is not None:
+                    f()
+                u, d, e = self._ude
                 self._finish(self._serial, u, d, e)
                 self._clear_temp()
             finally:
-                self._ude=None
-                self._transaction=None
+                self._ude = None
+                self._transaction = None
                 self._commit_lock_release()
         finally:
             self._lock_release()
@@ -246,8 +265,8 @@ class BaseStorage(UndoLogCompatible.UndoLogCompatible):
             restoring = 1
         else:
             restoring = 0
-        for transaction in other.iterator():
-            
+        fiter = other.iterator()
+        for transaction in fiter:
             tid=transaction.tid
             if _ts is None:
                 _ts=TimeStamp(tid)
@@ -265,21 +284,24 @@ class BaseStorage(UndoLogCompatible.UndoLogCompatible):
                         ok=1
 
             if verbose: print _ts
-            
+
             self.tpc_begin(transaction, tid, transaction.status)
             for r in transaction:
                 oid=r.oid
                 if verbose: print `oid`, r.version, len(r.data)
                 if restoring:
-                    self.restore(oid, r.serial, r.data, r.version, transaction)
+                    self.restore(oid, r.serial, r.data, r.version,
+                                 r.data_txn, transaction)
                 else:
                     pre=preget(oid, None)
                     s=self.store(oid, pre, r.data, r.version, transaction)
                     preindex[oid]=s
-                
+
             self.tpc_vote(transaction)
             self.tpc_finish(transaction)
 
+        fiter.close()
+
 class TransactionRecord:
     """Abstract base class for iterator protocol"""
 
