@@ -2,30 +2,36 @@
 #
 # Copyright (c) 2001, 2002 Zope Corporation and Contributors.
 # All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
-# 
+#
 ##############################################################################
 """Database objects
 
-$Id: DB.py,v 1.38.2.1 2002/02/13 16:37:04 gvanrossum Exp $"""
-__version__='$Revision: 1.38.2.1 $'[11:-2]
+$Id: DB.py,v 1.43.6.4 2003/06/24 20:15:53 jim Exp $"""
+__version__='$Revision: 1.43.6.4 $'[11:-2]
 
 import cPickle, cStringIO, sys, POSException, UndoLogCompatible
 from Connection import Connection
 from bpthread import allocate_lock
-from Transaction import Transaction
+from Transaction import Transaction, get_transaction
 from referencesf import referencesf
 from time import time, ctime
 from zLOG import LOG, ERROR
 
 from types import StringType
 
+def list2dict(L):
+    d = {}
+    for elt in L:
+        d[elt] = 1
+    return d
+
 class DB(UndoLogCompatible.UndoLogCompatible):
     """The Object Database
 
@@ -33,7 +39,8 @@ class DB(UndoLogCompatible.UndoLogCompatible):
     or more connections, which manage object spaces.  Most of the actual work
     of managing objects is done by the connections.
     """
-    klass = Connection
+    klass = Connection  # Class to use for connections
+    _activity_monitor = None
 
     def __init__(self, storage,
                  pool_size=7,
@@ -62,10 +69,10 @@ class DB(UndoLogCompatible.UndoLogCompatible):
         self._temps=[]
         self._pool_size=pool_size
         self._cache_size=cache_size
-        self._cache_deactivate_after=cache_deactivate_after
+        self._cache_deactivate_after = cache_deactivate_after
         self._version_pool_size=version_pool_size
         self._version_cache_size=version_cache_size
-        self._version_cache_deactivate_after=version_cache_deactivate_after
+        self._version_cache_deactivate_after = version_cache_deactivate_after
 
         self._miv_cache={}
 
@@ -100,7 +107,7 @@ class DB(UndoLogCompatible.UndoLogCompatible):
 
         if hasattr(storage, 'undoInfo'):
             self.undoInfo=storage.undoInfo
-            
+
 
     def _cacheMean(self, attr):
         # XXX this method doesn't work
@@ -119,20 +126,34 @@ class DB(UndoLogCompatible.UndoLogCompatible):
                       _silly=('__doc__',), _globals={}):
         return getattr(__import__(location, _globals, _globals, _silly),
                        name)
-            
+
     def _closeConnection(self, connection):
         """Return a connection to the pool"""
         self._a()
         try:
+            am = self._activity_monitor
+            if am is not None:
+                am.closedConnection(connection)
             version=connection._version
             pools,pooll=self._pools
-            pool, allocated, pool_lock = pools[version]
+            try:
+                pool, allocated, pool_lock = pools[version]
+            except KeyError:
+                # No such version. We must have deleted the pool.
+                # Just let the connection go.
+
+                # We need to break circular refs to make it really go:
+                connection.__dict__.clear()
+                
+                return
+                
             pool.append(connection)
             if len(pool)==1:
                 # Pool now usable again, unlock it.
                 pool_lock.release()
-        finally: self._r()
-        
+        finally:
+            self._r()
+
     def _connectionMap(self, f):
         self._a()
         try:
@@ -149,25 +170,29 @@ class DB(UndoLogCompatible.UndoLogCompatible):
                 self._temps=t
         finally: self._r()
 
-    def abortVersion(self, version):
-        AbortVersion(self, version)
+    def abortVersion(self, version, transaction=None):
+        if transaction is None:
+            transaction = get_transaction()
+        transaction.register(AbortVersion(self, version))
 
     def cacheDetail(self):
         """Return information on objects in the various caches
 
         Organized by class."""
 
-        detail={}
-        def f(con,detail=detail,have_detail=detail.has_key):
+        detail = {}
+        def f(con, detail=detail, have_detail=detail.has_key):
             for oid, ob in con._cache.items():
                 module = getattr(ob.__class__, '__module__', '')
                 module = module and '%s.' % module or ''
-                c="%s%s" % (module, ob.__class__.__name__)
-                if have_detail(c): detail[c]=detail[c]+1
-                else: detail[c]=1
-        
+                c = "%s%s" % (module, ob.__class__.__name__)
+                if have_detail(c):
+                    detail[c] = detail[c] + 1
+                else:
+                    detail[c] = 1
+
         self._connectionMap(f)
-        detail=detail.items()
+        detail = detail.items()
         detail.sort()
         return detail
 
@@ -177,7 +202,7 @@ class DB(UndoLogCompatible.UndoLogCompatible):
         def f(con, detail=detail, rc=sys.getrefcount, conn_no=conn_no):
             conn_no[0] = conn_no[0] + 1
             cn = conn_no[0]
-            for oid, ob in con._cache.items():
+            for oid, ob in con._cache_items():
                 id=''
                 if hasattr(ob,'__dict__'):
                     d=ob.__dict__
@@ -188,7 +213,7 @@ class DB(UndoLogCompatible.UndoLogCompatible):
 
                 module = getattr(ob.__class__, '__module__', '')
                 module = module and '%s.' % module or ''
-    
+
                 detail.append({
                     'conn_no': cn,
                     'oid': oid,
@@ -224,21 +249,40 @@ class DB(UndoLogCompatible.UndoLogCompatible):
     def cacheSize(self):
         m=[0]
         def f(con, m=m):
-            m[0]=m[0]+len(con._cache)
+            m[0] = m[0] + con._cache.cache_non_ghost_count
 
         self._connectionMap(f)
         return m[0]
 
-    def close(self): self._storage.close()
+    def cacheDetailSize(self):
+        m=[]
+        def f(con, m=m):
+            m.append({'connection':repr(con),
+                      'ngsize':con._cache.cache_non_ghost_count,
+                      'size':len(con._cache)})
+        self._connectionMap(f)
+        m.sort()
+        return m
 
-    def commitVersion(self, source, destination=''):
-        CommitVersion(self, source, destination)
+    def close(self):
+        self._storage.close()
+        for x, allocated in self._pools[1]:
+            for c in allocated:
+                c._breakcr()
+
+    def commitVersion(self, source, destination='', transaction=None):
+        if transaction is None:
+            transaction = get_transaction()
+        transaction.register(CommitVersion(self, source, destination))
 
     def exportFile(self, oid, file=None):
         raise 'Not yet implemented'
-                           
-    def getCacheDeactivateAfter(self): return self._cache_deactivate_after
-    def getCacheSize(self): return self._cache_size
+
+    def getCacheDeactivateAfter(self):
+        return self._cache_deactivate_after
+
+    def getCacheSize(self):
+        return self._cache_size
 
     def getName(self): return self._storage.getName()
 
@@ -248,24 +292,17 @@ class DB(UndoLogCompatible.UndoLogCompatible):
 
     def getVersionCacheDeactivateAfter(self):
         return self._version_cache_deactivate_after
-    def getVersionCacheSize(self): return self._version_cache_size
 
-    def getVersionPoolSize(self): return self._version_pool_size
+    def getVersionCacheSize(self):
+        return self._version_cache_size
+
+    def getVersionPoolSize(self):
+        return self._version_pool_size
 
     def importFile(self, file):
         raise 'Not yet implemented'
 
-    def begin_invalidation(self):
-        # Must be called before first call to invalidate and before
-        # the storage lock is held.
-        self._a()
-
-    def finish_invalidation(self):
-        # Must be called after begin_invalidation() and after final
-        # invalidate() call.
-        self._r()
-
-    def invalidate(self, oid, connection=None, version='',
+    def invalidate(self, oids, connection=None, version='',
                    rc=sys.getrefcount):
         """Invalidate references to a given oid.
 
@@ -277,9 +314,11 @@ class DB(UndoLogCompatible.UndoLogCompatible):
         if connection is not None:
             version=connection._version
         # Update modified in version cache
-        h=hash(oid)%131
-        o=self._miv_cache.get(h, None)
-        if o is not None and o[0]==oid: del self._miv_cache[h]
+        # XXX must make this work with list or dict to backport to 2.6
+        for oid in oids.keys():
+            h=hash(oid)%131
+            o=self._miv_cache.get(h, None)
+            if o is not None and o[0]==oid: del self._miv_cache[h]
 
         # Notify connections
         for pool, allocated in self._pools[1]:
@@ -288,7 +327,7 @@ class DB(UndoLogCompatible.UndoLogCompatible):
                     (not version or cc._version==version)):
                     if rc(cc) <= 3:
                         cc.close()
-                    cc.invalidate(oid)
+                    cc.invalidate(oids)
 
         temps=self._temps
         if temps:
@@ -297,7 +336,7 @@ class DB(UndoLogCompatible.UndoLogCompatible):
                 if rc(cc) > 3:
                     if (cc is not connection and
                         (not version or cc._version==version)):
-                        cc.invalidate(oid)
+                        cc.invalidate(oids)
                     t.append(cc)
                 else: cc.close()
             self._temps=t
@@ -312,8 +351,9 @@ class DB(UndoLogCompatible.UndoLogCompatible):
         cache[h]=oid, v
         return v
 
-    def objectCount(self): return len(self._storage)
-        
+    def objectCount(self):
+        return len(self._storage)
+
     def open(self, version='', transaction=None, temporary=0, force=None,
              waitflag=1):
         """Return a object space (AKA connection) to work in
@@ -329,9 +369,6 @@ class DB(UndoLogCompatible.UndoLogCompatible):
         Note that the connection pool is managed as a stack, to increate the
         likelihood that the connection's stack will include useful objects.
         """
-        if type(version) is not StringType:
-            raise POSException.Unimplemented, 'temporary versions'
-        
         self._a()
         try:
 
@@ -343,7 +380,7 @@ class DB(UndoLogCompatible.UndoLogCompatible):
                 else:
                     transaction._connections=connections={}
                 transaction=transaction._connections
-                    
+
 
             if temporary:
                 # This is a temporary connection.
@@ -351,16 +388,14 @@ class DB(UndoLogCompatible.UndoLogCompatible):
                 # a one-use connection.
                 c=self.klass(
                     version=version,
-                    cache_size=self._version_cache_size,
-                    cache_deactivate_after=
-                    self._version_cache_deactivate_after)
+                    cache_size=self._version_cache_size)
                 c._setDB(self)
                 self._temps.append(c)
                 if transaction is not None: transaction[id(c)]=c
                 return c
 
 
-            pools,pooll=self._pools
+            pools, pooll = self._pools
 
             # pools is a mapping object:
             #
@@ -386,7 +421,7 @@ class DB(UndoLogCompatible.UndoLogCompatible):
             # the last connection from the pool and just after adding
             # a connection to an empty pool.
 
-            
+
             if pools.has_key(version):
                 pool, allocated, pool_lock = pools[version]
             else:
@@ -402,20 +437,16 @@ class DB(UndoLogCompatible.UndoLogCompatible):
                     if self._version_pool_size > len(allocated) or force:
                         c=self.klass(
                             version=version,
-                            cache_size=self._version_cache_size,
-                            cache_deactivate_after=
-                            self._version_cache_deactivate_after)
+                            cache_size=self._version_cache_size)
                         allocated.append(c)
                         pool.append(c)
                 elif self._pool_size > len(allocated) or force:
                     c=self.klass(
                         version=version,
-                        cache_size=self._cache_size,
-                        cache_deactivate_after=
-                        self._cache_deactivate_after)
+                        cache_size=self._cache_size)
                     allocated.append(c)
                     pool.append(c)
-                    
+
                 if c is None:
                     if waitflag:
                         self._r()
@@ -453,6 +484,19 @@ class DB(UndoLogCompatible.UndoLogCompatible):
 
         finally: self._r()
 
+    def removeVersionPool(self, version):
+        pools, pooll = self._pools
+        info = pools.get(version)
+        if info:
+            del pools[version]
+            pool, allocated, pool_lock = info
+            pooll.remove((pool, allocated))
+            try: pool_lock.release()
+            except: pass
+            del pool[:]
+            del allocated[:]
+            6
+
     def connectionDebugInfo(self):
         r=[]
         pools,pooll=self._pools
@@ -465,14 +509,17 @@ class DB(UndoLogCompatible.UndoLogCompatible):
                     if len(d)==1: d=d[0]
                 else: d=''
                 d="%s (%s)" % (d, len(c._cache))
-                
+
                 r.append({
                     'opened': o and ("%s (%.2fs)" % (ctime(o), t-o)),
                     'info': d,
                     'version': version,
                     })
         return r
-        
+
+    def getActivityMonitor(self):
+        return self._activity_monitor
+
     def pack(self, t=None, days=0):
         if t is None: t=time()
         t=t-(days*86400)
@@ -480,22 +527,32 @@ class DB(UndoLogCompatible.UndoLogCompatible):
         except:
             LOG("ZODB", ERROR, "packing", error=sys.exc_info())
             raise
-                           
+
     def setCacheDeactivateAfter(self, v):
-        self._cache_deactivate_after=v
-        for c in self._pools[0][''][1]:
-            c._cache.cache_age=v
+        self._cache_deactivate_after = v
+        d = self._pools[0]
+        pool_info = d.get('')
+        if pool_info is not None:
+            for c in pool_info[1]:
+                c._cache.cache_age = v
 
     def setCacheSize(self, v):
-        self._cache_size=v
-        for c in self._pools[0][''][1]:
-            c._cache.cache_size=v
+        self._cache_size = v
+        d = self._pools[0]
+        pool_info = d.get('')
+        if pool_info is not None:
+            for c in pool_info[1]:
+                c._cache.cache_size = v
 
     def setClassFactory(self, factory):
-        self._classFactory=factory
+        self._classFactory = factory
+
+    def setPoolSize(self, v):
+        self._pool_size=v
+
+    def setActivityMonitor(self, am):
+        self._activity_monitor = am
 
-    def setPoolSize(self, v): self._pool_size=v
-    
     def setVersionCacheDeactivateAfter(self, v):
         self._version_cache_deactivate_after=v
         for ver in self._pools[0].keys():
@@ -509,12 +566,12 @@ class DB(UndoLogCompatible.UndoLogCompatible):
             if ver:
                 for c in self._pools[0][ver][1]:
                     c._cache.cache_size=v
-        
+
     def setVersionPoolSize(self, v): self._version_pool_size=v
 
     def cacheStatistics(self): return () # :(
 
-    def undo(self, id):
+    def undo(self, id, transaction=None):
         storage=self._storage
         try: supportsTransactionalUndo = storage.supportsTransactionalUndo
         except AttributeError:
@@ -524,11 +581,15 @@ class DB(UndoLogCompatible.UndoLogCompatible):
 
         if supportsTransactionalUndo:
             # new style undo
-            TransactionalUndo(self, id)
+            if transaction is None:
+                transaction = get_transaction()
+            transaction.register(TransactionalUndo(self, id))
         else:
             # fall back to old undo
+            d = {}
             for oid in storage.undo(id):
-                self.invalidate(oid)
+                d[oid] = 1
+            self.invalidate(d)
 
     def versionEmpty(self, version):
         return self._storage.versionEmpty(version)
@@ -547,20 +608,23 @@ class CommitVersion:
         self.tpc_begin=s.tpc_begin
         self.tpc_vote=s.tpc_vote
         self.tpc_finish=s.tpc_finish
-        get_transaction().register(self)
+        self._sortKey=s.sortKey
+
+    def sortKey(self):
+        return "%s:%s" % (self._sortKey(), id(self))
 
     def abort(self, reallyme, t): pass
 
     def commit(self, reallyme, t):
-        db=self._db
         dest=self._dest
-        oids=db._storage.commitVersion(self._version, dest, t)
-        for oid in oids: db.invalidate(oid, version=dest)
+        oids = self._db._storage.commitVersion(self._version, dest, t)
+        oids = list2dict(oids)
+        self._db.invalidate(oids, version=dest)
         if dest:
             # the code above just invalidated the dest version.
             # now we need to invalidate the source!
-            for oid in oids: db.invalidate(oid, version=self._version)
-    
+            self._db.invalidate(oids, version=self._version)
+
 class AbortVersion(CommitVersion):
     """An object that will see to version abortion
 
@@ -568,11 +632,9 @@ class AbortVersion(CommitVersion):
     """
 
     def commit(self, reallyme, t):
-        db=self._db
         version=self._version
-        oids = db._storage.abortVersion(version, t)
-        for oid in oids:
-            db.invalidate(oid, version=version)
+        oids = self._db._storage.abortVersion(version, t)
+        self._db.invalidate(list2dict(oids), version=version)
 
 
 class TransactionalUndo(CommitVersion):
@@ -580,13 +642,11 @@ class TransactionalUndo(CommitVersion):
 
     in cooperation with a transaction manager.
     """
-    
-    # I'm lazy. I'm reusing __init__ and abort and reusing the
-    # version attr for the transavtion id. There's such a strong
-    # similarity of rythm, that I think it's justified.
+
+    # I (Jim) am lazy.  I'm reusing __init__ and abort and reusing the
+    # version attr for the transaction id.  There's such a strong
+    # similarity of rhythm that I think it's justified.
 
     def commit(self, reallyme, t):
-        db=self._db
-        oids=db._storage.transactionalUndo(self._version, t)
-        for oid in oids:
-            db.invalidate(oid)
+        oids = self._db._storage.transactionalUndo(self._version, t)
+        self._db.invalidate(list2dict(oids))
