@@ -2,14 +2,14 @@
 #
 # Copyright (c) 2001, 2002 Zope Corporation and Contributors.
 # All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
-# 
+#
 ##############################################################################
 """Demo ZODB storage
 
@@ -45,7 +45,7 @@ There are three main data structures:
 
 A record is a tuple:
 
-  oid, serial, pre, vdata, p, 
+  oid, serial, pre, vdata, p,
 
 where:
 
@@ -79,7 +79,7 @@ method::
 and call it to monitor the storage.
 
 """
-__version__='$Revision: 1.8.16.2 $'[11:-2]
+__version__='$Revision: 1.14.2.1 $'[11:-2]
 
 import base64, time, string
 from ZODB import POSException, BaseStorage, utils
@@ -90,7 +90,6 @@ from BTrees import OOBTree
 class DemoStorage(BaseStorage.BaseStorage):
 
     def __init__(self, name='Demo Storage', base=None, quota=None):
-
         BaseStorage.BaseStorage.__init__(self, name, base)
 
         # We use a BTree because the items are sorted!
@@ -109,7 +108,7 @@ class DemoStorage(BaseStorage.BaseStorage):
     def __len__(self):
         base=self._base
         return (base and len(base) or 0) + len(self._index)
-        
+
     def getSize(self):
         s=100
         for tid, (p, u, d, e, t) in self._data.items():
@@ -131,32 +130,32 @@ class DemoStorage(BaseStorage.BaseStorage):
             raise POSException.StorageTransactionError(self, transaction)
         if not src:
             raise POSException.VersionCommitError("Invalid version")
-        
+
         self._lock_acquire()
         try:
-            v=self._vindex.get(src, None)
-            if not v: return
-            
-            tindex=self._tindex
-            oids=[]
+            v = self._vindex.get(src, None)
+            if not v:
+                return
+
+            oids = []
             for r in v.values():
                 oid, serial, pre, (version, nv), p = r
+                oids.append(oid)
                 if nv:
-                    oids.append(oid)
                     oid, serial, pre, vdata, p = nv
-                    tindex.append([oid, serial, r, None, p])
+                    self._tindex.append([oid, serial, r, None, p])
                 else:
                     # effectively, delete the thing
-                    tindex.append([oid, None, r, None, None]) 
+                    self._tindex.append([oid, None, r, None, None])
 
             return oids
 
         finally: self._lock_release()
-        
+
     def commitVersion(self, src, dest, transaction):
         if transaction is not self._transaction:
             raise POSException.StorageTransactionError(self, transaction)
-        
+
         if not src:
             raise POSException.VersionCommitError("Invalid source version")
         if src == dest:
@@ -167,7 +166,8 @@ class DemoStorage(BaseStorage.BaseStorage):
         try:
             v=self._vindex.get(src, None)
             if v is None: return
-            
+
+            newserial = self._serial
             tindex=self._tindex
             oids=[]
             for r in v.values():
@@ -178,8 +178,8 @@ class DemoStorage(BaseStorage.BaseStorage):
                     new_vdata = dest, vdata[1]
                 else:
                     new_vdata = None
-                tindex.append([oid, serial, r, new_vdata, p])
-                
+                tindex.append([oid, newserial, r, new_vdata, p])
+
 
             return oids
 
@@ -205,10 +205,10 @@ class DemoStorage(BaseStorage.BaseStorage):
 
             if p is None:
                 raise KeyError, oid
-            
+
             return p, serial
         finally: self._lock_release()
-                    
+
     def modifiedInVersion(self, oid):
         self._lock_acquire()
         try:
@@ -225,29 +225,32 @@ class DemoStorage(BaseStorage.BaseStorage):
 
         self._lock_acquire()
         try:
-            old=self._index.get(oid, None)
+            old = self._index.get(oid, None)
             if old is None:
                 # Hm, nothing here, check the base version:
-                try: p, oserial = self._base.load(oid, '')
-                except: pass
-                else:
-                    old= oid, oserial, None, None, p
-            
+                if self._base:
+                    try:
+                        p, oserial = self._base.load(oid, '')
+                    except KeyError:
+                        pass
+                    else:
+                        old = oid, oserial, None, None, p
+
             nv=None
             if old:
                 oid, oserial, pre, vdata, p = old
-                
+
                 if vdata:
                     if vdata[0] != version:
                         raise POSException.VersionLockError, oid
-                    
+
                     nv=vdata[1]
                 else:
                     nv=old
 
                 if serial != oserial:
                     raise POSException.ConflictError(serials=(oserial, serial))
-                
+
             serial=self._serial
             r=[oid, serial, old, version and (version, nv) or None, data]
             self._tindex.append(r)
@@ -269,38 +272,38 @@ class DemoStorage(BaseStorage.BaseStorage):
     def supportsVersions(self): return 1
 
     def _clear_temp(self):
-        self._tindex=[]
-        self._tsize=self._size+160
+        self._tindex = []
+        self._tsize = self._size + 160
 
     def _begin(self, tid, u, d, e):
-        self._tsize=self._size+120+len(u)+len(d)+len(e)
-    
+        self._tsize = self._size + 120 + len(u) + len(d) + len(e)
+
     def _finish(self, tid, user, desc, ext):
+        self._size = self._tsize
 
-        index=self._index
-        tindex=self._tindex
-        vindex=self._vindex
-
-        self._size=self._tsize
-
-        self._data[tid]=None, user, desc, ext, tuple(tindex)
-        for r in tindex:
+        self._data[tid] = None, user, desc, ext, tuple(self._tindex)
+        for r in self._tindex:
             oid, serial, pre, vdata, p = r
-            old=index.get(oid, None)
+            old = self._index.get(oid)
+            # If the object had version data, remove the version data.
             if old is not None:
-                oldvdata=old[3]
+                oldvdata = old[3]
                 if oldvdata:
-                    v=vindex[oldvdata[0]]
+                    v = self._vindex[oldvdata[0]]
                     del v[oid]
-                    if not v: del vindex[oldvdata[0]]
-                        
-            index[oid]=r
-            
+                    if not v:
+                        # If the version info is now empty, remove it.
+                        del self._vindex[oldvdata[0]]
+
+            self._index[oid] = r
+
+            # If there is version data, then udpate self._vindex, too.
             if vdata:
-                version=vdata[0]
-                v=vindex.get(version, None)
-                if v is None: v=vindex[version]={}
-                v[oid]=r
+                version = vdata[0]
+                v = self._vindex.get(version)
+                if v is None:
+                    v = self._vindex[version] = {}
+                v[oid] = r
 
     def undo(self, transaction_id):
         self._lock_acquire()
@@ -321,7 +324,7 @@ class DemoStorage(BaseStorage.BaseStorage):
             for r in t:
                 oid, serial, pre, vdata, p = r
                 if pre:
-                    
+
                     index[oid] = pre
                     oids.append(oid)
 
@@ -338,7 +341,7 @@ class DemoStorage(BaseStorage.BaseStorage):
                         v=vindex.get(version, None)
                         if v is None: v=vindex[version]={}
                         v[oid]=pre
-                        
+
                 else:
                     del index[oid]
                     if vdata:
@@ -354,33 +357,35 @@ class DemoStorage(BaseStorage.BaseStorage):
         finally: self._lock_release()
 
     def undoLog(self, first, last, filter=None):
+        if last < 0:
+            last = first - last + 1
         self._lock_acquire()
         try:
-            transactions=self._data.items()
-            pos=len(transactions)
-            encode=base64.encodestring
-            r=[]
-            append=r.append
-            i=0
+            # XXX Shouldn't this be sorted?
+            transactions = self._data.items()
+            pos = len(transactions)
+            r = []
+            i = 0
             while i < last and pos:
-                pos=pos-1
+                pos = pos - 1
                 if i < first:
-                    i = i+1
+                    i = i + 1
                     continue
                 tid, (p, u, d, e, t) = transactions[pos]
-                if p: continue
-                d={'id': encode(tid)[:-1],
-                   'time': TimeStamp(tid).timeTime(),
-                   'user_name': u, 'description': d}
+                if p:
+                    continue
+                d = {'id': base64.encodestring(tid)[:-1],
+                     'time': TimeStamp(tid).timeTime(),
+                     'user_name': u, 'description': d}
                 if e:
                     d.update(loads(e))
 
                 if filter is None or filter(d):
-                    append(d)
-                    i=i+1
-                
+                    r.append(d)
+                    i = i + 1
             return r
-        finally: self._lock_release()
+        finally:
+            self._lock_release()
 
     def versionEmpty(self, version):
         return not self._vindex.get(version, None)
@@ -412,7 +417,7 @@ class DemoStorage(BaseStorage.BaseStorage):
                         v=vindex[oldvdata[0]]
                         del v[oid]
                         if not v: del vindex[oldvdata[0]]
-                        
+
                 index[oid]=r
 
                 if vdata:
@@ -427,16 +432,16 @@ class DemoStorage(BaseStorage.BaseStorage):
         # Packing is hard, at least when undo is supported.
         # Even for a simple storage like this one, packing
         # is pretty complex.
-        
+
         self._lock_acquire()
         try:
 
             stop=`apply(TimeStamp, time.gmtime(t)[:5]+(t%60,))`
             _data=self._data
-    
+
             # Build indexes up to the pack time:
             index, vindex = self._build_indexes(stop)
-    
+
             # Now build an index of *only* those objects reachable
             # from the root.
             rootl=['\0\0\0\0\0\0\0\0']
@@ -446,13 +451,13 @@ class DemoStorage(BaseStorage.BaseStorage):
             while rootl:
                 oid=pop()
                 if referenced(oid): continue
-    
+
                 # Scan non-version pickle for references
                 r=index.get(oid, None)
                 if r is None:
-                    # Base storage
-                    p, s = self._base.load(oid, '')
-                    referencesf(p, rootl)
+                    if self._base:
+                        p, s = self._base.load(oid, '')
+                        referencesf(p, rootl)
                 else:
                     pindex[oid]=r
                     oid, serial, pre, vdata, p = r
@@ -462,7 +467,7 @@ class DemoStorage(BaseStorage.BaseStorage):
                         if nv:
                             oid, serial, pre, vdata, p = nv
                             referencesf(p, rootl)
-                
+
             # Now we're ready to do the actual packing.
             # We'll simply edit the transaction data in place.
             # We'll defer deleting transactions till the end
@@ -483,7 +488,7 @@ class DemoStorage(BaseStorage.BaseStorage):
                         if vdata:
                             # Version record are current *only* if they
                             # are indexed
-                            continue 
+                            continue
                         else:
                             # OK, this isn't a version record, so it may be the
                             # non-version record for the indexed record.
@@ -499,28 +504,30 @@ class DemoStorage(BaseStorage.BaseStorage):
                                 # record for it.
                                 continue
                     o.append(r)
-                    
+
                 if o:
                     if len(o) != len(t):
-                        _data[tid]=1, u, d, e, tuple(o) # Reset data
+                        _data[tid] = 1, u, d, e, tuple(o) # Reset data
                 else:
                     deleted.append(tid)
-    
+
             # Now delete empty transactions
-            for tid in deleted: del _data[tid]
-    
+            for tid in deleted:
+                del _data[tid]
+
             # Now reset previous pointers for "current" records:
             for r in pindex.values():
-                r[2]=None # Previous record
-                if r[3]: # vdata
-                    r[3][1][2]=None
+                r[2] = None # Previous record
+                if r[3] and r[3][1]: # vdata
+                    # If this record contains version data and
+                    # non-version data, then clear it out.
+                    r[3][1][2] = None
 
-            pindex=None
-    
             # Finally, rebuild indexes from transaction data:
             self._index, self._vindex = self._build_indexes()
 
-        finally: self._lock_release()
+        finally:
+            self._lock_release()
         self.getSize()
 
     def _splat(self):
@@ -558,6 +565,6 @@ class DemoStorage(BaseStorage.BaseStorage):
             for oid, r in vitems:
                 if r: r=id(r)
                 o.append('    %s: %s' % (utils.u64(oid), r))
-                
-        
+
+
         return string.join(o,'\n')
