@@ -2,43 +2,41 @@
 #
 # Copyright (c) 2001, 2002 Zope Corporation and Contributors.
 # All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
-# 
+#
 ##############################################################################
 """BoboPOS-defined exceptions
 
-$Id: POSException.py,v 1.8.16.2 2002/02/20 15:05:06 fdrake Exp $"""
-__version__='$Revision: 1.8.16.2 $'.split()[-2:][0]
+$Id: POSException.py,v 1.14 2002/09/05 10:19:40 htrd Exp $"""
+__version__ = '$Revision: 1.14 $'.split()[-2:][0]
 
 from string import join
 from types import StringType, DictType
 from ZODB import utils
 
 class POSError(Exception):
-    """Persistent object system error
-    """
+    """Persistent object system error."""
 
 class POSKeyError(KeyError, POSError):
-    """Key not found in database
-    """
+    """Key not found in database."""
 
     def __str__(self):
         return "%016x" % utils.U64(self.args[0])
 
 class TransactionError(POSError):
-    """An error occured due to normal transaction processing
-    """
+    """An error occured due to normal transaction processing."""
 
 class ConflictError(TransactionError):
-    """Two transactions tried to modify the same object at once.  This
-    transaction should be resubmitted.
-  
+    """Two transactions tried to modify the same object at once.
+
+    This transaction should be resubmitted.
+
     Instance attributes:
       oid : string
         the OID (8-byte packed string) of the object in conflict
@@ -63,7 +61,7 @@ class ConflictError(TransactionError):
         else:
             self.message = message
 
-        if object is None or not hasattr(object, '_p_oid'):
+        if object is None:
             self.oid = None
             self.class_name = None
         else:
@@ -106,10 +104,30 @@ class ConflictError(TransactionError):
     def get_serials(self):
         return self.serials
 
+class DanglingReferenceError(TransactionError):
+    """The transaction stored an object A containing a reference to another
+    object B, but B does not exist
+
+    Instance attributes:
+
+    Aoid: oid of the object being written
+
+    Boid: referenced oid that does not have a corresponding object
+    """
+
+    def __init__(self,Aoid,Boid):
+        self.Aoid = Aoid
+        self.Boid = Boid
+
+    def __str__(self):
+        return "from %r to %r" % (self.Aoid,self.Boid)
+
+
 class ReadConflictError(ConflictError):
-    """A conflict detected at read time -- attempt to read an object
-    that has changed in another transaction (eg. another thread
-    or process).
+    """A conflict was detected at read time.
+
+    An attempt was made to read an object that has changed in another
+    transaction (eg. another thread or process).
     """
     def __init__(self, message=None, object=None, serials=None):
         if message is None:
@@ -118,28 +136,29 @@ class ReadConflictError(ConflictError):
                                serials=serials)
 
 class BTreesConflictError(ConflictError):
-    """A special subclass for BTrees conflict errors, which return
-    an undocumented four-tuple."""
+    """A special subclass for BTrees conflict errors.
+
+    These return an undocumented four-tuple.
+    """
     def __init__(self, *btree_args):
         ConflictError.__init__(self, message="BTrees conflict error")
         self.btree = btree_args
 
-
 class VersionError(POSError):
-    """An error in handling versions occurred
-    """
+    """An error in handling versions occurred."""
 
 class VersionCommitError(VersionError):
-    """An invalid combination of versions was used in a version commit
-    """
+    """An invalid combination of versions was used in a version commit."""
 
 class VersionLockError(VersionError, TransactionError):
-    """An attempt was made to modify an object that has
-    been modified in an unsaved version"""
+    """Modification to an object modified in an unsaved version.
+
+    An attempt was made to modify an object that has been modified in an
+    unsaved version.
+    """
 
 class UndoError(POSError):
-    """An attempt was made to undo a non-undoable transaction.
-    """
+    """An attempt was made to undo a non-undoable transaction."""
     def __init__(self, *reason):
         if len(reason) == 1: reason=reason[0]
         self.__reason=reason
@@ -162,38 +181,32 @@ class UndoError(POSError):
     __str__=__repr__
 
 class StorageError(POSError):
-    pass
+    """Base class for storage based exceptions."""
 
 class StorageTransactionError(StorageError):
-    """An operation was invoked for an invalid transaction or state
-    """
+    """An operation was invoked for an invalid transaction or state."""
 
 class StorageSystemError(StorageError):
-    """Panic! Internal storage error!
-    """
+    """Panic! Internal storage error!"""
 
 class MountedStorageError(StorageError):
-    """Unable to access mounted storage.
-    """
+    """Unable to access mounted storage."""
 
 class ReadOnlyError(StorageError):
-    """Unable to modify objects in a read-only storage.
-    """
+    """Unable to modify objects in a read-only storage."""
+
+class TransactionTooLargeError(StorageTransactionError):
+    """The transaction exhausted some finite storage resource."""
 
 class ExportError(POSError):
-    """An export file doesn't have the right format.
-    """
-    pass
+    """An export file doesn't have the right format."""
 
 class Unimplemented(POSError):
-    """An unimplemented feature was used
-    """
-    pass
+    """An unimplemented feature was used."""
 
 class Unsupported(POSError):
-    """An feature that is unsupported bt the storage was used.
-    """
-    
+    """An feature that is unsupported bt the storage was used."""
+
 class InvalidObjectReference(POSError):
     """An object contains an invalid reference to another object.
 
