@@ -15,16 +15,14 @@
 static char TimeStamp_module_documentation[] = 
 "Defines 64-bit TimeStamp objects used as ZODB serial numbers.\n"
 "\n"
-"\n$Id: TimeStamp.c,v 1.10.16.3 2002/03/08 19:13:01 jeremy Exp $\n";
+"\n$Id: TimeStamp.c,v 1.15.60.5 2003/06/20 18:11:37 tim_one Exp $\n";
 
-#include <stdlib.h>
-#include <time.h>
 #ifdef USE_EXTENSION_CLASS
 #include "ExtensionClass.h"
 #else
 #include "Python.h"
 #endif
-
+#include <time.h>
 
 /* ----------------------------------------------------- */
 
@@ -224,8 +222,7 @@ static PyObject *
 TimeStamp_laterThan(TimeStamp *self, PyObject *args)
 {
   TimeStamp *o=NULL;
-  unsigned char *s;
-  PyObject *a;
+  unsigned char s[8];
   int i;
   
   UNLESS(PyArg_ParseTuple(args, "O!", self->ob_type, &o)) return NULL;
@@ -238,9 +235,7 @@ TimeStamp_laterThan(TimeStamp *self, PyObject *args)
 
   self=o;
 
-  UNLESS(a=PyString_FromStringAndSize((char *) self->data, 8)) return NULL;
-  s=(unsigned char *)PyString_AsString(a);
-  
+  memcpy(s, self->data, 8);
   for (i=7; i > 3; i--) 
     {
       if (s[i] == 255) 
@@ -248,7 +243,8 @@ TimeStamp_laterThan(TimeStamp *self, PyObject *args)
       else
 	{
 	  s[i]++;
-	  return PyObject_CallFunction(OBJECT(self->ob_type), "O", a);
+	  return PyObject_CallFunction(OBJECT(self->ob_type), "N", 
+	             PyString_FromStringAndSize(s, 8));
 	}
     }
 
@@ -344,7 +340,10 @@ TimeStamp_str(TimeStamp *self)
 static int
 TimeStamp_compare(TimeStamp *v, TimeStamp *w)
 {
-  return memcmp(v->data, w->data, 8);
+  int cmp = memcmp(v->data, w->data, 8);
+  if (cmp < 0) return -1;
+  if (cmp > 0) return 1;
+  return 0;
 }
 
 static long
