@@ -14,11 +14,14 @@
 static char cPersistence_doc_string[] = 
 "Defines Persistent mixin class for persistent objects.\n"
 "\n"
-"$Id: cPersistence.c,v 1.47.16.3 2002/03/08 19:13:02 jeremy Exp $\n";
+"$Id: cPersistence.c,v 1.62.10.3 2003/01/30 23:25:13 bwarsaw Exp $\n";
 
-#include <string.h>
 #include "cPersistence.h"
 
+struct ccobject_head_struct {
+    CACHE_HEAD
+};
+
 #define ASSIGN(V,E) {PyObject *__e; __e=(E); Py_XDECREF(V); (V)=__e;}
 #define UNLESS(E) if(!(E))
 #define UNLESS_ASSIGN(V,E) ASSIGN(V,E) UNLESS(V)
@@ -84,7 +87,7 @@ static PyObject *
 callmethod(PyObject *self, PyObject *name)
 {
   self=PyObject_GetAttr(self,name);
-  if(self)
+  if (self)
     ASSIGN(self,PyObject_CallObject(self,NULL));
   return self;
 }
@@ -107,58 +110,137 @@ callmethod1(PyObject *self, PyObject *name, PyObject *arg)
   return self;
 }
 
-#define UPDATE_STATE_IF_NECESSARY(self, ER)                      \
-if(self->state < 0 && self->jar)                                 \
-{								 \
-  PyObject *r;							 \
-      								 \
-  self->state=cPersistent_CHANGED_STATE; 	                 \
-  UNLESS(r=callmethod1(self->jar,py_setstate,(PyObject*)self))   \
-    {                                                            \
-      self->state=cPersistent_GHOST_STATE;                       \
-      return ER;                                                 \
-    }								 \
-  self->state=cPersistent_UPTODATE_STATE;			 \
-  Py_DECREF(r);							 \
-}
 
+static void ghostify(cPersistentObject*);
+
+/* Load the state of the object, unghostifying it.  Upon success, return 1.
+ * If an error occurred, re-ghostify the object and return 0.
+ */
+static int
+unghostify(cPersistentObject *self)
+{
+    if (self->state < 0 && self->jar) {
+        PyObject *r;
+
+        /* XXX Is it ever possibly to not have a cache? */
+        if (self->cache) {
+	    CPersistentRing *home = &self->cache->ring_home;
+            /* Create a node in the ring for this unghostified object. */
+            self->cache->non_ghost_count++;
+            self->ring.next = home;
+            self->ring.prev = home->prev;
+	    home->prev->next = &self->ring;
+	    home->prev = &self->ring;
+            Py_INCREF(self);
+        }
+        self->state = cPersistent_CHANGED_STATE;
+        /* Call the object's __setstate__() */
+        r = callmethod1(self->jar, py_setstate, (PyObject*)self);
+        if (r == NULL) {
+            ghostify(self);
+            return 0;
+        }
+        self->state = cPersistent_UPTODATE_STATE;
+        Py_DECREF(r);
+    }
+    return 1;
+}
 
 /****************************************************************************/
 
 staticforward PyExtensionClass Pertype;
 
+static void
+accessed(cPersistentObject *self)
+{
+    /* Do nothing unless the object is in a cache and not a ghost. */
+    if (self->cache && self->state >= 0) {
+	CPersistentRing *home = &self->cache->ring_home;
+	self->ring.prev->next = self->ring.next;
+	self->ring.next->prev = self->ring.prev;
+	self->ring.next = home;
+	self->ring.prev = home->prev;
+	home->prev->next = &self->ring;
+	home->prev = &self->ring; 
+    }
+}
+
+static void
+ghostify(cPersistentObject *self)
+{
+    /* are we already a ghost? */
+    if (self->state == cPersistent_GHOST_STATE)
+        return;
+    /* XXX is it ever possible to not have a cache? */
+    if (self->cache == NULL) {
+        self->state = cPersistent_GHOST_STATE;
+        return;
+    }
+    /* if we're ghostifying an object, we better have some non-ghosts */
+    assert(self->cache->non_ghost_count > 0);
+
+    self->cache->non_ghost_count--;
+    self->ring.next->prev = self->ring.prev;
+    self->ring.prev->next = self->ring.next;
+    self->ring.prev = NULL;
+    self->ring.next = NULL;
+    self->state = cPersistent_GHOST_STATE;
+
+    /* We remove the reference to the just ghosted object that the ring
+     * holds.  Note that the dictionary of oids->objects has an uncounted
+     * reference, so if the ring's reference was the only one, this frees
+     * the ghost object.  Note further that the object's dealloc knows to
+     * inform the dictionary that it is going away.
+     */
+    Py_DECREF(self);
+}
+
+static void
+deallocated(cPersistentObject *self)
+{
+    if (self->state >= 0) 
+	ghostify(self);
+    if (self->cache) {
+	/* XXX This function shouldn't be able to fail? If not, maybe
+	   it shouldn't set an exception either.
+	*/
+	if (cPersistenceCAPI->percachedel(self->cache, self->oid) < 0)
+	    PyErr_Clear(); /* I don't think this should ever happen */
+    }
+    Py_XDECREF(self->jar);
+    Py_XDECREF(self->oid);
+}
+
 static int
 changed(cPersistentObject *self)
 {
-  static PyObject *builtins=0, *get_transaction=0, *py_register=0;
-  PyObject *T;
-  
   if ((self->state == cPersistent_UPTODATE_STATE ||
        self->state == cPersistent_STICKY_STATE)
        && self->jar)
     {
-      UNLESS (get_transaction)
-	{
-	  UNLESS (py_register=PyString_FromString("register")) return -1;
-	  UNLESS (T=PyImport_ImportModule("__main__")) return -1;
-	  ASSIGN(T,PyObject_GetAttrString(T,"__builtins__"));
-	  UNLESS (T) return -1;
-	  builtins=T;
-	  UNLESS (get_transaction=PyObject_GetAttrString(builtins,
-							 "get_transaction"))
-	    PyErr_Clear();
-	}
-      if (get_transaction)
-	{    
-	  UNLESS (T=PyObject_CallObject(get_transaction,NULL)) return -1;
-	  ASSIGN(T,PyObject_GetAttr(T,py_register));
-	  UNLESS (T) return -1;
-	  ASSIGN(T, PyObject_CallFunction(T,"O",self));
-	  if (T) Py_DECREF(T);
-	  else return -1;
-	}
+	PyObject *meth, *arg, *result;
+	static PyObject *s_register;
 
-      self->state=cPersistent_CHANGED_STATE;
+	if (s_register == NULL) 
+	    s_register = PyString_InternFromString("register");
+	meth = PyObject_GetAttr((PyObject *)self->jar, s_register);
+	if (meth == NULL)
+	    return -1;
+	arg = PyTuple_New(1);
+	if (arg == NULL) {
+	    Py_DECREF(meth);
+	    return -1;
+	}
+	PyTuple_SET_ITEM(arg, 0, (PyObject *)self);
+	result = PyEval_CallObject(meth, arg);
+	PyTuple_SET_ITEM(arg, 0, NULL);
+	Py_DECREF(arg);
+	Py_DECREF(meth);
+	if (result == NULL)
+	    return -1;
+	Py_DECREF(result);
+
+	self->state = cPersistent_CHANGED_STATE;
     }
 
   return 0;
@@ -167,25 +249,28 @@ changed(cPersistentObject *self)
 static PyObject *
 Per___changed__(cPersistentObject *self, PyObject *args)
 {
-  PyObject *v=0;
+    PyObject *v = NULL;
 
-  if (args && ! PyArg_ParseTuple(args, "|O",&v)) return NULL;
-  if (! v) return PyObject_GetAttr(OBJECT(self), py__p_changed);
+    if (args && !PyArg_ParseTuple(args, "|O:__changed__", &v)) 
+	return NULL;
+    if (!v) 
+	return PyObject_GetAttr(OBJECT(self), py__p_changed);
 
-  if (PyObject_IsTrue(v)) 
-    {
-      if (changed(self) < 0) return NULL;
+    if (PyObject_IsTrue(v)) {
+	if (changed(self) < 0) 
+	    return NULL;
     }
-  else if (self->state >= 0) self->state=cPersistent_UPTODATE_STATE;
+    else if (self->state >= 0) 
+	self->state = cPersistent_UPTODATE_STATE;
 
-  Py_INCREF(Py_None);
-  return Py_None;
+    Py_INCREF(Py_None);
+    return Py_None;
 }
 
 static PyObject *
 Per__p_deactivate(cPersistentObject *self, PyObject *args)
 {
-  PyObject *dict;
+  PyObject *dict,*dict2=NULL;
 
 #ifdef DEBUG_LOG
   if (idebug_log < 0) call_debug("reinit",self);
@@ -197,13 +282,22 @@ Per__p_deactivate(cPersistentObject *self, PyObject *args)
   if (self->state==cPersistent_UPTODATE_STATE && self->jar &&
       HasInstDict(self) && (dict=INSTANCE_DICT(self)))
     {
+      dict2 = PyDict_Copy(dict);
       PyDict_Clear(dict);
       /* Note that we need to set to ghost state unless we are 
 	 called directly. Methods that override this need to
          do the same! */
-      self->state=cPersistent_GHOST_STATE;
+      ghostify(self);
     }
 
+  /* need to delay releasing the last reference on instance attributes
+     until after we have finished accounting for losing our state */
+  if (dict2)
+  {
+      PyDict_Clear(dict2);
+      Py_DECREF(dict2);
+  }
+
   Py_INCREF(Py_None);
   return Py_None;
 }
@@ -212,53 +306,59 @@ Per__p_deactivate(cPersistentObject *self, PyObject *args)
 static int
 Per_setstate(cPersistentObject *self)
 {
-  UPDATE_STATE_IF_NECESSARY(self, -1);
-  self->state=cPersistent_STICKY_STATE;
-  return 0;
+    if (!unghostify(self))
+        return -1;
+    self->state = cPersistent_STICKY_STATE;
+    return 0;
 }
 
 static PyObject *
 Per__getstate__(cPersistentObject *self, PyObject *args)
 {
-  PyObject *__dict__, *d=0;
+    PyObject *__dict__, *d=0;
 
-  if (!checknoargs(args)) return NULL;
+    if (!checknoargs(args))
+        return NULL;
 
 #ifdef DEBUG_LOG
-  if(idebug_log < 0) call_debug("get",self);
+    if (idebug_log < 0)
+        call_debug("get",self);
 #endif
 
-  UPDATE_STATE_IF_NECESSARY(self, NULL);
+    if (!unghostify(self))
+        return NULL;
 
-  if(HasInstDict(self) && (__dict__=INSTANCE_DICT(self)))
-    {
-      PyObject *k, *v;
-      int pos;
-      char *ck;
+    if (HasInstDict(self) && (__dict__=INSTANCE_DICT(self))) {
+        PyObject *k, *v;
+        int pos;
+        char *ck;
 	  
-      for(pos=0; PyDict_Next(__dict__, &pos, &k, &v); )
-	{
-	  if(PyString_Check(k) && (ck=PyString_AS_STRING(k)) &&
-	     (*ck=='_' && ck[1]=='v' && ck[2]=='_'))
+        for(pos=0; PyDict_Next(__dict__, &pos, &k, &v); ) {
+            if (PyString_Check(k) && (ck=PyString_AS_STRING(k)) &&
+               (*ck=='_' && ck[1]=='v' && ck[2]=='_'))
 	    {
-	      UNLESS(d=PyDict_New()) goto err;
-	      for(pos=0; PyDict_Next(__dict__, &pos, &k, &v); )
-		UNLESS(PyString_Check(k) && (ck=PyString_AS_STRING(k)) &&
-		       (*ck=='_' && ck[1]=='v' && ck[2]=='_'))
-		  if(PyDict_SetItem(d,k,v) < 0) goto err;
-	      return d;
+                if ((d=PyDict_New()) == NULL)
+                    goto err;
+                
+                for (pos=0; PyDict_Next(__dict__, &pos, &k, &v); )
+                    UNLESS(PyString_Check(k) && (ck=PyString_AS_STRING(k)) &&
+                           (*ck=='_' && ck[1]=='v' && ck[2]=='_'))
+                    {
+                        if (PyDict_SetItem(d,k,v) < 0)
+                            goto err;
+                    }
+                return d;
 	    }
 	}
     }
-  else
-    __dict__=Py_None;
+    else
+        __dict__ = Py_None;
 
-  Py_INCREF(__dict__);
-  return __dict__;
-
-err:
-  Py_XDECREF(d);
-  return NULL;
+    Py_INCREF(__dict__);
+    return __dict__;
+  err:
+    Py_XDECREF(d);
+    return NULL;
 }  
 
 static PyObject *
@@ -267,21 +367,21 @@ Per__setstate__(cPersistentObject *self, PyObject *args)
   PyObject *__dict__, *v, *keys=0, *key=0, *e=0;
   int l, i;
 
-  if(HasInstDict(self))
+  if (HasInstDict(self))
     {
 
        UNLESS(PyArg_ParseTuple(args, "O", &v)) return NULL;
 #ifdef DEBUG_LOG
-       if(idebug_log < 0) call_debug("set",self);
+       if (idebug_log < 0) call_debug("set",self);
 #endif
-       if(v!=Py_None)
+       if (v!=Py_None)
 	 {
 	   __dict__=INSTANCE_DICT(self);
 	   
-	   if(PyDict_Check(v))
+	   if (PyDict_Check(v))
 	     {
 	       for(i=0; PyDict_Next(v, &i, &key, &e);)
-		 if(PyDict_SetItem(__dict__, key, e) < 0)
+		 if (PyDict_SetItem(__dict__, key, e) < 0)
 		   return NULL;
 	     }
 	   else
@@ -331,10 +431,10 @@ static void
 Per_dealloc(cPersistentObject *self)
 {
 #ifdef DEBUG_LOG
-  if(idebug_log < 0) call_debug("del",self);
+  if (idebug_log < 0) call_debug("del",self);
 #endif
-  Py_XDECREF(self->jar);
-  Py_XDECREF(self->oid);
+  deallocated(self);
+  Py_XDECREF(self->cache);
   Py_DECREF(self->ob_type);
   PyObject_DEL(self);
 }
@@ -353,21 +453,21 @@ Per_getattr(cPersistentObject *self, PyObject *oname, char *name,
 {
   char *n=name;
 
-  if(n && *n++=='_')
-    if(*n++=='p' && *n++=='_')
+  if (n && *n++=='_')
+    if (*n++=='p' && *n++=='_')
       {
 	switch(*n++)
 	  {
 	  case 'o':
-	    if(*n++=='i' && *n++=='d' && ! *n) return orNothing(self->oid);
+	    if (*n++=='i' && *n++=='d' && ! *n) return orNothing(self->oid);
 	    break;
 	  case 'j':
-	    if(*n++=='a' && *n++=='r' && ! *n) return orNothing(self->jar);
+	    if (*n++=='a' && *n++=='r' && ! *n) return orNothing(self->jar);
 	    break;
 	  case 'c':
-	    if(strcmp(n,"hanged")==0)
+	    if (strcmp(n,"hanged")==0)
 	      {
-		if(self->state < 0)
+		if (self->state < 0)
 		  {
 		    Py_INCREF(Py_None);
 		    return Py_None;
@@ -377,17 +477,18 @@ Per_getattr(cPersistentObject *self, PyObject *oname, char *name,
 	      }
 	    break;
 	  case 's':
-	    if(strcmp(n,"erial")==0)
+	    if (strcmp(n,"erial")==0)
 	      return PyString_FromStringAndSize(self->serial, 8);
-	    if(strcmp(n,"elf")==0) 
+	    if (strcmp(n,"elf")==0) 
 	      return orNothing(OBJECT(self));
 	    break;
 	  case 'm':
-	    if(strcmp(n,"time")==0)
+	    if (strcmp(n,"time")==0)
 	      {
-		UPDATE_STATE_IF_NECESSARY(self, NULL);
+                  if (!unghostify(self))
+                      return NULL;
 
-		self->atime=((long)(time(NULL)/3))%65536;
+		accessed(self);
 
 		if (self->serial[7]=='\0' && self->serial[6]=='\0' &&
 		    self->serial[5]=='\0' && self->serial[4]=='\0' &&
@@ -413,13 +514,14 @@ Per_getattr(cPersistentObject *self, PyObject *oname, char *name,
 
 	return getattrf((PyObject *)self, oname);
       }
-  if(! (name && *name++=='_' && *name++=='_' &&
+  if (! (name && *name++=='_' && *name++=='_' &&
 	(strcmp(name,"dict__")==0 || strcmp(name,"class__")==0
 	 || strcmp(name, "of__")==0)))
     {
-      UPDATE_STATE_IF_NECESSARY(self, NULL);
+        if (!unghostify(self))
+            return NULL;
 
-      self->atime=((long)(time(NULL)/3))%65536;
+      accessed(self);
     }
 
   return getattrf((PyObject *)self, oname);
@@ -457,59 +559,110 @@ static int
 _setattro(cPersistentObject *self, PyObject *oname, PyObject *v,
 	     int (*setattrf)(PyObject *, PyObject*, PyObject*))
 {
-  char *name="";
+  char *name = "";
 
-  UNLESS(oname) return -1;
-  if(PyString_Check(oname)) UNLESS(name=PyString_AS_STRING(oname)) return -1;
+  if (oname == NULL)
+      return -1;
+  if (!PyString_Check(oname)) 
+      return -1;
+  name = PyString_AS_STRING(oname);
+  if (name == NULL)
+      return -1;
 	
-  if(*name=='_' && name[1]=='p' && name[2]=='_')
-    {
-      if(name[3]=='o' && name[4]=='i' && name[5]=='d' && ! name[6])
-	{
+  if (*name == '_' && name[1] == 'p' && name[2] == '_') {
+      if (strcmp(name + 3, "oid") == 0) {
+	  if (self->cache) {
+	      int result;
+	      if (v == NULL) {
+		  PyErr_SetString(PyExc_ValueError,
+				  "can not delete oid of cached object");
+		  return -1;
+	      }
+	      if (PyObject_Cmp(self->oid, v, &result) < 0)
+		  return -1;
+	      if (result) {
+		  PyErr_SetString(PyExc_ValueError,
+				  "can not change oid of cached object");
+		  return -1;
+	      }
+	  }
 	  Py_XINCREF(v);
 	  ASSIGN(self->oid, v);
 	  return 0;
-	}
-      if(name[3]=='j' && name[4]=='a' && name[5]=='r' && ! name[6])
-	{
+      }
+      else if (strcmp(name + 3, "jar") == 0) {
+	  if (self->cache && self->jar) {
+	      int result;
+	      if (v == NULL) {
+		  PyErr_SetString(PyExc_ValueError,
+				  "can not delete jar of cached object");
+		  return -1;
+	      }
+	      if (PyObject_Cmp(self->jar, v, &result) < 0)
+		  return -1;
+	      if (result) {
+		  PyErr_SetString(PyExc_ValueError,
+				  "can not change jar of cached object");
+		  return -1;
+	      }
+	  }
 	  Py_XINCREF(v);
 	  ASSIGN(self->jar, v);
 	  return 0;
-	}
-      if(name[3]=='s' && strcmp(name+4,"erial")==0)
-	{
-	  if (v)
-	    {
-	      if (PyString_Check(v) && PyString_Size(v)==8)
-		memcpy(self->serial, PyString_AS_STRING(v), 8);
-	      else
-		{
+      }
+      else if (strcmp(name + 3, "serial") == 0) {
+	  if (v) {
+	      if (PyString_Check(v) && PyString_GET_SIZE(v) == 8)
+		  memcpy(self->serial, PyString_AS_STRING(v), 8);
+	      else {
 		  PyErr_SetString(PyExc_ValueError,
 				  "_p_serial must be an 8-character string");
 		  return -1;
-		}
-	    }
-	  else
-	    memset(self->serial, 0, 8);
+	      }
+	  } else 
+	      memset(self->serial, 0, 8);
 	  return 0;
-	}
-      if(name[3]=='c' && strcmp(name+4,"hanged")==0) 
-	{
-	  if (! v)
+      }
+      else if (strcmp(name+3, "changed") == 0) {
+	  int deactivate = 0;
+	  if (!v)
 	    {
 	      /* delatter is used to invalidate the object
 	         *even* if it has changed.
 	       */
 	      if (self->state != cPersistent_GHOST_STATE)
 		self->state = cPersistent_UPTODATE_STATE;
-	      v=Py_None;
+	      deactivate = 1;
 	    }
-	  if (v==Py_None)
+	  else if (v == Py_None)
+	      deactivate = 1;
+	  if (deactivate)
 	    {
-	      v=PyObject_GetAttr(OBJECT(self), py__p_deactivate);
-	      if (v) { ASSIGN(v, PyObject_CallObject(v, NULL)); }
-	      if (v) { Py_DECREF(v); }
-	      self->state=cPersistent_GHOST_STATE;
+	      PyObject *res;
+	      PyObject *meth = PyObject_GetAttr(OBJECT(self), 
+						py__p_deactivate);
+	      if (meth == NULL)
+		  return -1;
+	      res = PyObject_CallObject(meth, NULL);
+	      if (res) {
+		  Py_DECREF(res);
+	      } 
+	      else {
+		  /* an error occured in _p_deactivate().  
+
+		  It's not clear what we should do here.  The code is
+		  obviously ignoring the exception, but it shouldn't
+		  return 0 for a getattr and set an exception.  The
+		  simplest change is to clear the exception, but that
+		  simply masks the error. 
+
+		  XXX We'll print an error to stderr just like
+		  exceptions in __del__().  It would probably be
+		  better to log it but that would be painful from C.
+		  */
+		  PyErr_WriteUnraisable(meth);
+	      }
+	      Py_DECREF(meth);
 	      return 0;
 	    }
 	  if (PyObject_IsTrue(v)) return changed(self);
@@ -519,16 +672,16 @@ _setattro(cPersistentObject *self, PyObject *oname, PyObject *v,
     }
   else
     {
-      UPDATE_STATE_IF_NECESSARY(self, -1);
+        if (!unghostify(self))
+            return -1;
       
-      /* Record access times */
-      self->atime=((long)(time(NULL)/3))%65536;
+      accessed(self);
 
-      if((! (*name=='_' && name[1]=='v' && name[2]=='_'))
+      if ((! (*name=='_' && name[1]=='v' && name[2]=='_'))
 	 && (self->state != cPersistent_CHANGED_STATE && self->jar)
 	 && setattrf
 	 )
-	if(changed(self) < 0) return -1;
+	if (changed(self) < 0) return -1;
     }
 
   if (setattrf)
@@ -544,10 +697,9 @@ Per_setattro(cPersistentObject *self, PyObject *oname, PyObject *v)
   PyObject *m;
 
   if (v && (((PyExtensionClass*)self->ob_type)->class_flags 
-	    & EXTENSIONCLASS_USERSETATTR_FLAG)
-      )
+	    & EXTENSIONCLASS_USERSETATTR_FLAG))
     {
-      r=_setattro(self,oname, v, NULL);
+      r = _setattro(self, oname, v, NULL);
       if (r < 1) return r;
 
       m=PyObject_GetAttr(OBJECT(self), py___setattr__);
@@ -651,7 +803,7 @@ set_debug_log(PyObject *ignored, PyObject *args)
 {
   Py_INCREF(args);
   ASSIGN(debug_log, args);
-  if(debug_log) idebug_log=-1;
+  if (debug_log) idebug_log=-1;
   else idebug_log=0;
   Py_INCREF(Py_None);
   return Py_None;
@@ -680,9 +832,15 @@ truecPersistenceCAPI = {
   (getattrofunc)Per_getattro,	/*tp_getattr with object key*/
   (setattrofunc)Per_setattro,	/*tp_setattr with object key*/
   changed,
+  accessed,
+  ghostify,
+  deallocated,
   (intfunctionwithpythonarg)Per_setstate,
   (pergetattr)Per_getattr,
   (persetattr)_setattro,
+  NULL /* The percachedel slot is initialized in cPickleCache.c when
+          the module is loaded.  It uses a function in a different
+          shared library. */
 };
 
 void
@@ -690,7 +848,7 @@ initcPersistence(void)
 {
   PyObject *m, *d, *s;
 
-  s = PyString_FromString("TimeStamp");
+  s = PyString_FromString("ZODB.TimeStamp");
   if (s == NULL)
       return;
   m = PyImport_Import(s);
@@ -698,11 +856,13 @@ initcPersistence(void)
       Py_DECREF(s);
       return;
   }
-  TimeStamp = PyObject_GetAttr(m, s);
+  TimeStamp = PyObject_GetAttrString(m, "TimeStamp");
+  assert(TimeStamp);
   Py_DECREF(m);
   Py_DECREF(s);
 
-  if (init_strings() < 0) return;
+  if (init_strings() < 0) 
+      return;
 
   m = Py_InitModule4("cPersistence", cP_methods, cPersistence_doc_string,
 		     (PyObject*)NULL, PYTHON_API_VERSION);
@@ -713,12 +873,8 @@ initcPersistence(void)
   PyExtensionClass_Export(d, "Persistent",  Pertype);
   PyExtensionClass_Export(d, "Overridable", Overridable);
 
-  cPersistenceCAPI=&truecPersistenceCAPI;
-  s = PyCObject_FromVoidPtr(cPersistenceCAPI,NULL);
+  cPersistenceCAPI = &truecPersistenceCAPI;
+  s = PyCObject_FromVoidPtr(cPersistenceCAPI, NULL);
   PyDict_SetItemString(d, "CAPI", s);
   Py_XDECREF(s);
-
-  /* Check for errors */
-  if (PyErr_Occurred())
-    Py_FatalError("can't initialize module cPersistence");
 }
