@@ -1,14 +1,14 @@
 ##############################################################################
 #
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
-# 
+#
 ##############################################################################
 """XML-RPC support module
 
@@ -19,18 +19,21 @@ See http://www.xmlrpc.com/ and http://linux.userland.com/ for more
 information about XML-RPC and Zope.
 """
 
-import sys
+import re
+import sys, types
 from HTTPResponse import HTTPResponse
 import xmlrpclib
 
+from zExceptions import Unauthorized
+
 def parse_input(data):
     """Parse input data and return a method path and argument tuple
 
     The data is a string.
     """
-    # 
+    #
     # For example, with the input:
-    #     
+    #
     #   <?xml version="1.0"?>
     #   <methodCall>
     #      <methodName>examples.getStateName</methodName>
@@ -40,9 +43,9 @@ def parse_input(data):
     #            </param>
     #         </params>
     #      </methodCall>
-    # 
+    #
     # the function should return:
-    # 
+    #
     #     ('examples.getStateName', (41,))
     params, method = xmlrpclib.loads(data)
     # Translate '.' to '/' in meth to represent object traversal.
@@ -53,7 +56,7 @@ def parse_input(data):
 #
 # def response(anHTTPResponse):
 #     """Return a valid ZPublisher response object
-# 
+#
 #     Use data already gathered by the existing response.
 #     The new response will replace the existing response.
 #     """
@@ -62,8 +65,6 @@ def parse_input(data):
 #     r=Response()
 #     r.__dict__.update(anHTTPResponse.__dict__)
 #     return r
-    
-    
 
 ########################################################################
 # Possible implementation helpers:
@@ -81,9 +82,11 @@ class Response:
     It's probably possible to improve the 'exception' method quite a bit.
     The current implementation, however, should suffice for now.
     """
+    
+    _error_format = 'text/plain' # No html in error values
 
     # Because we can't predict what kind of thing we're customizing,
-    # we have to use delegation, rather than inheritence to do the
+    # we have to use delegation, rather than inheritance to do the
     # customization.
 
     def __init__(self, real): self.__dict__['_real']=real
@@ -91,17 +94,28 @@ class Response:
     def __getattr__(self, name): return getattr(self._real, name)
     def __setattr__(self, name, v): return setattr(self._real, name, v)
     def __delattr__(self, name): return delattr(self._real, name)
-    
+
     def setBody(self, body, title='', is_error=0, bogus_str_search=None):
         if isinstance(body, xmlrpclib.Fault):
             # Convert Fault object to XML-RPC response.
             body=xmlrpclib.dumps(body, methodresponse=1)
         else:
+            if type(body) == types.InstanceType:
+                # Avoid disclosing private members. Private members are
+                # by convention named with a leading underscore char.
+                orig = body.__dict__
+                dict = {}
+                for key in orig.keys():
+                    if key[:1] != '_':
+                        dict[key] = orig[key]
+                body = dict
+
             # Marshall our body as an XML-RPC response. Strings will be sent
             # strings, integers as integers, etc. We do *not* convert
             # everything to a string first.
-            if body is None:
-                body=xmlrpclib.False # Argh, XML-RPC doesn't handle null
+            # Previously this had special handling if the response
+            # was a Python None. This is now patched in xmlrpclib to
+            # allow Nones nested inside data structures too.
             try:
                 body = xmlrpclib.dumps((body,), methodresponse=1)
             except:
@@ -119,18 +133,37 @@ class Response:
         if type(info) is type(()) and len(info)==3: t,v,tb = info
         else: t,v,tb = sys.exc_info()
 
-        # Create an appropriate Fault object. Unfortunately, we throw away
-        # most of the debugging information. More useful error reporting is
-        # left as an exercise for the reader.
+        # Don't mask 404 respnses, as some XML-RPC libraries rely on the HTTP
+        # mechanisms for detecting when authentication is required. Fixes Zope
+        # Collector issue 525.
+        if t == 'Unauthorized' or (
+            isinstance(t, types.ClassType) and issubclass(t, Unauthorized)
+            ):
+
+            return self._real.exception(fatal=fatal, info=info)
+
+        # Create an appropriate Fault object. Containing error information
         Fault=xmlrpclib.Fault
         f=None
         try:
+            # Strip HTML tags from the error value
+            vstr = str(v)
+            remove = [r"<[^<>]*>", r"&[A-Za-z]+;"]
+            for pat in remove:
+                vstr = re.sub(pat, " ", vstr)
+            from Globals import DevelopmentMode
+            if DevelopmentMode:
+                from traceback import format_exception
+                value = '\n' + ''.join(format_exception(t, vstr, tb))
+            else:
+                value = '%s - %s' % (t, vstr)
+                
             if isinstance(v, Fault):
                 f=v
             elif isinstance(v, Exception):
-                f=Fault(-1, "Unexpected Zope exception: " + str(v))
+                f=Fault(-1, 'Unexpected Zope exception: %s' % value)
             else:
-                f=Fault(-2, "Unexpected Zope error value: " + str(v))
+                f=Fault(-2, 'Unexpected Zope error value: %s' % value)
         except:
             f=Fault(-3, "Unknown Zope fault type")
 
