@@ -1,19 +1,19 @@
 ##############################################################################
 #
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
-# 
+#
 ##############################################################################
 
 """Commonly used functions for WebDAV support modules."""
 
-__version__='$Revision: 1.15 $'[11:-2]
+__version__='$Revision: 1.17 $'[11:-2]
 
 import  time, urllib, re
 from App.Common import iso8601_date, rfc850_date, rfc1123_date
@@ -59,7 +59,9 @@ def generateLockToken():
     return '%s-%s-00105A989226:%.03f' %  \
          (_randGen.random(),_randGen.random(),time.time())
 
-    
+def isDavCollection(object):
+    """Return true if object is a DAV collection."""
+    return getattr(object, '__dav_collection__', 0)
 
 def tokenFinder(token):
     # takes a string like '<opaquelocktoken:afsdfadfadf> and returns the token
