@@ -3,21 +3,23 @@
 ##############################################################################
 #
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
-# 
+#
 ##############################################################################
 
 """ Request log profiler script """
 
-__version__='$Revision: 1.14.8.1 $'[11:-2]
+__version__='$Revision: 1.17 $'[11:-2]
 
 import string, sys, time, getopt, tempfile, math, cPickle
+try: import gzip
+except: pass
 
 class ProfileException(Exception): pass
 
@@ -34,7 +36,7 @@ class Request:
         self.t_end = None
         self.elapsed = "I"
         self.active = 0
-        
+
     def put(self, code, t, desc):
         if code not in ('A', 'B', 'I', 'E'):
             raise "unknown request code %s" % code
@@ -50,7 +52,7 @@ class Request:
         elif code == 'E':
             self.t_end = t
             self.elapsed = int(self.t_end - self.start)
-            
+
     def isfinished(self):
         return not self.elapsed == "I"
 
@@ -67,13 +69,13 @@ class Request:
             return time.strftime('%H:%M:%S', t)
         else:
             return "NA"
-        
+
     def win(self):
         if self.t_recdinput is not None and self.start is not None:
             return self.t_recdinput - self.start
         else:
             return "NA"
-        
+
     def wout(self):
         if self.t_recdoutput is not None and self.t_recdinput is not None:
             return self.t_recdoutput - self.t_recdinput
@@ -128,7 +130,7 @@ class Request:
             self.total(), self.endstage(), self.prettyosize(),
             self.prettyhttpcode(), self.active, self.url
             )
-        return self.fmt % body 
+        return self.fmt % body
 
     fmt = "%19s %4s %4s %4s %3s %1s %7s %4s %4s %s"
 
@@ -140,7 +142,7 @@ class Request:
 class StartupRequest(Request):
     def endstage(self):
         return "U"
-        
+
     def total(self):
         return 0
 
@@ -150,12 +152,12 @@ class Cumulative:
         self.times = []
         self.hangs = 0
         self.allelapsed = None
-        
+
     def put(self, request):
         elapsed = request.elapsed
         if elapsed == "I": self.hangs = self.hangs + 1
         self.times.append(elapsed)
-        
+
     def all(self):
         if self.allelapsed == None:
             self.allelapsed = []
@@ -174,18 +176,18 @@ class Cumulative:
     def getheader(self):
         return self.fmt % ('Hangs', 'Hits', 'Total', 'Max', 'Min', 'Median',
                            'Mean', 'URL')
-        
+
     fmt = "%5s %5s %5s %5s %5s %6s %5s %s"
 
     def hits(self):
         return len(self.times)
-        
+
     def max(self):
         return max(self.all())
-        
+
     def min(self):
         return min(self.all())
-        
+
     def mean(self):
         l = len(self.times)
         if l == 0:
@@ -194,7 +196,7 @@ class Cumulative:
             t = self.total()
             if t == "I": return "I"
             return t/l
-        
+
     def median(self):
         all = self.all()
         l = len(all)
@@ -213,7 +215,7 @@ class Cumulative:
                 v2 = all[i2]
                 if type(v1) is type('') or type(v2) is type(''): return "I"
                 else: return (v1 + v2) / 2
-    
+
     def total(self):
         t = 0
         all = self.all()
@@ -333,7 +335,7 @@ def analyze(requests, top, sortf, start=None, end=None, mode='cumulative',
         requests = cumulative.values()
         requests.sort(sortf)
         write(requests, top)
-        
+
     elif mode=='timed':
         computed_start = requests[0].start
         computed_end = requests[-1].t_end
@@ -349,7 +351,7 @@ def analyze(requests, top, sortf, start=None, end=None, mode='cumulative',
     elif mode == 'urlfocus':
         requests.sort(sortf)
         urlfocuswrite(requests, urlfocusurl, urlfocustime)
-    
+
     else:
         requests.sort(sortf)
         write(requests, top)
@@ -470,7 +472,7 @@ def timewrite(requests, start, end, resolution):
         num = d.get(slice, 0)
         if num>max_requests: max_requests = num
         hits = hits + num
-        
+
         if avg_requests is None:
             avg_requests = num
         else:
@@ -478,24 +480,24 @@ def timewrite(requests, start, end, resolution):
 
         s = tick2str(slice)
         s = s + "     %6d         %4.2lf" % (num,num*1.0/resolution)
-        print s 
+        print s
 
-    print '='*78 
+    print '='*78
     print " Peak:                  %6d         %4.2lf" % \
         (max_requests,max_requests*1.0/resolution)
     print "  Avg:                  %6d         %4.2lf" % \
         (avg_requests,avg_requests*1.0/resolution)
     print "Total:                  %6d          n/a " % (hits)
-    
+
 def tick2str(t):
     return time.strftime('%Y-%m-%dT%H:%M:%S', time.localtime(t))
-    
+
 def codesort(v1, v2):
     v1 = v1.endstage()
     v2 = v2.endstage()
     if v1 == v2:
         return 0
-    
+
     if v1 == "B":
         return -1 # v1 is smaller than v2
     if v1 == "I":
@@ -525,7 +527,7 @@ class Sort:
         else:
             if self.ascending: return 1
             else: return -1
-            
+
 def detailedusage():
     details = usage(0)
     pname = sys.argv[0]
@@ -596,8 +598,10 @@ If the 'verbose' argument is specified, do not trim url to fit into 80 cols.
 
 If the 'today' argument is specified, limit results to hits received today.
 
+If the 'daysago' argument is specified, limit results to hits received n days ago.
+
 The 'resolution' argument is used only for timed reports and specifies the
-number of seconds between consecutive lines in the report 
+number of seconds between consecutive lines in the report
 (default is 60 seconds).
 
 The 'urlfocustime' argument is used only for urlfocus reports and specifies the
@@ -634,12 +638,17 @@ Examples:
     Show cumulative report statistics sorted by mean for entries in the log
     which happened today, and do not trim the URL in the resulting report.
 
+  %(pname)s debug.log --cumulative --sort=mean --daysago=3 --verbose
+
+    Show cumulative report statistics sorted by mean for entries in the log
+    which happened three days ago, and do not trim the URL in the resulting report.
+
   %(pname)s debug.log --urlfocus='/manage_main' --urlfocustime=60
 
     Show 'urlfocus' report which displays statistics about requests
     surrounding the invocation of '/manage_main'.  Focus on the time periods
     60 seconds before and after each invocation of the '/manage_main' URL.
-    
+
   %(pname)s debug.log --detailed --start='2001/05/10 06:00:00'
     --end='2001/05/11 23:00:00'
 
@@ -677,12 +686,12 @@ Usage: %s filename1 [filename2 ...]
           [--sort=spec]
           [--top=n]
           [--verbose]
-          [--today | [--start=date] [--end=date] ]
+          [--today | [--start=date] [--end=date] | --daysago=n ]
           [--writestats=filename | --readstats=filename]
           [--urlfocus=url]
           [--urlfocustime=seconds]
           [--help]
-        
+
 Provides a profile of one or more Zope "-M" request log files.
 """ % sys.argv[0]
         )
@@ -710,12 +719,15 @@ if __name__ == '__main__':
     statsfname = None
     readstats = 0
     writestats = 0
-    
+
     files = []
     i = 1
     for arg in sys.argv[1:]:
         if arg[:2] != '--':
-            files.append(open(arg))
+            if arg[-3:] == '.gz' and globals().has_key('gzip'):
+                files.append(gzip.GzipFile(arg,'r'))
+            else:
+                files.append(open(arg))
             sys.argv.remove(arg)
             i = i + 1
 
@@ -723,10 +735,11 @@ if __name__ == '__main__':
         opts, extra = getopt.getopt(
             sys.argv[1:], '', ['sort=', 'top=', 'help', 'verbose', 'today',
                                'cumulative', 'detailed', 'timed','start=',
-                               'end=','resolution=', 'writestats=',
+                               'end=','resolution=', 'writestats=','daysago=',
                                'readstats=','urlfocus=','urlfocustime=']
             )
         for opt, val in opts:
+
             if opt=='--readstats':
                 statsfname = val
                 readstats = 1
@@ -749,7 +762,15 @@ if __name__ == '__main__':
                 end = list(now)
                 end[3] = 23; end[4] = 59; end[5] = 59
                 end = time.mktime(end)
-
+            if opt=='--daysago':
+                now = time.localtime(time.time() - int(val)*3600*24 )
+                # for testing - now = (2001, 04, 19, 0, 0, 0, 0, 0, -1)
+                start = list(now)
+                start[3] = start[4] = start[5] = 0
+                start = time.mktime(start)
+                end = list(now)
+                end[3] = 23; end[4] = 59; end[5] = 59
+                end = time.mktime(end)
             if opt=='--start':
                 start = getdate(val)
             if opt=='--end':
