@@ -2,20 +2,20 @@
 ##############################################################################
 #
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
-# 
+#
 ##############################################################################
 """Zope user bootstrap system"""
 
-__version__='$Revision: 1.13 $ '[11:-2]
+__version__='$Revision: 1.17.6.2 $ '[11:-2]
 
-import sys, string, sha, binascii, whrandom, getopt, getpass, os
+import sys,  sha, binascii, random, getopt, getpass, os
 
 try:
     from crypt import crypt
@@ -27,10 +27,10 @@ def generate_salt():
     salt_choices = ("ABCDEFGHIJKLMNOPQRSTUVWXYZ"
                     "abcdefghijklmnopqrstuvwxyz"
                     "0123456789./")
-    return whrandom.choice(salt_choices)+whrandom.choice(salt_choices)
+    return random.choice(salt_choices)+random.choice(salt_choices)
 
 def generate_passwd(password, encoding):
-    encoding=string.upper(encoding)
+    encoding=encoding.upper()
     if encoding == 'SHA':
         pw = '{SHA}' + binascii.b2a_base64(sha.new(password).digest())[:-1]
     elif encoding == 'CRYPT':
@@ -41,19 +41,18 @@ def generate_passwd(password, encoding):
     return pw
 
 def write_generated_password(home, ac_path, username):
-    import whrandom
     pw_choices = ("ABCDEFGHIJKLMNOPQRSTUVWXYZ"
                   "abcdefghijklmnopqrstuvwxyz"
                   "0123456789!")
     acfile=open(ac_path, 'w')
     pw = ''
     for i in range(8):
-        pw = pw + whrandom.choice(pw_choices)
+        pw = pw + random.choice(pw_choices)
     acfile.write('%s:%s' % (username, generate_passwd(pw, 'SHA')))
     acfile.close()
     os.system('chmod 644 %s' % ac_path)
     return pw
-    
+
 def write_access(home, user='', group=''):
     ac_path=os.path.join(home, 'access')
     if not os.path.exists(ac_path):
@@ -112,17 +111,17 @@ for all necessary information.  The available options are:
     -d / --domains=
     Set the domain names that the user user can log in from.  Defaults to
     any. OPTIONAL.
-    
+
     Filename is required and should be the name of the file to store the
     information in (usually "inituser" or "access").
-    
+
 Copyright (C) 1999, 2000 Digital Creations, Inc.
 """ % argv[0]
 
     try:
         if len(argv) < 2:
             raise "CommandLineError"
-        
+
         optlist, args = getopt.getopt(sys.argv[1:], short_options, long_options)
 
         if len(args) != 1:
@@ -135,7 +134,7 @@ Copyright (C) 1999, 2000 Digital Creations, Inc.
             username = ''
             encoding = 'SHA'
             domains = ''
-        
+
             for opt in optlist:
                 if (opt[0] == '-u') or (opt[0] == '--username'):
                     username = opt[1]
@@ -160,7 +159,7 @@ Copyright (C) 1999, 2000 Digital Creations, Inc.
                 username = raw_input("Username: ")
                 if username != '':
                     break
-               
+
             while 1:
                 password = getpass.getpass("Password: ")
                 verify = getpass.getpass("Verify password: ")
@@ -174,12 +173,15 @@ Copyright (C) 1999, 2000 Digital Creations, Inc.
                 print """
 Please choose a format from:
 
-SHA - SHA-1 hashed password
+SHA - SHA-1 hashed password (default)
 CRYPT - UNIX-style crypt password
-CLEARTEXT - no protection.
+CLEARTEXT - no protection
 """
                 encoding = raw_input("Encoding: ")
-                if encoding != '':
+                if encoding == '':
+                    encoding = 'SHA'
+                    break
+                if encoding.upper() in ['SHA', 'CRYPT', 'CLEARTEXT']:
                     break
 
             domains = raw_input("Domain restrictions: ")
@@ -188,12 +190,11 @@ CLEARTEXT - no protection.
             access_file.write(username + ":" +
                               generate_passwd(password, encoding) +
                               domains)
-            
+
     except "CommandLineError":
         sys.stderr.write(usage)
         sys.exit(1)
 
-    
+
 # If called from the command line
 if __name__=='__main__': main(sys.argv)
-
